#!/usr/bin/python3
#  coding: utf-8
#
#       coupling_OF_fistr.py
#
#           指定したpatchの
#               faceCenter座標
#               faceを構成するnodeNo（順番）
#               point座標
#           を取得し、OpenFOAMとFrontISTRを連成させる。
#
#   23/01/26    新規作成
#      03/17    getSavedFolders:毎回連成する時、facePressデータが削除され
#               latestTime以外の再計算が実現できなかった事を修正。
#      03/18    getNextCouplingTime:nStepOF=2,writeIntervalの次のstepで
#               hangupする事がある為、修正。
#

import sys, os
import glob
import shutil
import subprocess
import time
import mmap
import multiprocessing
import math
import createPointCloudFromOpenFoam as createPtc
import pyTreeFoam_ptc as pyTreeFoam
import pyFistr_ptc as pyFistr
import mappingFromPointCloud as mapping

OF_pid = ""             #OpenFOAMのpid
waitCount = 5000        #最長で50sまで待つ
waitTime = 0.01
delTimes = []           #不要な時間のlist
stepTimeDict = {}       #writeTimeとstepの対応list

commonSize = 10000


#
#  putProcStream
#----------------
def putProcStream(procNo, title, text):
    """ 指定したprocNoのstreamに文字をセットする"""
    size = commonSize
    myPid = os.getpid()
    header = "PID:" + str(myPid) + " title:" + title
    cont = header + "\01" + text + "\04"
    cont = cont.encode()
    dataList = []
    for i in range(0, len(cont), size):
        data = cont[i:i+size]
        dataList.append(data)
    #textをstreamにセットする
    for i in range(len(dataList)):
        if i == 0:
            waitUntilBufferInitialSet(procNo)
        else:
            waitUntilBufferEmpty(procNo)
        data = dataList[i]
        mmCommFile = mmCommFiles[int(procNo)]
        mmCommFile.seek(0)
        mmCommFile.write(data)
        mmCommFile.flush()
    mmCommFile.seek(commonSize-1)
    mmCommFile.write(b"\04")        #最後に書き込み
    mmCommFile.flush()
    waitUntilBufferEmpty(procNo)    #相手が読むまで待つ

#
#  getProcSteam
#---------------
def getProcStream(procNo, title):
    """ 指定したprocNoのstraemからtitleと文字列を取り出す"""
    waitUntilBufferSet(procNo, title)
    mmCommFile = mmCommFiles[int(procNo)]
    #buffer読み込み
    mmCommFile.seek(0)
    conts = mmCommFile.read().decode()
    #bufferクリア
    mmCommFile.seek(0)
    mmCommFile.write(b"\00\00")
    mmCommFile.seek(commonSize-1)
    mmCommFile.write(b"\00")
    mmCommFile.flush()
    #streamのheaderを取得
    ns = conts.find("\01")
    header = conts[:ns]
    #header内のtitleを取得
    nt = header.find("title:")
    title = header[nt + len("title:"):]
    #streamのtextを取得
    text = ""
    cont = conts[ns+1:]
    ne = cont.find("\04")
    if ne >= 0:
        text = cont[:ne]
        return title, text
    text += cont
    while True:
        waitUntilBufferSet(procNo, title)
        #読み込み
        mmCommFile.seek(0)
        cont = mmCommFile.read().decode()
        #bufferクリア
        mmCommFile.seek(0)
        mmCommFile.write(b"\00\00")
        mmCommFile.seek(commonSize-1)
        mmCommFile.write(b"\00")
        mmCommFile.flush()
        #読み込み内容を処理
        ne = cont.find("\04")
        if ne >= 0:
            text += cont[:ne]
            break
        text += cont
    return title, text

#
#  getProcStreamLines
#---------------------
def getProcStreamLines(procNo, title):
    """ 指定したprocNoのstraemからlinesを取り出す"""
    _title, text = getProcStream(procNo, title)
    if text[-1] == "\n":
        text = text[:-1]
    lines = text.split("\n")
    lines = list(map(lambda x: x+"\n", lines))
    return lines

#
#  waitUntilBufferInitialSet
def waitUntilBufferInitialSet(procNo):
    """ bufferがemptyの場合、直ぐにPIDをセットし、bufferを使用中にする"""
    mmCommFile = mmCommFiles[int(procNo)]
    count = waitCount
    sleepTime = waitTime
    bPid = str(os.getpid()).encode()
    while True:
        mmCommFile.flush()
        if mmCommFile[:2] == b"\00\00" and mmCommFile[-1:] == b"\00":
            mmCommFile.write(bPid)
            mmCommFile.flush()
            mmCommFile.seek(0)
            break
        count -= -1
        if count < 0:
            break
        time.sleep(sleepTime)
    return

#
#  waitUntilBufferEmpty
def waitUntilBufferEmpty(procNo):
    """ 送信可能になるまで待つ。bufferが空になるまで"""
    mmCommFile = mmCommFiles[int(procNo)]
    count = waitCount
    sleepTime = waitTime
    while True:
        if mmCommFile[:2] == b"\00\00" and mmCommFile[-1:] == b"\00":
            break
        count -= -1
        if count < 0:
            break
        time.sleep(sleepTime)
    return

#
#  waitUntilBufferSet
def waitUntilBufferSet(procNo, title):
    """ streamに文字がセットされるまで待つ"""
    mmCommFile = mmCommFiles[int(procNo)]
    count = waitCount
    sleepTime = waitTime
    while True:
        mmCommFile.flush()
        #最初と最後がb"\00"以外？
        if mmCommFile[:2] != b"\00\00" and mmCommFile[-1:] != b"\00":
            break
        count -= 1
        if count < 0:
            break
        time.sleep(sleepTime)
    return  

#-------------------
#  clearDataFolder
#-------------------
def clearDataFolder():
    """ dataFolder内をクリアする。"""
    #couplingのdatafolder内をクリア
    coupDir = fluidCaseDir + "/coupling_FrontISTR"
    files = glob.glob(coupDir + "/data/*")
    for file in files:
        name = os.path.basename(file)
        if name != "times":
            os.remove(file)
    #FrontISTRのlogファイルを初期化
    name = fluidCaseDir + "/solidSolve.log"
    f = open(name, "w"); f.close()

#
#  getFaceCenterLocs
#--------------------
def getFaceCenterLocs(faceCenterDict):
    """ faceCenterDictからfaceCenterLocsを取得して返す"""
    faceNos = list(faceCenterDict.keys())
    faceNos.sort()
    faceCenterLocs = []
    for faceNo in faceNos:
        loc = faceCenterDict[faceNo]
        strLoc = list(map(str, loc))
        faceCenterLocs.append(strLoc)
    return faceCenterLocs
    
#
#  getDictsFromFaceCenterDict
#-----------------------------
def getDictsFromFaceCenterDict(faceCenterDict, allFaceDict, allNodeDict):
    """ mapPointFaceDictとnodeOrderDictを作成。"""
    mapPointFaceDict = {}
    nodeOrderDict = {}
    order = 0
    for faceNo in faceCenterDict.keys():
        nodeNos = allFaceDict[faceNo]["nodes"]
        for nodeNo in nodeNos:
            if not nodeNo in nodeOrderDict.values():
                loc = allNodeDict[nodeNo]
                mapPointFaceDict[nodeNo] = [loc, nodeNos]
                nodeOrderDict[order] = nodeNo
                order += 1
    return mapPointFaceDict, nodeOrderDict

#
#  saveFaceNodes
#----------------
def saveFaceNodes(faceCenterLocs, mapPointFaceDict, nodeOrderDict):
    """ couplingで使用するmesh内容を保存する"""
    #faceCenterLocを保存
    saveDir = fluidCaseDir + "/coupling_FrontISTR/data/"
    name = patchName + "_faceCenterLocs"
    fileName = saveDir + name
    lines = []
    for strLoc in faceCenterLocs:
        line = " ".join(strLoc) + "\n"
        lines.append(line)
    f = open(fileName, "w"); f.writelines(lines); f.close()
    #mapPointFaceDictを保存
    name = patchName + "_mapPointFaceNodes"
    fileName = saveDir + name
    lines = []
    nodeOrders = list(nodeOrderDict.keys())
    nodeOrders.sort()
    num = len(mapPointFaceDict.keys())
    for orderNo in nodeOrders:
        nodeNo = nodeOrderDict[orderNo]
        loc = mapPointFaceDict[nodeNo][0]
        strLoc = list(map(str, loc))
        faceNodeNos = mapPointFaceDict[nodeNo][1]
        strNodeNos = list(map(str, faceNodeNos))
        line = " ".join([str(nodeNo)] + strLoc + strNodeNos) + "\n"
        lines.append(line)
    f = open(fileName, "w"); f.writelines(lines); f.close()

#
#  saveFaceNodesPara
#--------------------
def saveFaceNodesPara(faceCenterLocs, mapPointFaceDict, nodeOrderDict, pNo):
    """ couplingで使用するmesh内容を保存する"""
    #faceCenterLocを保存
    saveDir = fluidCaseDir + "/coupling_FrontISTR/data/"
    name = patchName + "_faceCenterLocs_" + pNo
    fileName = saveDir + name
    lines = []
    for strLoc in faceCenterLocs:
        line = " ".join(strLoc) + "\n"
        lines.append(line)
    f = open(fileName, "w"); f.writelines(lines); f.close()

    #mapPointFaceDictを保存
    name = patchName + "_mapPointFaceNodes_" + pNo
    fileName = saveDir + name
    lines = []
    nodeOrders = list(nodeOrderDict.keys())
    nodeOrders.sort()
    num = len(mapPointFaceDict.keys())
    for orderNo in nodeOrders:
        nodeNo = nodeOrderDict[orderNo]
        loc = mapPointFaceDict[nodeNo][0]
        strLoc = list(map(str, loc))
        faceNodeNos = mapPointFaceDict[nodeNo][1]
        strNodeNos = list(map(str, faceNodeNos))
        line = " ".join([str(nodeNo)] + strLoc + strNodeNos) + "\n"
        lines.append(line)
    f = open(fileName, "w"); f.writelines(lines); f.close()
    return

#------------------
#  createPatchCont
#------------------
def createPatchCont():
    """ OpenFOAM側のfaceCenterLocs（face中心座標）などmesh情報を取得"""
    if nProcs == "1":
        #singleコアの場合
        fluidCase = pyTreeFoam.case(fluidCaseDir)
        time = fluidCase.getCurrTimeFolder()
        region= "."
        pMesh = fluidCase.getCurrMeshDir(time, region, "boundary")
        meshDir = fluidCaseDir + os.sep + pMesh.split(os.sep)[0]
        polyMesh = createPtc.ofMesh(fluidCaseDir, meshDir)
        faceCenterDict = polyMesh.getFaceCenterDict([patchName])
        #faceCenter座標を取得
        faceCenterLocs = getFaceCenterLocs(faceCenterDict)
        #mapPointFaceDict, nodeOrderDictを作成
        allFaceDict = polyMesh.faceDict()
        allNodeDict = polyMesh.nodeDict()
        mapPointFaceDict, nodeOrderDict = getDictsFromFaceCenterDict(
            faceCenterDict, allFaceDict, allNodeDict)
        #取得したmesh情報を保存
        saveFaceNodesPara(faceCenterLocs, mapPointFaceDict, nodeOrderDict, "0")
    else:
        #parallelの場合
        fluidParaCase = pyTreeFoam.paraCase(fluidCaseDir)
        time = fluidParaCase.getCurrTimeFolder()
        region= "."
        pMesh = fluidParaCase.getCurrMeshDir(time, region, "boundary")
        
        for p in range(int(nProcs)):
            pNo = str(p)
            meshDir = fluidCaseDir + "/processor" + pNo + "/" + pMesh.split(os.sep)[0]
            polyMesh = createPtc.ofMesh(fluidCaseDir + "/processor" + pNo, meshDir)
            faceCenterDict = polyMesh.getFaceCenterDict([patchName])
            #faceCenter座標を取得
            faceCenterLocs = getFaceCenterLocs(faceCenterDict)
            #mapPointFaceDict, nodeOrderDictを作成
            allFaceDict = polyMesh.faceDict()
            allNodeDict = polyMesh.nodeDict()
            mapPointFaceDict, nodeOrderDict = getDictsFromFaceCenterDict(
                faceCenterDict, allFaceDict, allNodeDict)
            saveFaceNodesPara(faceCenterLocs, mapPointFaceDict, nodeOrderDict, pNo)

#
#  getNeedNodesFromDict
#-----------------------
def getNeedNodesFromDict(sgrpDict, nodeDict):
    """ 必要なnodeを抽出したnodeDictを取得して返す。"""
    newNodeDict = {}
    for key in sgrpDict.keys():
        nodeNos = sgrpDict[key][1]
        for nodeNo in nodeNos:
            if not nodeNo in newNodeDict.keys():
                newNodeDict[nodeNo] = nodeDict[nodeNo]
    return newNodeDict

#
#  saveFistrFaceData
#--------------------
def saveFistrFaceData(sgrpDict, nodeDict):
    """ FrontISTR側のfaceDataを保存する"""
    #sgrpDictを保存
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    name = sgrpName + "_mapFaceCenterFaceNodes"
    fileName = dataDir + "/" + name
    lines = []
    for key in sgrpDict.keys():
        elmNo, faceNo = key
        loc = sgrpDict[key][0]
        nodeNos = sgrpDict[key][1]
        nums = [elmNo, faceNo] + loc + nodeNos
        line = " ".join(map(str, nums)) + "\n"
        lines.append(line)
    f = open(fileName, "w"); f.writelines(lines); f.close()
    #nodeDictを保存
    name = sgrpName + "_nodeLocs"
    fileName = dataDir + "/" + name
    lines = []
    for nodeNo in nodeDict.keys():
        loc = nodeDict[nodeNo]
        nums = [nodeNo] + loc
        line = " ".join(map(str, nums)) + "\n"
        lines.append(line)
    f = open(fileName, "w"); f.writelines(lines); f.close()

#-----------------------
#  createSgrpCont
#-----------------------
def createSgrpCont():
    """ FrontISTR側のSGRPの情報を取得して保存する"""
    mesh = mapping.fistrMesh(solidCaseDir)
    meshHeaderData = mesh.getMeshHeaderData()
    nodeDict = mesh.nodeDict(meshHeaderData)
    elementDict = mesh.elementDict(meshHeaderData)
    sgrpData = mesh.getSgrpData(sgrpName, meshHeaderData)
    sgrpDict = mesh.getSgrpDict(sgrpData, nodeDict, elementDict)
    newNodeDict = getNeedNodesFromDict(sgrpDict, nodeDict)
    #Dictを保存
    saveFistrFaceData(sgrpDict, newNodeDict)
    return (sgrpDict, newNodeDict)

#
#  clearRestartSetting
#----------------------
def clearRestartSetting():
    """ FistrMode.cntファイルのresartの設定をクリアする"""
    fileName = solidCaseDir + "/FistrModel.cnt"
    f = open(fileName); lines = f.readlines(); f.close()
    #restartの設定
    i = pyFistr.getNextHeaderName(lines, 0, "!RESTART", "")
    lines[i] = pyFistr.setValName(lines[i], "FREQUENCY=1")
    #nStepsをクリア
    i = pyFistr.getNextHeaderName(lines, 0, "!DYNAMIC", "")
    i += 2
    if i >= len(lines):
        print("error: it is not DYNAMIC analysis. must set at DYNMIC.")
        print()
        return
    words = pyFistr.deleteSp(lines[i]).split(",")
    #words[2] = nSteps * nSubSteps
    words[2] = nSteps
    lines[i] = ", ".join(words) + "\n"
    #保存
    f = open(fileName, "w"); f.writelines(lines); f.close()
    #結果fileを全て削除
    delFiles = glob.glob(solidCaseDir + "/*.pvtu")
    delFiles += glob.glob(solidCaseDir + "/FistrModel.res.*")
    delFiles += glob.glob(solidCaseDir + "/*.particles")
    delFiles += glob.glob(solidCaseDir + "/*.vtk")
    delFiles += glob.glob(solidCaseDir + patchName + "_faceCenterPress*.particles")
    for delFile in delFiles:
        os.remove(delFile)
    delFolders = glob.glob(solidCaseDir + "/FistrModel.vis_psf.*")
    for delFolder in delFolders:
        if os.path.isdir(delFolder) == True:
            shutil.rmtree(delFolder)

#
#  setInitialSettingFistr
#-------------------------
def setInitialSettingFistr():
    """ Fistr側の初期設定"""
    #cntファイル読み込み
    fileName = solidCaseDir + "/FistrModel.cnt"
    f = open(fileName); lines = f.readlines(); f.close()
    #step時間を取得
    dT = float(deltaT)
    couplingInterval = dT * int(nStepsOF)
    stepTime = str(couplingInterval / int(nSteps))
    #step数とstep時間を設定（時間増分）
    i = pyFistr.getNextHeaderName(lines, 0, "!DYNAMIC", "")
    i += 2
    words = pyFistr.deleteSp(lines[i]).split(",")
    words[2] = nSteps
    words[3] = stepTime
    lines[i] = ", ".join(words) + "\n"
    #結果の書き込み頻度を設定
    fr = str(int(nSteps) * nSubSteps)
    i = pyFistr.getNextHeaderName(lines, 0, "!WRITE", "")
    lines[i] = pyFistr.setValName(lines[i], "FREQUENCY="+fr)
    i = pyFistr.getNextHeaderName(lines, i+1, "!WRITE", "")
    lines[i] = pyFistr.setValName(lines[i], "FREQUENCY="+fr)
    #cntファイル書き込み
    fw = open(fileName, "w"); lines = fw.writelines(lines); fw.close()

#----------------
#  runCoupling
#----------------
def runCoupling(sgrpDict, nodeDict):
    """ FrontISTRの起動"""
    global startStep, endStep, oldTime, beforeTime, currTime
    global mappingTriDict, deltaT, procOF
    #openFOAMの起動
    procOF = runOpenFOAM()
    mappingTriDict = {}
    #Fistrを起動する
    beforeCouplingT = float(deltaT) * int(nStepsOF)
    iniFlag = 0
    loop = True
    while loop:
        oldTime = beforeTime
        beforeCurrTime = currTime
        beforeTime, currTime, statOF = getNextCouplingTime(procOF, iniFlag)     #OF停止もチェック
        #openFOAMの生死確認
        if statOF == "halt":
            break
        #deltaTが変化している場合、cntファイルの時間増分を書き換え
        couplingT = float(currTime) - float(beforeCurrTime)
        if float(beforeCouplingT) != float(couplingT) and iniFlag != 0:
            setDeltaTInCnt(couplingT)
        beforeCouplingT = couplingT
        #圧力を取得する
        print("pressure value is mapping for FrontISTR...", flush=True)
        ptcFiles = []
        #圧力をFistr側にmappingする
        mappingPtcFile(ptcFiles, sgrpDict, nodeDict, iniFlag)
        iniFlag = 1
        print("FrontISTR is running from step " + str(startStep) +
                " to " + str(endStep) + " time " + currTime + 
                "...", flush=True)
        #Fistrを起動
        runFistr()
        print("displacement PTC is creating...")
        #変位のptcファイルを作成
        createDispPtcFile(nodeDict)
        #結果ファイルを削除
        clearPressPtcResFile(beforeTime, currTime)
        #次のstep数を取得し、設定
        startStep = endStep
        endStep = setNewEndStepRestart()
    #if int(nStepsOF) > 1:
    #    clearPressPtcResFile(beforeTime, currTime)
    mmTimes.flush()
    mmTimes.close()
    mmCoupling.flush()
    mmCoupling.close()

#
#  getNextCouplingTime
#-----------------------
def getNextCouplingTime(procOF, iniFlag):
    """ 次のbeforeTime, currTimeを取得する"""
    global delTimes
    statOF = "run"
    skipSteps = int(nStepsOF)
    #iniFlag = 1
    #counterをリセットして、更新内容を取得
    count = waitCount
    sleepTime = waitTime
    xTime = beforeTime
    bTime = beforeTime; cTime = currTime
    #iniFlag = 0
    while True:
        #時間を読み込み
        mmTimes.seek(0)
        conts = mmTimes.read().decode()
        lines = conts.split("\n")
        if len(lines) >= 3:
            bTime = lines[0].split()[1]
            cTime = lines[1].split()[1]
            #coupling = lines[2].split()[1]
            #timeが更新？
            if float(xTime) != float(bTime):
                xTime = bTime
                delTimes.append(bTime)
                #skipStepsを設定
                if isWriteInterval(cTime) == True:
                    #writeIntervalの時
                    skipSteps = 0
                else:
                    #通常の場合
                    skipSteps -= 1
                #skipStepsを保存
                lines[2] = "skipSteps  " + str(skipSteps)
                conts = "\n".join(lines) + "\n"
                mmCoupling.seek(0)
                mmCoupling.write(conts.encode())
                mmCoupling.flush()
                #coupling?
                if int(nStepsOF) == 1:
                    #nStepOF=1は、毎回cpupling
                    break
                elif skipSteps <= 0 and iniFlag != 0:
                    #couplingのtiming
                    break
                iniFlag = 1
        #counter以内？
        count -= 1
        if count < 0:
            break
        #OFの生死を確認
        if procOF.poll() != None:
            statOF = "halt"
            break
        #待ち
        time.sleep(sleepTime)
    return bTime, cTime, statOF

#
#  isWriteInterval
def isWriteInterval(bTime):
    """ bTimeが、writeIntervalかどうか確認する"""
    curr = float(bTime)
    wInt = float(writeInterval)
    diff = (curr / wInt) - float(int(curr / wInt + 0.50))
    diff3 = float(int(diff * 1000 + 0.5)) / 1000.0
    if abs(diff3) < 1e-3:
        return True
    else:
        return False

#
#  checkPressdata
def checkPressdata(ptcData):
    """ 圧力の初期値は、全てzeroではエラーが出るので修正する。計算の初回のみ"""
    for i in range(len(ptcData)):
        if ptcData[i][1] == 0.0:
            ptcData[i][1] = 1.0e-37     #zeroに近い値を設定
    return ptcData

#
#  getStreamEachProcs
#--------------------
def getStreamEachProcs(procNo, q):
    """ 並列処理用のプロセス。並列数分起動される。
    自身のstreamから内容を取り出す。"""
    lines = getProcStreamLines(procNo, "pressurePtcData")
    q.put(lines)

#
#  mappingPtcFile
#-----------------
def mappingPtcFile(ptcFiles, sgrpDict, nodeDict, iniFlag):
    """ 散らばっている圧力ptcをまとめ、Fistr側にmappingする。"""
    global mappingTriDict
    #stream内にある圧力ptcデータを取得
    #  並列数分の読み込みprocessを起動する
    queue = []
    procs = []
    for i in range(int(nProcs)):
        q = multiprocessing.Queue()
        queue.append(q)
        p = multiprocessing.Process(
            target=getStreamEachProcs, args=(str(i), q))
        p.start()
        procs.append(p)
    #  全steamの内容を取得
    lines = []
    for i in range(int(nProcs)):
        lines += queue[i].get()
    #fileに保存
    name = patchName + "_faceCenterPress.particles"
    allPtcFile = solidCaseDir + "/" + name
    f = open(allPtcFile, "w"); f.writelines(lines); f.close()
    #ptcData(数値data）を取得
    mesh = mapping.fistrMesh(solidCaseDir)
    ptcData = mesh.getPtcDataFromLines(lines)
    if len(ptcData) < 3:
        return "error could not get pressure from point cloud."
    #初回計算？
    if iniFlag == 0:
        #圧力の値をチェック
        ptcData = checkPressdata(ptcData)
        #mapping用の3点を取得
        mapStat, mappingTriDict = mesh.getMappingTriLocs(ptcData, sgrpDict, nodeDict)
        if mapStat != "OK":
            return "error in paressure mapping."
    #圧力をmapping
    mappingDict = mesh.setMappingDataFromTriLocs(ptcData, sgrpDict, mappingTriDict)
    #mapping結果を保存
    lines = mesh.createMappingFile(mappingDict, sgrpName, allPtcFile)
    name = "mappingSP_" + sgrpName + ".map"
    fileName = solidCaseDir + os.sep + name
    f = open(fileName, "w"); f.writelines(lines); f.close()
    #mapping結果をcntファイルに反映
    remakeFistrCntFile(lines)

#
#  remakeFistrCntFile
#----------------------
def remakeFistrCntFile(mapLines):
    """ mapping結果をcntファイルに反映"""
    #追加行を作成
    coeff = float(mapCoeff)
    addLines = []
    for line in mapLines:
        if line[0] != "#":
            words = line.split()
            if len(words) >= 3:
                words[2] = str(float(words[2]) * coeff)
                words[1] = "P" + words[1]
                newLine = ", ".join(words) + "\n"
                addLines.append(newLine)
    #cntFileを修正
    cntFile = solidCaseDir + os.sep + "FistrModel.cnt"
    f = open(cntFile); lines = f.readlines(); f.close()
    #挿入場所は、最初の!DLOADの該当SGRPに挿入する。
    i = 0
    while i < len(lines):
        i = pyFistr.getNextHeaderName(lines, i, "!DLOAD", "")
        i += 1
        if i >= len(lines):
            mess = "error: '" + sgrpName + "' does not exist in !DLOAD.\n"
            print(mess)
        words = pyFistr.deleteSp(lines[i]).split(",")
        if words[0] == sgrpName:
            si = i - 1
            ei = pyFistr.skipNextHeader(lines, i)
            break
    if si < len(lines):
        #cntFile修正、書き込み
        name = "mappingSP_" + sgrpName + ".map"
        grpid = pyFistr.getValName(lines[si], "GRPID")
        words = ["!DLOAD", "GRPID="+grpid, "forceType=mapping", "mapFile="+name,
                 "mapCoeff="+mapCoeff]
        lines[si] = ", ".join(words) + "\n"
        lines[si+1] = sgrpName + ", S, 0.0\n"
        newLines = lines[:si+2] + addLines + lines[ei:]
        f = open(cntFile, "w"); f.writelines(newLines); f.close()

#
#  runOpenFOAM
#---------------
def runOpenFOAM():
    """ openFOAMの起動"""
    global OF_pid, timePrec
    contDict = pyTreeFoam.case().getControlDict()
    timePrec = contDict["timePrecision"]
    solver = contDict["application"]
    if int(nProcs) > 1:
        #parallel設定
        comm = "mpirun -np " + nProcs + " " + solver + " -parallel 2>&1 | teeCorrect.py solve.log"
    else:
        #singleコア
        comm = solver + " 2>&1 | teeCorrect.py solve.log"
    #OpenFOAMを起動
    procOF = subprocess.Popen(
        comm,
        shell=True)
    OF_pid = procOF.pid     #OpenFOAMのpidを取得
    return procOF

#
#  runFistr
#-----------
def runFistr():
    """ FrontISTRを起動"""
    saveDir = os.getcwd()
    os.chdir(solidCaseDir)
    logFile = fluidCaseDir + "/solidSolve.log"
    fo = open(logFile, "ab")
    t = time.time()
    comm = solidAnaComm + " -t " + nThreads
    procFistr = subprocess.run(
        comm,
        shell=True,
        stdout=fo,
        stderr=fo
        )
    l = time.time() - t
    l = int(l * 100 + 0.5) / 100
    line = "FrontISTR total time ----> " + str(l) + " sec\n\n"
    fo.write(line.encode())
    fo.close()
    os.chdir(saveDir)

#
#  putStreamEachProcs
#---------------------
def putStreamEachProcs(procNo, title, conts, q):
    """ 並列処理用のプロセス。並列数分起動される。
    自身のstreamにcontsを出力する。"""
    putProcStream(procNo, title, conts)
    q.put("")

#
#  createDispPtcFile
#--------------------
def createDispPtcFile(nodeDict):
    """ 変位の点群ファイル作成"""
    #変位の点群データをresFileから取得、作成
    locDisp = createDispPtc(nodeDict)
    #disp点群ファイル作成
    lines = []
    for loc, disp in locDisp:
        data = loc + disp
        line = " ".join(map(str, data)) + "\n"
        lines.append(line)
    lines.append("#end\n")
    #各streamにdataセット
    conts = "".join(lines)
    title = "displacement"
    #並列数分のprocessを起動する
    queue = []
    procs = []
    for i in range(int(nProcs)):
        q = multiprocessing.Queue()
        queue.append(q)
        p = multiprocessing.Process(
            target=putStreamEachProcs,
            args=(str(i), title, conts, q))
        p.start()
        procs.append(p)
    #全セットprocessの終了を待つ
    for i in range(int(nProcs)):
        a = queue[i].get()

#
#  readDispFromResFile
def readDispFromResFile(resFile):
    """ 変位データをresFileから取得"""
    #resFileを読み込み
    f = open(resFile);
    #*dataまでskip
    while True:
        line = f.readline()
        if line[:len("*data")] == "*data":
            break
    #node数読み込み
    line = f.readline()
    nNodes = int(line.split()[0])
    #節点の値の位置を取得
    _dummy = f.readline()
    line = f.readline()
    words = line.split()
    nums = list(map(int, words))
    nVals = sum(nums)
    dispLoc = 0
    flag = 0
    for i in range(len(words)):
        valName = f.readline().split()[0]
        if valName == "DISPLACEMENT":
            flag = 1
        else:
            if flag == 0:
                dispLoc += nums[i]
    #DISPLACEMENTの値を読み込む
    dispDict = {}
    n = 0
    while True:
        #nodeNo取得
        line = f.readline()
        nodeNo = int(line.split()[0])
        #disp = [nodeNo]
        #disp読み込み
        vals = []
        while True:
            line = f.readline()
            words = line.split()
            vals += words
            if len(vals) >= nVals:
                break
        dispWords = vals[dispLoc:dispLoc+3]
        dispVal = list(map(float, dispWords))
        dispDict[nodeNo] = dispVal
        n += 1
        if n >= nNodes:
            break
    f.close()
    return dispDict

#
#  createDispPtc
def createDispPtc(nodeDict):
    """ 変位の点群データ取得"""
    #計算したstepを取得
    step = endStep
    #resFile名を取得、読み込み
    resFile = solidCaseDir + "/FistrModel.res.0." + str(step)
    dispDict = readDispFromResFile(resFile)
    #点群作成
    dispPtc = []
    for nodeNo in nodeDict.keys():
        disp = dispDict[nodeNo]
        loc = nodeDict[nodeNo]
        dispPtc.append([loc, disp])
    return dispPtc

#
#  clearPressPtcResFile
#-----------------------
def clearPressPtcResFile(beforeTime, currtime):
    """ 圧力の点群ファイル、resファイルを削除"""
    global delTimes, stepTimeDict
    delFiles = []
    delFolders = []
    #起動時のstartStepを記録
    if len(stepTimeDict) == 0:
        stepTimeDict[startStep] = beforeTime
    #OpenFOAM側のファイル削除
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data/"
    dels = delTimes.copy()
    #保存するfolderと修正名を取得
    saveFolder, corrFolder = getSavedFolders(dels)
    for i in range(len(dels)):
        delTime = dels[i]
        if float(delTime) < float(beforeTime):
            if delTime != saveFolder:
                #削除可能な時間（writeInterval以外）-------
                #圧力の点群ファイル
                pattern = solidCaseDir + "/" + patchName + "_faceCenterPress_" + delTime + "_*.particles"
                fileNames = getAllProcsFiles(pattern, int(nProcs))                
                delFiles += fileNames
                #dispを削除
                pattern = dataDir + patchName + "_disp_" + delTime + "_*"
                fileNames = getAllProcsFiles(pattern, int(nProcs))
                delFiles += fileNames
                #dispPtcを削除
                pattern = dataDir + sgrpName + "_nodeDisp_" + delTime + "_*.particles"
                fileNames = getAllProcsFiles(pattern, int(nProcs))
                delFiles += fileNames
                #facePressを削除
                pattern = dataDir + patchName + "_facePress_" + delTime + "_*"
                fileNames = getAllProcsFiles(pattern, int(nProcs))
                delFiles += fileNames
            else:
                #writeInterval時の場合------------
                #圧力の点群ファイル
                pattern = solidCaseDir + "/" + patchName + "_faceCenterPress_" + delTime + "_*.particles"
                fileNames = getAllProcsFiles(pattern, int(nProcs))                
                delFiles += fileNames
                #dispを削除
                pattern = dataDir + patchName + "_disp_" + delTime + "_*"
                fileNames = getAllProcsFiles(pattern, int(nProcs))
                delFiles += fileNames
                #dispPtcを削除
                pattern = dataDir + sgrpName + "_nodeDisp_" + delTime + "_*.particles"
                fileNames = getAllProcsFiles(pattern, int(nProcs))
                delFiles += fileNames
            #listの項目を削除
            delTimes.remove(delTime)
    #restartFileの設定
    cTime = ""
    if isWriteInterval(currTime) == True:
        if not(currTime in stepTimeDict.values()):
            cTime = currTime
            corrFolder = currTime
    elif saveFolder != "":
        if not(saveFolder in stepTimeDict.values()):
            cTime = saveFolder
    if cTime != "" and float(cTime) != float(startTime):
        #restartFileを保存(各procs毎に保存)
        restartFile = solidCaseDir + "/FistrModel.restart.0"
        newFile = solidCaseDir + "/FistrModel.restart_" + corrFolder + "_" + str(endStep) + ".0"
        shutil.copy(restartFile, newFile)
        #timeのstepを保存
        stepTimeDict[endStep] = cTime
    #resファイル,フォルダを取得
    resFiles = [solidCaseDir + "/FistrModel.res.0." + str(startStep)]
    vtuFiles = [solidCaseDir + "/FistrModel.vis_psf." + ("%04d" % startStep) + ".pvtu"]
    vtuFolders = [solidCaseDir + "/FistrModel.vis_psf." + ("%04d" % startStep)]
    #forntIstr側の削除
    for resFile in resFiles:
        step = int(resFile.split(".")[-1])
        if (step >= startStep and step < endStep):
            if not (step in stepTimeDict.keys()):
                delFiles.append(resFile)            #削除
    for i in range(len(vtuFiles)):
        step = int(vtuFiles[i].split(".")[-2])
        if (step >= startStep and step < endStep):
            if not (step in stepTimeDict.keys()):
                delFiles.append(vtuFiles[i])        #削除
                delFolders.append(vtuFolders[i])    #削除
    #file、folderを削除
    for delFile in delFiles:
        if os.path.exists(delFile) == True:
            os.remove(delFile)
    for delFolder in delFolders:
        if os.path.isdir(delFolder) == True:
            shutil.rmtree(delFolder)

#
#  getSavedFolders
def getSavedFolders(dels):
    """ dels内から保存するfolder名と修正するfolder名を取得"""
    #delsが1個以下の場合戻る
    if len(dels) <= 1:
        return "", ""
    #開始時？
    if dels[0] == startTime:
        return "", ""
    #nStepsOF=1の場合？
    if len(dels) == 2:
        #nStepsOF=1（毎回連成する場合）
        if isWriteInterval(dels[0]) == True:
            savedFolder = dels[0]
            corrFolder = dels[0]
            return savedFolder, corrFolder
    else:
        #nStepsOF>1の場合
        if isWriteInterval(dels[1]) == True:
            #連成時が保存folderの場合
            savedFolder = dels[1]
            corrFolder = dels[1]
            return savedFolder, corrFolder
    #保存folderを取得する
    savedFolder = ""; corrFolder = ""
    flag = 0
    for i in range(1, len(dels)):
        #途中（連成以外）が保存folderの場合
        name = dels[i]
        if isWriteInterval(name) == True:
            corrFolder = name   #修正名を取得
            flag = 1
    if flag == 1:
        savedFolder = dels[1]   #保存するfolder（連成した時）
        return savedFolder, corrFolder
    return "", ""

#
#  getAllProcsFiles
def getAllProcsFiles(pattern, np):
    """ 全proc数のfile名を取得して返す"""
    names = pattern.split("*")
    files = []
    for i in range(np):
        fileName = names[0] + str(i) + names[1]
        files.append(fileName)
    return files

#
#  setDeltaTInCnt
#-----------------
def setDeltaTInCnt(couplingInterval):
    """ deltaTのみをcntファイルにセット。毎回deltaTが変化する場合"""
    cntFile = solidCaseDir + "/FistrModel.cnt"
    f = open(cntFile); lines = f.readlines(); f.close()
    i = pyFistr.getNextHeaderName(lines, 0, "!DYNAMIC", "")
    i += 2
    words = pyFistr.deleteSp(lines[i]).split(",")
    #時間増分を算出
    num = couplingInterval / int(nSteps)
    #桁を丸める
    stepTime = roundVal(num, int(timePrec)+2)
    #時間増分セット
    words[3] = str(stepTime)
    lines[i] = ", ".join(words) + "\n"
    #cntFile保存
    f = open(cntFile, "w"); f.writelines(lines); f.close()

#
#  roundVal
def roundVal(x, k):
    """ xを有効数字k桁に丸める"""
    x = round(x, k - math.floor(math.log10(abs(x)))- 1)
    return x

#
#  setNewEndStepRestart
#------------------------
def setNewEndStepRestart():
    """ cntファイルに新しいstep数を設定、restartの設定"""
    #endStepを設定
    #newEndStep = endStep + int(nSteps) * nSubSteps
    newEndStep = endStep + int(nSteps)
    cntFile = solidCaseDir + "/FistrModel.cnt"
    f = open(cntFile); lines = f.readlines(); f.close()
    i = pyFistr.getNextHeaderName(lines, 0, "!DYNAMIC", "")
    i += 2
    words = pyFistr.deleteSp(lines[i]).split(",")
    words[2] = str(newEndStep)
    #words[3] = deltaT
    lines[i] = ", ".join(words) + "\n"
    #Restartの設定
    i = pyFistr.getNextHeaderName(lines, 0, "!RESTART", "")
    lines[i] = pyFistr.setValName(lines[i], "FREQUENCY=-1")
    #cntFile保存
    f = open(cntFile, "w"); f.writelines(lines); f.close()
    return newEndStep

#----------------------
#  getEndStepOfFistr
#----------------------
def getEndStepOfFistr():
    """ FrontISTRのendStepを算出"""
    cntFile = solidCaseDir + "/FistrModel.cnt"
    f = open(cntFile); lines = f.readlines(); f.close()
    i = pyFistr.getNextHeaderName(lines, 0, "!DYNAMIC", "")
    i += 2
    words = pyFistr.deleteSp(lines[i]).split(",")
    endStep = int(words[2])
    return endStep

#
#  getSubStepsNumInFistr
#-----------------------
def getSubStepsNumInFistr():
    """ FistrModel.cntファイルからsubStep数を取得する。
    !STEPがない場合は、「1」を返す。"""
    nSubSteps = 1
    fileName = solidCaseDir + "/FistrModel.cnt"
    f = open(fileName); lines = f.readlines(); f.close()
    i = pyFistr.getNextHeaderName(lines, 0, "!STEP", "")
    if i < len(lines):
        name = pyFistr.getValName(lines[i], "SUBSTEPS")
        nSubSteps = int(name)
    return nSubSteps

#
#  setReastartFile
#------------------
def setReastartFile(startTime):
    """ FrontISTRのrestartFileを設定する"""
    global endStep
    pattern = solidCaseDir + "/FistrModel.restart_*.0"
    files = glob.glob(pattern)
    restartFile = ""
    for name in files:
        resTime = name.split("_")[-2]
        if float(resTime) == float(startTime):
            restartFile = name
            break
    if restartFile == "":
        print("error: restart file does not exist. could not restart.")
        return
    #endStepを取得
    restFile = ".".join(restartFile.split(".")[:-1])
    endStep = int(restFile.split("_")[-1])
    #restartFileを設定
    orgName = restFile + ".0"
    newName = solidCaseDir + "/FistrModel.restart.0"
    shutil.copy(orgName, newName)
    #コピーが完了するまで待つ(シェルが認識できるまで待つ)
    time.sleep(1.0)
    comm = ["#!/bin/bash\n"]
    for np in range(int(nProcs)):
        newName = solidCaseDir + "/FistrModel.restart.0"
        comm += ["FILE=" + newName + "\n" ]
        comm += ["while [ ! -e $FILE ]\n"]
        comm += ["do\n"]
        comm += ["  sleep 0.1\n"]
        comm += ["done\n"]
    exeFile = solidCaseDir + "/run"
    f = open(exeFile, "w"); f.writelines(comm); f.close()
    pyFistr.run(solidCaseDir).command("chmod a+x run")
    pyFistr.run(solidCaseDir).command(exeFile)
    #endStepをcntファイルに書き込み
    endStep += int(nSteps) 
    #初期の時間増分を設定
    stepTime = float(deltaT) * int(nStepsOF) / int(nSteps)
    cntFile = solidCaseDir + "/FistrModel.cnt"
    f = open(cntFile); lines = f.readlines(); f.close()
    i = pyFistr.getNextHeaderName(lines, 0, "!DYNAMIC", "")
    i += 2
    words = pyFistr.deleteSp(lines[i]).split(",")
    words[2] = str(endStep)     #endStepを設定
    words[3] = str(stepTime)    #時間増分を設定
    lines[i] = ", ".join(words) + "\n"
    f = open(cntFile, "w"); f.writelines(lines); f.close()

#
#  createPidFile
#-----------------
def createPidFile():
    """ signal送信の為、全プロセスのPIDを保存するfile。
    OpenFOAMが起動するPIDから順番に記録し、最後に自身のPIDを記録"""
    np = int(nProcs)
    cont = " " * (np * 10)
    pid = os.getpid()
    word = "%10s" % pid
    cont += word
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/pids"
    f = open(fileName, "w"); f.write(cont); f.close()
    f = open(fileName, "r+b"); mmPid = mmap.mmap(f.fileno(), 0); f.close()
    return mmPid

#
#  createInitialTimesFile
#------------------------
def createInitialTimesFile():
    """ 初期のtimesファイルを作成する。lengthは、200bytesで固定"""
    length = 200
    cont = " " * length
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/times"
    f = open(fileName, "w"); f.write(cont); f.close()
    f = open(fileName, "r+b"); mmTimes = mmap.mmap(f.fileno(), 0); f.close()
    return mmTimes

#
#  createCommonFiles
#-------------------
def createCommonFiles():
    """ data送受信の為の共通fileを設定"""
    mmCommFiles = []
    size = commonSize
    chara = b"\0"
    cont = chara * size
    for np in range(int(nProcs)):
        fileName = fluidCaseDir + "/coupling_FrontISTR/data/common_" + str(np)
        f = open(fileName, "wb"); f.write(cont); f.close()
        f = open(fileName, "r+b"); mmCommFile = mmap.mmap(f.fileno(), 0); f.close()
        mmCommFiles.append(mmCommFile)
    return mmCommFiles

#
#  createCouplingStepFile
#-------------------------
def createCouplingStepFile():
    """ couplingStepファイルの共有。lengthは、200bytesで固定"""
    couplingStepFile = fluidCaseDir + "/coupling_FrontISTR/data/couplingStep"
    length = 200
    cont = " " * length
    f = open(couplingStepFile, "w"); f.write(cont); f.close()
    f = open(couplingStepFile, "r+b"); mmCoupling = mmap.mmap(f.fileno(), 0); f.close()
    return mmCoupling

#------------------
#  getCouplingData
#------------------
def getCouplingData():
    """ couplingDataを取得する"""
    fileName = fluidCaseDir + "/coupling_FrontISTR/couplingData"
    #f = open(fileName); lines = f.readlines(); f.close()
    #他のプロセスでも読むので、mmapの設定
    f = open(fileName, "r+b"); mm = mmap.mmap(f.fileno(), 0); f.close()
    mm.seek(0)
    cont = mm.read().decode()
    lines = cont.split("\n")
    lines = list(map(lambda x: x+"\n", lines))
    startTime = ""; deltaT = ""; writeInterval = ""; nStepsOF = ""
    mappingPatch = ""; streamBuffer = ""; maxWaitTime = ""
    solidCaseDir = ""; mappingSgrp = ""; mappingCoeff = ""
    nThreads = ""; solidAnaComm = ""; nSteps = ""; nProcs = ""
    for line in lines:
        if line[0] != "#":
            words = line.split()
            if len(words) >= 2:
                if words[0] == "startTime":
                    startTime = words[1]
                elif words[0] == "deltaT":
                    deltaT = words[1]
                elif words[0] == "writeInterval":
                    writeInterval = words[1]
                elif words[0] == "nStepsOF":
                    nStepsOF = words[1]
                elif words[0] == "mappingPatch":
                    mappingPatch = words[1]
                elif words[0] == "streamBuffer":
                    streamBuffer = words[1]
                elif words[0] == "maxWaitTime":
                    maxWaitTime = words[1]
                elif words[0] == "solidCaseDir":
                    solidCaseDir = words[1]
                elif words[0] == "mappingSgrp":
                    mappingSgrp = words[1]
                elif words[0] == "mappingCoeff":
                    mappingCoeff = words[1]
                elif words[0] == "nThreads":
                    nThreads = words[1]
                elif words[0] == "solidAnaComm":
                    solidAnaComm = words[1]
                elif words[0] == "nSteps":
                    nSteps = words[1]
                elif words[0] == "nProcs":
                    nProcs = words[1]
    return (startTime, deltaT, writeInterval, nStepsOF,
            mappingPatch, streamBuffer, maxWaitTime, solidCaseDir,
            mappingSgrp, mappingCoeff, nThreads, solidAnaComm,
            nSteps, nProcs)


if __name__ == "__main__":
    import gettext
    gettext.install("app") # replace with the appropriate catalog name
    _ = gettext.gettext

    #初期設定
    deltaT = ""
    startTime = ""
    fluidCaseDir = os.getcwd()
    (startTime, deltaT, writeInterval, nStepsOF, patchName,
    streamBuffer, maxWaitTime, solidCaseDir, sgrpName, mapCoeff,
    nThreads, solidAnaComm, nSteps, nProcs) = getCouplingData()
    solidCaseDir = os.path.abspath(solidCaseDir)
    commonSize = int(streamBuffer)
    waitCount = int(maxWaitTime)
    skipSteps = int(nStepsOF)
    nSubSteps = getSubStepsNumInFistr()
    if float(startTime) == 0.0:
        #初期化---------------------------
        #folder内をクリア
        clearDataFolder()
        #Fistrのfolder内をクリア
        clearRestartSetting()
        #Fistrの初期設定
        setInitialSettingFistr()
    else:
        #途中から計算開始-----------
        #restartFileとendStepを設定
        setReastartFile(startTime)
    #共有メモリの設定
    mmPid = createPidFile()
    mmTimes = createInitialTimesFile()
    mmCommFiles = createCommonFiles()
    mmCoupling = createCouplingStepFile()
    #meshのpatch内容を取得(face,nodeの座標を取得)
    createPatchCont()
    #mshのSGRPの内容を取得
    (sgrpDict, nodeDict) = createSgrpCont()
    #openFOAM - FrontISTR coupling
    endStep = getEndStepOfFistr()
    startStep = endStep - int(nSteps)
    oldTime = "-2"; beforeTime = "-1"; currTime = "0"
    #couplig開始
    runCoupling(sgrpDict, nodeDict)

    


