#!/usr/bin/python3
#  coding: utf-8
#
#       setPress_getDisp.py
#
#       OpenFOAMの圧力pをFrontISTRのcntファイルにセットし、
#       変位の点群ファイルを作成する。
# 　　　変位の点群ファイルからOFのpointにmappingした結果fileを
#       作成する。（OF側がこのfileを読み込む）
#
#   23/01/24    新規作成
#


import sys
import os
import time
import glob
import mmap
import mappingFromPointCloud as mapping

waitCount = 5000
waitTime = 0.01

commonSize = 10000


#
#  putStream
#-------------
def putStream(title, text):
    """ streamにtextをセットする"""
    size = commonSize
    myPid = os.getpid()
    header = "PID:" + str(myPid) + " title:" + title
    cont = header + "\01" + text + "\04"
    cont = cont.encode()
    dataList = []
    for i in range(0, len(cont), size):
        data = cont[i:i+size]
        dataList.append(data)
    #textをstreamにセットする
    for i in range(len(dataList)):
        if i == 0:
            waitUntilBufferInitialSet()
        else:
            waitUntilBufferEmpty()
        data = dataList[i]
        mmCommFile.seek(0)
        mmCommFile.write(data)
        mmCommFile.flush()
    mmCommFile.seek(commonSize-1)
    mmCommFile.write(b"\04")    #最後に書き込み
    mmCommFile.flush()
    waitUntilBufferEmpty()      #相手が読み込むまで待つ

#
#  getStream
#------------
def getStream(title):
    """ streamの内容を取得する"""
    #dataがセットされるまで待つ
    waitUntilBufferSet(title)
    #buffer読み込み
    mmCommFile.seek(0)
    conts = mmCommFile.read().decode()
    #bufferクリア
    mmCommFile.seek(0)
    mmCommFile.write(b"\00\00")
    mmCommFile.seek(commonSize-1)
    mmCommFile.write(b"\00")
    mmCommFile.flush()
    #streamのheaderを取得
    ns = conts.find("\01")
    header = conts[:ns]
    #header内のtitleを取得
    nt = header.find("title:")
    title = header[nt + len("title:"):]
    #streamのtextを取得
    text = ""
    cont = conts[ns+1:]
    ne = cont.find("\04")
    if ne >= 0:
        text = cont[:ne]
        return title, text
    text += cont
    while True:
        waitUntilBufferSet(title)
        #読み込み
        mmCommFile.seek(0)
        cont = mmCommFile.read().decode()
        #bufferクリア
        mmCommFile.seek(0)
        mmCommFile.write(b"\00\00")
        mmCommFile.seek(commonSize-1)
        mmCommFile.write(b"\00")
        mmCommFile.flush()
        #読み込み内容を処理
        ne = cont.find("\04")
        if ne >= 0:
            text += cont[:ne]
            break
        text += cont
    return title, text

#
#  getStreamLines
#-----------------
def getStreamLines(title):
    _title, text = getStream(title)
    if text[-1] == "\n":
        text = text[:-1]
    lines = text.split("\n")
    lines = list(map(lambda x: x+"\n", lines))
    return lines

#
#  waitUntilBufferInitialSet
#----------------------------
def waitUntilBufferInitialSet():
    """ bufferがemptyの場合、直ぐにPIDをセットし、bufferを使用中にする"""
    count = waitCount
    sleepTime = waitTime
    bPid = str(os.getpid()).encode()
    while True:
        mmCommFile.flush()
        if mmCommFile[:2] == b"\00\00" and mmCommFile[-1:] == b"\00":
            mmCommFile.write(bPid)
            mmCommFile.flush()
            mmCommFile.seek(0)
            break
        count -= -1
        if count < 0:
            break
        time.sleep(sleepTime)
    return

#
#  waitUntilBufferEmpty
def waitUntilBufferEmpty():
    """ 送信可能になるまで待つ。bufferが空になるまで"""
    count = waitCount
    sleepTime = waitTime
    while True:
        if mmCommFile[:2] == b"\00\00" and mmCommFile[-1:] == b"\00":
            break
        count -= -1
        if count < 0:
            break
        time.sleep(sleepTime)
    return

#
#  waitUntilBufferSet
def waitUntilBufferSet(title):
    """ streamに文字がセットされるまで待つ"""
    count = waitCount
    sleepTime = waitTime
    while True:
        mmCommFile.flush()
        if mmCommFile[:2] != b"\00\00" and mmCommFile[-1:] != b"\00":
            break
        count -= 1
        if count < 0:
            break
        time.sleep(sleepTime)
    return  

#
#  readMmapLines
#-----------------
def readMmapLines(fileName):
    """ fileObjectの共有メモリからlinesを読み込み返す"""
    f = open(fileName, "r+b")
    mm = mmap.mmap(f.fileno(), 0)
    f.close()
    lines = []
    while True:
        line = mm.readline().decode()
        if line == "":
            break
        lines.append(line)
    mm.close()
    return lines

#
#  getMappedPointLocs
#---------------------
def getMappedPointLocs():
    """ mappingするpointの座標を取得する"""
    name = patchName + "_mapPointFaceNodes_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    mapPointFaceDict = {}
    mapPointDict = {}
    for line in lines:
        words = line.split()
        if len(words) > 4:
            nodeNo = int(words[0])
            pointLoc = list(map(float, words[1:4]))
            faceNodes = list(map(int, words[4:]))
            mapPointFaceDict[nodeNo] = [pointLoc, faceNodes]
            mapPointDict[nodeNo] = pointLoc
    return mapPointFaceDict, mapPointDict

#
# createPtcFile
#---------------
def createPtcFile(pressFile):
    """ 圧力の点群file作成"""
    global mmList
    #faceCenterLoc読み込み
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    faceCenterFile = dataDir + "/" + patchName + "_faceCenterLocs_" + procNo
    f = open(faceCenterFile); lines = f.readlines(); f.close()
    #facePressを読み込み
    fp = open(pressFile); pLines = fp.readlines(); fp.close()
    newLines = []
    for i in range(len(lines)):
        newLine = lines[i][:-1] + " " + pLines[i]
        newLines.append(newLine)
    newLines.append("#end\n")
    #streamにセットし、送信
    cont = "".join(newLines)
    title = "pressurePtcData"
    putStream(title, cont)
    #圧力のptcファイルを保存
    name = patchName + "_faceCenterPress_" + beforeTime + "_" + procNo + ".particles"
    ptcFileName = solidCaseDir + "/" + name
    f = open(ptcFileName, "w"); f.writelines(newLines); f.close()
    return ptcFileName

#
#  sendSignal
#--------------
def sendSignal(signum):
    """ signalを親に送信する"""
    i = 10 * int(nProcs)
    parPid = int(mmPid[i:i+10])     #親のPID
    os.kill(parPid, signum)         #signal送信

#
#  saveMappingDisp
#------------------
def saveMappingDisp(mappingDict):
    """ mappingDispの結果を保存。このファイルをOF側が読み込む"""
    saveDir = fluidCaseDir + "/coupling_FrontISTR/data"
    name = patchName + "_mapPointFaceNodes_" + procNo
    fileName = saveDir + "/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    #保存のfilename
    name = patchName + "_disp_" + beforeTime + "_" + procNo
    dispFile = saveDir + "/" + name
    f = open(dispFile, "w")
    for line in lines:
        words = line.split()
        nodeNo = int(words[0])
        disp = mappingDict[nodeNo]
        line = " ".join(list(map(str, disp))) + "\n"
        f.write(line)
    f.close()
    if procNo == "0":
        print("displacement values were created for OpenFOAM at " + timeName + ".")
    return dispFile

#
#  getLocDisplacement
#---------------------
def getLocDisplacement():
    """ 変位の点群ファイルStreamから読み込み"""
    #straemから読み込み
    lines = getStreamLines("displacement")
    locDisp = []
    for line in lines:
        if line[0] != "#":
            words = line.split()
            loc = list(map(float, words[:3]))
            disp = list(map(float, words[3:]))
            locDisp.append([loc, disp])
    return locDisp

#
#  isMappingTriFile
def isMappingTriFile():
    """ mapping用の3点を取得したfileが存在するかチェック"""
    name = "mappingTriFile_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    return os.path.exists(fileName)

#
#  createMappingTriFile
def createMappingTriFile(mappingTriDict):
    """ mapping用の3点の座標をfileに保存する"""
    name = "mappingTriFile_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    lines = []
    for key in mappingTriDict.keys():
        faceNo = key
        word = str(faceNo)
        triLocs = mappingTriDict[key]
        loc0, loc1, loc2 = triLocs
        word0 = " ".join(list(map(str, loc0)))
        word1 = " ".join(list(map(str, loc1)))
        word2 = " ".join(list(map(str, loc2)))
        line = " ".join([word, word0, word1, word2]) + "\n"
        lines.append(line)
    f = open(fileName, "w"); f.writelines(lines); f.close()

#
#  readMappingTriFile
def readMappingTriFile():
    """ 保存したmapping用の3点の座標を読み込む"""
    name = "mappingTriFile_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    mappingTriDict = {}
    for line in lines:
        words = line.split()
        faceNo = int(words[0])
        key = faceNo
        loc0 = list(map(float, words[1:4]))
        loc1 = list(map(float, words[4:7]))
        loc2 = list(map(float, words[7:10]))
        mappingTriDict[key] = [loc0, loc1, loc2]
    return mappingTriDict

#---------------------
#  setPress_getDisp
#---------------------
def setPress_getDisp():
    """ pressをmappingしてfistrの結果を待ち、変位を取得する"""
    mesh = mapping.fistrMesh(solidCaseDir)
    #ptcDataの読み込み
    ptcDir = fluidCaseDir + "/coupling_FrontISTR/data"
    pressFile = ptcDir + "/" + patchName + "_facePress_" + beforeTime + "_" + procNo
    #圧力の点群ファイルを作成 
    ptcFile = createPtcFile(pressFile)
    #mappingするpointの座標を予め取得しておく（待ち時間を利用）
    mapPointFaceDict, mapPointDict = getMappedPointLocs()
    #変位の点群dataをstreamから読み込み
    locDisp = getLocDisplacement()
    #mappingTriFile(mapping用の3点の座標を保存)があるか？
    if isMappingTriFile() == False:    
        #存在しない場合、mappingTriDictを取得
        mapStat, mappingTriDict = mesh.getMappingTriLocs(locDisp, mapPointFaceDict, mapPointDict)
        if mapStat != "OK":
            return "error in displacement mapping."
        #mappingTriFileを作成
        createMappingTriFile(mappingTriDict)
    else:
        #fileを読み込む
        mappingTriDict = readMappingTriFile()
    #変位をmapping
    mappingDict = mesh.setMappingDataFromTriLocs(locDisp, mapPointFaceDict, mappingTriDict)
    #mapping結果を保存(このfileをOpenFOAM側が読み込む)
    mapDispFile = saveMappingDisp(mappingDict)
    return ""

#
#  beginning
#------------
def beginning(coupling):
    """ beforeTime, currTimeをファイルに保存"""
    #processor0のみ
    if procNo == "0":
        #時間を書き込み
        lines =  ["beforeTime " + beforeTime + "\n"]
        lines += ["currTime   " + timeName + "\n"]
        #lines += ["coupling   " + coupling + "\n"]
        lines += ["coupling   yesno\n"]
        lines += ["#end\n"]
        cont = "".join(lines)
        mmTimes.seek(0)
        mmTimes.write(cont.encode())
        mmTimes.flush()
    #couplingFileに反映されるまで待つ
    waitUntilCouplingWrited(beforeTime)
    return

#
#  waitUntilCouplingWrited
def waitUntilCouplingWrited(beforeTime):
    """ couplingStepにbeforeTimeが反映されるまで待つ"""
    count = waitCount
    sleepTime = waitTime
    while True:
        mmCoupling.seek(0)
        cont = mmCoupling.read().decode()
        lines = cont.split("\n")
        if len(lines) >= 3:
            bTime = lines[0].split()[1]
            if float(bTime) == float(beforeTime):
                #beforeTimeが反映された場合戻る
                break
        count -= 1
        if count < 0:
            break
        time.sleep(sleepTime)
    return

#
#  readSetPidFile
#------------------
def readSetPidFile():
    """ signal送信の為、自身のPIDをpidファイルに記録"""
    #pidを保存
    pid = os.getpid()
    word = "%10s" % pid
    pidFile = fluidCaseDir + "/coupling_FrontISTR/data/pids"
    f = open(pidFile, "r+b"); mmPid = mmap.mmap(f.fileno(), 0); f.close()
    i = int(procNo) * 10
    mmPid[i:i+10] = word.encode()
    return mmPid

#
#  creteCommonFile
#------------------
def creteCommonFile():
    """ data送受信の為の共通fileを設定"""
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/common_" + procNo
    f = open(fileName, "r+b"); mmCommFile = mmap.mmap(f.fileno(), 0); f.close()
    return mmCommFile

#
#  createTimesFile
#-------------------
def createTimesFile():
    """ timesファイルの共有"""
    timeFile = fluidCaseDir + "/coupling_FrontISTR/data/times"
    f = open(timeFile, "r+b"); mmTimes = mmap.mmap(f.fileno(), 0); f.close()
    return mmTimes
#
#  createCouplingStepFile
#--------------------------
def createCouplingStepFile():
    """ couplingStepファイルの共有"""
    couplingStepFile = fluidCaseDir + "/coupling_FrontISTR/data/couplingStep"
    f = open(couplingStepFile, "r+b"); mmCoupling = mmap.mmap(f.fileno(), 0); f.close()
    return mmCoupling

#
#  createInitialDispFile
#------------------------
def createInitialDispFile():
    """ 初期の変位dataを作成し、セットする"""
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    name = patchName + "_mapPointFaceNodes_" + procNo
    fileName = dataDir + "/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    #保存のfileName
    name = patchName + "_disp_" + beforeTime + "_" + procNo
    dispFile = dataDir + "/" + name
    dataLine = "0.0 0.0 0.0\n"
    f = open(dispFile, "w"); 
    for i in range(len(lines)):
        f.write(dataLine)
    f.close()

#
#  createInitialPressFile
#-------------------------
def createInitialPressFile():
    """ 初期の圧力dataを作成し、セットする。"""
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    name = patchName + "_faceCenterLocs_" + procNo
    fileName = dataDir + "/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    #保存のfileName
    name = patchName + "_facePress_" + beforeTime + "_" + procNo
    pressFile = dataDir + "/" + name
    dataLine = "0.0\n"
    f = open(pressFile, "w"); 
    for i in range(len(lines)):
        f.write(dataLine)
    f.close()

#
#  deleteMappingTriFile
#-----------------------
def deleteMappingTriFile():
    """ 初回起動時に、mappingTriFileを削除する"""
    name = "mappingTriFile_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    if os.path.exists(fileName) == True:
        os.remove(fileName)

#
#  copyMapPointDispFile
#-----------------------
def copyMapPointDispFile():
    """ mappingDispのfileをrenameする"""
    name = patchName + "_disp_*_" + procNo
    pattern = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    files = glob.glob(pattern)
    fileTimes = []
    for fileName in files:
        ft = fileName.split("_")[-2]
        fileTimes.append([float(ft), ft, fileName])
    fileTimes.sort()
    oldName = patchName + "_disp_" + fileTimes[-1][1] + "_" + procNo
    oldFileName = fluidCaseDir + "/coupling_FrontISTR/data/" + oldName
    newName = patchName + "_disp_" + beforeTime + "_" + procNo
    newFileName = fluidCaseDir + "/coupling_FrontISTR/data/" + newName
    os.rename(oldFileName, newFileName)

#
#  isWriteInterval
#------------------
def isWriteInterval(bTime):
    """ bTimeがwriteIntervalかどうか確認する"""
    curr = float(bTime)
    wInt = float(writeInterval)
    diff = (curr / wInt) - float(int(curr / wInt + 0.50))
    diff3 = float(int(diff * 1000 + 0.5)) / 1000.0
    if abs(diff3) < 1e-3:
        return True
    else:
        return False

#
#  isCouplingTime
#-----------------
def isCouplingTime(beforeTime):
    """ couplingTime（連成させる時間）かどうか確認
    skipSeps=1の場合、次はcouplingになる。"""
    coupling = False
    if int(nStepsOF) == 1:
        #NSTEPSOF=1の場合は、毎回連成
        coupling = True
        return coupling
    if isWriteInterval(timeName) == True:
        #writeIntervalでは連成する
        coupling = True
        return coupling
    #procNo=0の場合
    if procNo == "0":
        mmCoupling.seek(0)
        conts = mmCoupling.read().decode()
        lines = conts.split("\n")
        if len(lines) < 3:
            #計算初回は連成
            coupling = True
            return coupling
        bTime = lines[0].split()[1]
        skipSteps = int(lines[2].split()[1])
        if float(bTime) == float(startTime) and procNo != "0":
            #計算初回は連成
            coupling = True
            return coupling
        #skipStepsとprocNoで判断する
        if skipSteps == 1 and procNo == "0":
            #procNo=0:前回のstep＝１の場合は、今回が0になるので、連成
            coupling = True
            return coupling
    #procNo=0以外
    else:
        waitUntilCouplingWrited(beforeTime)
        mmCoupling.seek(0)
        conts = mmCoupling.read().decode()
        lines = conts.split("\n")
        bTime = lines[0].split()[1]
        cTime = lines[1].split()[1]
        skipSteps = int(lines[2].split()[1])
        if float(bTime) == 0:
            #初回は、連成
            coupling = True
            return coupling
        elif float(cTime) == float(timeName):
            #couplingのcTimeがcurrTimeの場合
            if skipSteps == 0:
                coupling = True
                return coupling
        else:
            #couplingのctimeがbeforeTimeの場合
            if skipSteps == 1:
                coupling = True
                return coupling
    return coupling

#
#  getCouplingData
#------------------
def getCouplingData():
    """ couplingData（FrontISTR側のデータ）を取得する"""
    fileName = fluidCaseDir + "/coupling_FrontISTR/couplingData"
    lines = readMmapLines(fileName)
    nStepsOF = ""; startTime = ""; writeInterval = ""; streamBuffer = ""
    maxWaitTime = ""; solidCaseDir = ""; mappingSgrp = ""; mappingCoeff = ""
    solidAnaComm = ""; nSteps = ""
    for line in lines:
        if line[0] != "#":
            words = line.split()
            if len(words) >= 2:
                if words[0] == "startTime":
                    startTime = words[1]
                elif words[0] == "writeInterval":
                    writeInterval = words[1]
                elif words[0] == "nStepsOF":
                    nStepsOF = words[1]
                elif words[0] == "streamBuffer":
                    streamBuffer = words[1]
                elif words[0] == "maxWaitTime":
                    maxWaitTime = words[1]
                elif words[0] == "solidCaseDir":
                    solidCaseDir = words[1]
                elif words[0] == "mappingSgrp":
                    mappingSgrp = words[1]
                elif words[0] == "mappingCoeff":
                    mappingCoeff = words[1]
                elif words[0] == "solidAnaComm":
                    solidAnaComm = words[1]
                elif words[0] == "nSteps":
                    nSteps = words[1]
    return (startTime, writeInterval, nStepsOF, streamBuffer,
            maxWaitTime, solidCaseDir, mappingSgrp, mappingCoeff,
            solidAnaComm, nSteps)
    

if __name__ == "__main__":
    import gettext
    gettext.install("app") # replace with the appropriate catalog name
    _ = gettext.gettext

    timeName = sys.argv[1]      #currTime
    beforeTime = sys.argv[2]    #前回のtime
    patchName = sys.argv[3]     #patchName
    procNo = sys.argv[4]        #procNo
    nProcs = sys.argv[5]        #nProcs
    fluidCaseDir = os.getcwd()
    (startTime, writeInterval, nStepsOF, streamBuffer, maxWaitTime,
    solidCaseDir, sgrpName, mapCoeff, solidAnaComm, nSteps) = getCouplingData()
    waitCount = int(maxWaitTime)
    #共有メモリの設定
    commonSize = int(streamBuffer)
    mmPid = readSetPidFile()
    mmCommFile = creteCommonFile()
    mmTimes = createTimesFile()
    mmCoupling = createCouplingStepFile()
    #錬成させる時間？
    if isCouplingTime(beforeTime) == False:
        #連成させない時間の場合、前回の結果をコピーして終了。
        beginning(coupling="no")
        copyMapPointDispFile()
        exit()
    #計算開始時？（初期化の有無）
    elif (float(beforeTime) == float(startTime) 
        and float(startTime) == 0.0):
        #初回起動時の設定
        #beginning(coupling="no")
        createInitialDispFile()
        deleteMappingTriFile()
        if int(nStepsOF) > 1:
            beginning(coupling="no")
            exit()
        createInitialPressFile()
    #途中から再開した場合（latestTime）
    elif (float(beforeTime) == float(startTime)
        and int(nStepsOF) > 1):
        beginning(coupling="no")
        copyMapPointDispFile()
        exit()
    #連続計算
    beginning(coupling="yes")
    solidCaseDir = os.path.abspath(solidCaseDir)
    stat = setPress_getDisp()
    if stat != "":
        raise NameError(stat)
