#!/usr/bin/python3
# coding: utf-8
#
#   cfMeshDialog.py
#
#       cfMeshDict作成、実行
#
#   15/09/29    新規作成
#      11/11    createFmsFile:cfMesh用のcsvファイルかどうかのエラーチェック追加
#   17/02/13    runCfMesh_sub:gnome-terminalのオプションを「-t」→「--name=」に変更。
#      02/27    runOnlyCfMesh,checkMesh:  ↑
#   18/06/05    gnome-terminalのオプション「-x」を「--」に変更
#   19/10/23    GTK+3, python3用に大幅修正
#   22/02/01    getStlDir:stlDirをそのまま設定する様に修正。
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
import os, sys
import glob
import shutil

import pyTreeFoam
import getOpenFolderDDialog
import inputTextDDialog
import createCfMeshDict 
import runParaFoamOptionDialog
import universalDialogs as unvDlg

import logFileCreater
logFileCreater.log()

#------------
#  windowApp
#------------
class windowApp:

    def __init__(self, caseDir, modelDir):
        self.builder = Gtk.Builder()
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path+"cfMeshDialog.glade")
        self.mainWindow = self.builder.get_object("window1")
        self.mainWindow.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        #変数の設定
        self.caseDir = caseDir
        #GUIのobject名を取得
        self.setGtkObject()
        #mesh作成データを取得
        self.meshDataDict = self.getCreateMeshData()
        self.meshDataDict["stlDir"] = modelDir
        #mesh作成データをdialogに設定
        self.setCreateMeshData()
        #configDictを取得
        self.configDict = pyTreeFoam.readConfigTreeFoam()
        #comboBoxにデータをセット
        self.setCsvFileNameToBox()

    def setGtkObject(self):
        """ glade内のobject名を取得する"""
        self.entry_stlDir = self.builder.get_object("entry_stlDir")
        self.combo_csvFiles = self.builder.get_object("combo_csvFiles")

    def main(self):
        """ GUIの表示"""
        self.mainWindow.show()
        Gtk.main()

    def close(self, *args):
        """ 閉じる"""
        Gtk.main_quit()


    #----- event handler ---------
    #開く
    def onOpenStlFolder(self, event):
        self.openStlFolder()
    #stl場所参照
    def onGetStlDir(self, event):
        self.getStlDir()
    #stlファイルチェック
    def onStlCheck(self, event):
        self.stlCheck()
    #csvファイル作成
    def onCreateCsv(self, event):
        self.createCsv()
    #csvファイル編集
    def onEditCsv(self, event):
        self.editCsv()
    #meshDict作成
    def onCreateMeshDict(self, event):
        self.createMeshDict()
    #meshDict編集
    def onEditMeshDict(self, event):
        self.editMeshDict()
    #cfMesh実行
    def onRunCfMesh(self, event):
        self.runCfMesh()
    #boundaryの整合
    def onChangePatchName(self, event):
        self.changePatchName()
    #checkMesh実行
    def onCheckMesh(self, event):
        self.checkMesh()
    #paraFoam起動
    def onRunParaFoam(self, event):
        self.runParaFoam()
    #閉じる
    def onClose(self, event):
        self.ending()
        self.close()


    #
    #  getCreateMeshData
    def getCreateMeshData(self):
        """ メッシュ作成データを取得する"""
        dataDict = pyTreeFoam.case(self.caseDir).getCreateMeshData()
        if dataDict["surfaceAngle"] == "":
            dataDict["surfaceAngle"] = "150"
        if dataDict["clearMesh"] == "":
            dataDict["clearMesh"] = "yes"
        if dataDict["parallel"] == "":
            dataDict["parallel"] = "no"
        if dataDict["nCpu"] == "":
            dataDict["nCpu"] = "4"
        if dataDict["nSeparate"] == "":
            dataDict["nSeparate"] = "2 2 2"
        return dataDict

    #
    #  setCreateMeshData
    def setCreateMeshData(self):
        """ メッシュ作成データをdialogに表示する。
        stlDirをentryBoxに設定。"""
        stlDir = self.meshDataDict["stlDir"]
        self.entry_stlDir.set_text(stlDir)

    #
    #  getStlDirFromTextBox
    def getStlDirFromTextBox(self):
        """ textBoxからstlDirをfullPathで取得する"""
        stlDir = self.entry_stlDir.get_text()
        if stlDir[0] != os.sep:
            stlDir = self.caseDir + os.sep + stlDir
            stlDir = os.path.abspath(stlDir)
        return stlDir

    #
    #  setCsvFileNameToBox
    def setCsvFileNameToBox(self):
        """ csvファイル名をcomboBoxにセット"""
        stlDir = self.getStlDirFromTextBox()
        csvFiles = glob.glob(stlDir + os.sep + "*.csv")
        self.csvFileNames = list(map(lambda x: x.split(os.sep)[-1], csvFiles))
        self.csvFileNames.sort()
        self.combo_csvFiles.clear()
        listModel = Gtk.ListStore(str)
        for name in self.csvFileNames:
            listModel.append([name])
        cell = Gtk.CellRendererText()
        self.combo_csvFiles.pack_start(cell, True)
        self.combo_csvFiles.add_attribute(cell, "text", 0)
        self.combo_csvFiles.set_model(listModel)
        self.combo_csvFiles.set_entry_text_column(0)

    #
    #  comboSelectFile
    def comboSelectFile(self):
        """ comboBox内のself.csvFileNameを選択表示する"""
        if self.csvFileName in self.csvFileNames:
            selNo = self.csvFileNames.index(self.csvFileName)
        else:
            selNo = 0
        self.combo_csvFiles.set_active(selNo)

    #
    #  ending
    def ending(self):
        """ 終了処理"""
        stlDir = self.entry_stlDir.get_text()
        dataDict = {}
        dataDict["stlDir"] = stlDir
        pyTreeFoam.case(self.caseDir).setCreateMeshData(dataDict)

    #
    #  createDefaultMeshDict
    def createDefaultMeshDict(self):
        """ defaultのmeshDictを作成する"""
        TreeFoamPath = os.getenv("TreeFoamPath")
        cfMeshFolders = glob.glob(TreeFoamPath + "/data/cfMesh/*")
        cfMeshFolders.sort()
        latestFolder = cfMeshFolders[-1]
        copyFile = latestFolder + "/meshDict"
        pasteDir = self.caseDir + "/system"
        shutil.copy(copyFile, pasteDir)

    #
    #  runCfMesh_sub
    def runCfMesh_sub(self):
        """ meshDictを作成、cfMesh実行"""
        stlDir = self.getStlDirFromTextBox()
        #fmsファイル作成、meshDict作成
        iter = self.combo_csvFiles.get_active_iter()
        if iter == None:
            title = _("csvファイル編集")
            mess = _("csvファイルが選択されていません。\n")
            mess += _("編集するcsvファイルを選択してください。")
            self.errDialog(title, mess)
            return
        model = self.combo_csvFiles.get_model()
        csvFileName = model.get_value(iter, 0)
        error = self.createFmsFile(stlDir, csvFileName)
        if error != "":
            title = _("エラー")
            self.errDialog(title, error)
            return
        title = _("meshDict作成")
        mess = _("meshDictを作成しました")
        mess += _("\n\n引き続き、cartesianMeshを実行して、meshを作成しますか？")
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return
        #mesh作成
        caseName = caseDir.split("/")[-1]
        envOpenFOAM = self.configDict["bashrcFOAM"]
        cont  = "#!/bin/bash\n"
        cont += ". " + envOpenFOAM + "\n"
        cont += "cartesianMesh | tee solve.log\n"
        cont += os.getenv("TreeFoamPath") + "/python/meetPatchNameDeleteNullPatch.py " + self.caseDir + "\n"
        comm = "echo " + "'" + cont + "'" + " > run"
        run = pyTreeFoam.run(self.caseDir)
        run.command(comm)
        run.command("chmod a+x run")
        run.foamTerminal("./run")
        

        
        



    #
    #  createFmsFile
    def createFmsFile(self, stlDir, csvFileName):
        """ fmsファイルを作成する"""
        fileName = stlDir + "/" + csvFileName
        (otherData, data, keyword) = self.readCsvData(fileName)
        #エラーチェック
        flag = 0
        for word in keyword:
            if word == "cfMesh":
                flag = 1
                break
        if flag == 0:
            error = _(u"選択しているcsvファイルは、cfMesh用ではありません。")
            error += _(u"\n「csv編集」ボタンで、ファイル内容を確認してください。")
            return error
        #stlFileを取得
        stlFiles = []
        for line in data:
            if (line[1] == "wall" or line[1] == "patch" or line[1] == "empty" or
                line[1] == "symmetry" or line[1] == "symmetryPlane"):
                stlFiles.append(stlDir + "/" + line[0] + ".stl")
        #solid名を各ファイルに設定
        error = self.setSolidNameInStlFile(stlFiles)
        if error != "":
            return error

        print(_(u"stlファイルにsolid名を追加しました。"))
        #stlFileを結合
        mergeName = stlDir + "/assy.stl"
        if len(glob.glob(mergeName)) != 0:
            os.remove(mergeName)
        self.mergeStlFiles(stlFiles, mergeName)
        print(_(u"stlファイルを結合して、assy.stlを作成しました。"))
        #fmsファイル作成
        envOpenFoam = self.configDict["bashrcFOAM"]
        angle = otherData[2]
        surfaceFile = "model.fms"
        comm =  ". " + envOpenFoam + "; " 
        comm += "surfaceFeatureEdges -angle " + angle + " " + mergeName + " " + stlDir + "/" + surfaceFile
        print(comm)
        (stat, _res, _err) = pyTreeFoam.run(self.caseDir).commandReturnCont(comm)
        if stat != "OK":
            error = _("特徴線が抽出出来ませんでした。")
            error += _("\nlogを確認してください。")
            return error
        print(_(u"assy.stlから特徴線を抽出し、model.fmsファイルを作成しました。"))
        #meshDic作成
        createCfMeshDict.createMeshDict(self.caseDir, stlDir, surfaceFile, otherData, data)
        return ""

    #
    #  mergeStlFiles
    def mergeStlFiles(self, stlNames, mergeName):
        """ stlファイルを結合"""
        fa = open(mergeName, "a")
        for stlName in stlNames:
            readName = stlName
            f=open(readName); cont=f.read(); f.close()
            if cont[-1] != "\n":
                cont += "\n"
            fa.write(cont)
        fa.close()





    
    #
    #  readCsvData
    def readCsvData(self, fileName):
        """ csvデータの読み込み"""
        if os.path.isfile(fileName) == False:
            return ([], [], [])

        conts = pyTreeFoam.readCsvData(fileName)
        #dataの取り込み
        otherData = []; data = []; keyword = []
        readStl = 0
        maxCellSize = ""; minCellSize = ""; featureAngle = ""
        for row in conts:
            flag = 0
            for val in row:
                if val != "":
                    flag = 1
                    break
            if flag == 1:
                if row[1] == "maxCellSize":
                    maxCellSize = row[2]
                elif row[1] == "minCellSize":
                    minCellSize = row[2]
                elif row[1] == "featureAngle":
                    featureAngle = row[2]
                elif row[0] == "stlFile":
                    readStl = 1
                else:
                    if readStl == 1:
                        data.append(row)
                    elif len(row[0]) > 1:
                        if row[0][0] == "<":
                            word = row[0][1:].split(">")
                            if word[0] != "":
                                keyword.append(word[0])
        otherData = [maxCellSize, minCellSize, featureAngle]
        return (otherData, data, keyword)

    #
    #  setSolidNameInStlFile
    def setSolidNameInStlFile(self, stlFiles):
        """ 各stlファイルにsolid名をセット"""
        error = ""
        flag = 0
        for stlName in stlFiles:
            #flag = 0
            fileName = stlName
            solidName = stlName.split("/")[-1][:-4]
            f=open(fileName); lines=f.readlines(); f.close()
            words = lines[0].split()
            oldName = ""
            if len(words)>1:
                oldName = words[1]
            lines[0] = "solid " + solidName + "\n"
            i = len(lines) - 1
            while i> 0:
                words = lines[i].split()
                if words[0] == "endsolid":
                    endName = ""
                    if len(words) > 1:
                        endName = words[1]
                    break
                i -= 1
            if oldName == endName:
                #solid名をセット
                lines[0] = "solid " + solidName + "\n"
                lines[i] = "endsolid " + solidName + "\n"
                f=open(fileName, "w")
                for line in lines:
                    f.write(line)
                f.close()
            else:
                #solid名セットできず
                flag += 1
        if flag != 0:
            error = str(flag) + _(u" ヶのファイルでsolid名がセットできていません。")
            error += _("\nstlファイルの書式を確認してください。")
        return error





    #
    #  openStlFolder
    #  -------------
    def openStlFolder(self):
        """ stlDirを開く"""
        stlDir = self.getStlDirFromTextBox()
        if len(glob.glob(stlDir)) == 0:
            mess = _("設定したfolderは存在しません。\n場所を再設定してください。")
            self.errDialog(_("エラー"), mess)
            return
        fileManager = pyTreeFoam.fileManager
        comm = fileManager + " " + stlDir + " &"
        pyTreeFoam.run(self.caseDir).command(comm)

    #
    #  getStlDir
    #  ---------
    def getStlDir(self):
        """ stlDirを取得する"""
        print(_("stlDirを取得する"))
        # stlDir = self.getStlDirFromTextBox()
        # if len(glob.glob(stlDir)) == 1:
        #     selDir = stlDir
        # else:
        #     selDir = self.caseDir
        selDir = self.entry_stlDir.get_text()
        title = _("開く")
        cont = _("folderを選択する")
        rootDir = self.configDict["rootDir"]
        currDir = self.caseDir
        dialog = getOpenFolderDDialog.dialogApp(title, cont, rootDir, currDir, selDir)
        openDir = dialog.showDialog()
        if openDir != "":
            self.entry_stlDir.set_text(openDir)

    #
    #  stlCheck
    #  --------
    def stlCheck(self):
        """ stlファイルを編集する。"""
        print(_("stlファイルを編集する"))
        stlDir = self.getStlDirFromTextBox()
        path = os.getenv("TreeFoamPath") + os.sep + "python"
        comm = "./editStlFilesDialog.py " + self.caseDir + " " + stlDir + " &"
        pyTreeFoam.run(path).command(comm)

    #
    #  createCsv
    #  ---------
    def createCsv(self):
        """ meshDict作成用のcsvファイルを作成する"""
        #file名の取得
        title = _("ファイル名の入力")
        message = _("csvファイル名を入力してください。")
        inputText = "cfMeshDict.csv"
        dialog = inputTextDDialog.dialogApp(title, message, inputText)
        self.csvFileName = dialog.show()
        if self.csvFileName == "":
            return
        #csvファイル作成
        stlDir = self.getStlDirFromTextBox()
        createCfMeshDict.createCsvFile(caseDir, stlDir, self.csvFileName)
        title = _("csvファイル作成")
        mess = _("cfMeshDict作成用のcsvファイルをstlDir内に作成しました。\n")
        mess += _("office等でcsvファイルを編集し、cellSize等を設定してください。")
        self.okDialog(title, mess)
        self.setCsvFileNameToBox()
        self.comboSelectFile()
        return

    #
    #  editCsv
    #  -------
    def editCsv(self):
        """ csvファイルを編集する"""
        stlDir = self.getStlDirFromTextBox()
        #csvFileを取得
        iter = self.combo_csvFiles.get_active_iter()
        if iter == None:
            title = _("csvファイル編集")
            mess = _("csvファイルが選択されていません。\n")
            mess += _("編集するcsvファイルを選択してください。")
            self.errDialog(title, mess)
            return
        model = self.combo_csvFiles.get_model()
        self.csvFileName = model.get_value(iter, 0)
        fileName = stlDir + os.sep + self.csvFileName
        comm = pyTreeFoam.Office + " " + fileName + " &"
        pyTreeFoam.run(stlDir).command(comm)

    #
    #  createMeshDict
    #  --------------
    def createMeshDict(self):
        """ meshDict作成とcfMesh実行"""
        meshDict = caseDir + "/system/meshDict"
        mess = ""
        title = _("cfMesh実行")
        if len(glob.glob(meshDict)) == 0:
            self.createDefaultMeshDict()
            mess += _("「meshDict」ファイルがsystemフォルダ内に存在しない為、defaultのmeshDictを作成しました\n\n")
        #実行の確認
        mess += _("csvファイルから、cfMesh用のmeshDictを作り出します。")
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return
        #mesh作成
        self.runCfMesh_sub()

    #
    #  editMeshDict
    #  ------------
    def editMeshDict(self):
        """ meshDictの編集"""
        editFiles = [self.caseDir + "/system/meshDict"]
        pyTreeFoam.run(self.caseDir).editor(editFiles)

    #
    #  runCfMesh
    #  ---------
    def runCfMesh(self):
        """ cfMesh実行し、メッシュを作成する"""
        envOpenFoam = self.configDict["bashrcFOAM"]
        cont  = "#!/bin/bash\n"
        cont += ". " + envOpenFoam + "\n"
        cont += "cartesianMesh | tee solve.log\n"
        comm = "echo " + "'" + cont + "'" + " > run"
        run = pyTreeFoam.run(self.caseDir)
        run.command(comm)
        run.command("chmod a+x run")
        run.foamTerminal("./run")

    #
    #  changePatchName
    #  ---------------
    def changePatchName(self):
        """ patch名の修正"""
        title = _("boundary整合")
        mess = _("boundaryと各boundaryFieldの整合をとります。")
        mess += _("\n(boundaryの整合と空patchの削除)")
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return
        comm = os.getenv("TreeFoamPath") + "/python/meetPatchNameDeleteNullPatch.py " + self.caseDir + "\n"
        pyTreeFoam.run(self.caseDir).command(comm)

    #
    #  checkMesh
    #  ---------
    def checkMesh(self):
        """ checkMeshの実行"""
        envOpenFoam = self.configDict["bashrcFOAM"]
        cont  = "#!/bin/bash\n"
        cont += ". " + envOpenFoam + "\n"
        cont += "checkMesh | tee solve.log\n"
        comm = "echo " + "'" + cont + "'" + " > run"
        run = pyTreeFoam.run(self.caseDir)
        run.command(comm)
        run.command("chmod a+x run")
        run.foamTerminal("./run")

    #
    #  runParaFoam
    #  -----------
    def runParaFoam(self):
        """ paraFoamの実行"""
        (isRegion, _loc, _regNames) = pyTreeFoam.case(self.caseDir).isMultiRegion()
        (stat, option) = runParaFoamOptionDialog.Show(isRegion)
        if stat == "OK":
            #出力をlogFileにもteeで出力させる
            paraFoam = self.configDict["paraFoam"]
            comm = paraFoam + " " + option + " &"
            pyTreeFoam.run(self.caseDir).commandWithLog(comm)



    def okDialog(self, title, mess):
        unvDlg.okDialog(self, title, mess)
    def errDialog(self, title, mess):
        unvDlg.errDialog(self, title, mess)
    def okCancelDialog(self, title, mess):
        return unvDlg.okCancelDialog(self, title, mess)

if __name__ == "__main__":
    import gettext
    import locale
    localeDir = os.getenv("TreeFoamPath") + "/data/locale"
    locale.bindtextdomain("treefoam", localeDir)
    gettext.install("treefoam", localeDir)
    #_ = gettext.gettext

    caseDir = sys.argv[1]
    modelDir = sys.argv[2]
    winApp = windowApp(caseDir, modelDir)
    winApp.main()

