#!/usr/bin/python3
# coding: utf-8
#
#       changeConfigTreeFoamDialog.py
#
#       configTreeFoamの修正用dialog
#
#   12/05.20    新規作成
#   12/09/12    OpenFOAMのバージョンセットを追加
#   13/04/30    バグ修正（helyxOSの変更ができていなかった。）
#      10/09    言語の設定を追加
#      12/21    logFile項目追加
#               stdout、stderrの設定（import logFileCreater）を追加
#      12/22    バグ修正（未設定項目が書き込めない）
#   14/07/12    dialogをframeに変更（TreeFoamが「segmentation fault」発生する為）
#   15/07/17    dialog起動方法を修正
#      07/23    dialogを作り直し
#               　fileの編集とappフォルダ内容をcomboBoxの候補に設定
#      07/25    setSelectDataToComboBoxes:バグ修正
#      07/26    ボタンを追加（setDefaultContents）
#   16/06/02    onSet:OFversionの書き込み追加。
#   17/09/10    setOFversion:bashrcの設定をbashFileを作成して取得する様に修正
#   19/09/09    python3, GTK+3用に書き換え
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
import os
import sys
import glob
import shutil

try:
    import pyTreeFoam
except:
    import python.pyTreeFoam as pyTreeFoam

configDict = {}

#
#  Show
#  ----
def Show(cDict):
    """ configTreeFoamの修正dialogをdialogを表示して、その結果を辞書で戻す"""
    global configDict
    configDict = cDict
    dialog = dialogApp()
    newConfigDict = dialog.main()
    return newConfigDict


#
#  dialogApp class
#
class dialogApp:
    """ configTreeFoamの設定変更の為のdialog"""
    def __init__(self):
        self.builder = Gtk.Builder()
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path + "changeConfigTreeFoamDialog.glade")
        self.dialogApp = self.builder.get_object("dialog1")
        self.dialogApp.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        self.setGtkObject()

    #
    #  main
    #
    def main(self):
        """ dialogを表示、閉じた後はnewConfigDictを返す"""
        self.initialize()
        self.dialogApp.run()
        return self.newConfigDict

    def close(self, *args):
        """ windowを閉じた時"""
        self.newConfigDict = {}
        self.dialogApp.destroy()

    def setGtkObject(self):
        """ GUIのidを取得"""
        self.cmbLang = self.builder.get_object("cmbLanguage")
        self.cmbLog = self.builder.get_object("cmbLogFile")
        self.cmbBashrc = self.builder.get_object("cmbBashrcFOAM")
        self.cmbParaFoam = self.builder.get_object("cmbRunParaFoam")
        self.cmbSalome = self.builder.get_object("cmbRunSalomeMeca")
        self.cmbCad = self.builder.get_object("cmbRunCad")
        self.cmbEditor = self.builder.get_object("cmbRunEditor")

    def onApply(self, event):
        """ 設定ボタンのクリック時"""
        self.newConfigDict = self.getAllItems()
        pyTreeFoam.writeConfigTreeFoam(self.newConfigDict)
        #OpenFOAMのversionを記録
        bashrc = self.newConfigDict["bashrcFOAM"]
        cont = "#!/bin/bash\n"
        cont += ". " + bashrc + "\n"
        cont += os.getenv("TreeFoamPath") + os.sep + "bin" + os.sep + "getOFversionWriteConfig.py\n"
        fileName = os.getenv("TreeFoamUserPath") + "/temp/runBash"
        f = open(fileName, "w"); f.write(cont); f.close()
        pyTreeFoam.run().command("chmod a+x " + fileName)
        pyTreeFoam.run().commandWithLog(fileName)
        if self.newConfigDict["language"] != configDict["language"]:
            title = _("警告")
            mess  = _("使用言語が変更されました。") + "\n"
            mess += _("変更内容は、次回TreeFoam起動時に適用されます。")
        else:
            title = _("configTreeFoamの設定")
            mess = _("設定内容を設定しました")
        self.okDialog(title, mess)
        self.dialogApp.destroy()

    def onCancel(self, event):
        """ キャンセルボタンのクリック時"""
        self.newConfigDict = {}
        self.dialogApp.destroy()

    #------ event handler ---------------------
    def onEditBashrcFOAM(self, event):
        self.editBashrcFOAM()
    def onEditRunParaFoam(self, event):
        self.editRunParaFoam()
    def onEditRunSalomeMeca(self, event):
        self.editRunSalomeMeca()
    def onEditRunCad(self, event):
        self.editRunCad()
    def onEditRunEditor(self, event):
        self.editRunEditor()
    def onEditConfigTreeFoam(self, event):
        self.editConfigTreeFoam()
    def onOpenAppFolder(self, event):
        self.openAppFolder()
    def onReloadFolder(self, event):
        self.reloadFolder()
    def onSetDefault(self, event):
        self.setDefault()
    # -------------------------------------

    def initialize(self):
        """ 初期化"""
        self.appDir = os.getenv("TreeFoamUserPath") + os.sep + "app"
        self.loadAppDirSetComboBox()

    def loadAppDirSetComboBox(self):
        """ configTreefoamの内容をcomboBoxに設定する"""
        files = glob.glob(self.appDir+os.sep+"*")
        #itemsを設定
        langs = ["English", "Japanese"]
        logs = ["yes", "no"]
        bashrcFiles = []
        runParaFoamFiles = []
        runSalomeFiles = []
        runCadFiles = []
        runEditorFiles = []
        otherFiles = []
        addPath = "$TreeFoamUserPath" + os.sep + "app" + os.sep
        for fileName in files:
            name = fileName.split(os.sep)[-1]
            if name[:len("bashrc-FOAM")] == "bashrc-FOAM":
                bashrcFiles.append(addPath+name)
            elif name[:len("runParaFoam")] == "runParaFoam":
                runParaFoamFiles.append(addPath+name)
            elif name[:len("runSalomeMeca")] == "runSalomeMeca":
                runSalomeFiles.append(addPath+name)
            elif name[:len("runCAD")] == "runCAD":
                runCadFiles.append(addPath+name)
            elif name[:len("runEditor")] == "runEditor":
                runEditorFiles.append(addPath+name)
            else:
                otherFiles.append(addPath+name)
        bashrcFiles.sort()
        runParaFoamFiles.sort()
        runSalomeFiles.sort()
        runCadFiles.sort()
        runEditorFiles.sort()
        #comboBoxにdataをセット
        self.cmbLang = self.setDataInComboBox(self.cmbLang, langs)
        self.cmbLog = self.setDataInComboBox(self.cmbLog, logs)
        self.cmbBashrc = self.setDataInComboBox(self.cmbBashrc, bashrcFiles)
        self.cmbParaFoam = self.setDataInComboBox(self.cmbParaFoam, runParaFoamFiles)
        self.cmbSalome = self.setDataInComboBox(self.cmbSalome, runSalomeFiles)
        self.cmbCad = self.setDataInComboBox(self.cmbCad, runCadFiles)
        self.cmbEditor = self.setDataInComboBox(self.cmbEditor, runEditorFiles)
        #comboBox_entryにデータをセット
        self.cmbLang = self.setComboEntry(self.cmbLang, configDict["language"], langs)
        self.cmbLog = self.setComboEntry(self.cmbLog, configDict["logFile"], logs)
        self.cmbBashrc = self.setComboEntry(self.cmbBashrc, configDict["bashrcFOAM"], bashrcFiles)
        self.cmbParaFoam = self.setComboEntry(self.cmbParaFoam, configDict["paraFoam"], runParaFoamFiles)
        self.cmbSalome = self.setComboEntry(self.cmbSalome, configDict["salomeMeca"], runSalomeFiles)
        self.cmbCad = self.setComboEntry(self.cmbCad, configDict["CAD"], runCadFiles)
        self.cmbEditor = self.setComboEntry(self.cmbEditor, configDict["editor"], runEditorFiles)

    def setDataInComboBox(self, comboBox, setNames):
        """ comboBoxにsetNamesを設定する"""
        comboBox.clear()
        listModel = Gtk.ListStore(str)
        for name in setNames:
            listModel.append([name])
        comboBox.set_model(listModel)
        cell = Gtk.CellRendererText()
        comboBox.pack_start(cell, True)
        comboBox.add_attribute(cell, "text", 0)
        comboBox.set_entry_text_column(0)
        return comboBox

    def setComboEntry(self, comboBox, setName, setNames):
        """ comboBoxのentryにデータをセットする"""
        if setName in setNames:
            selNo = setNames.index(setName)
            comboBox.set_active(selNo)
        else:
            entry = comboBox.get_child()
            entry.set_text(setName)
        return comboBox

    def getAllItems(self):
        """ 全comboBoxの設定内容を辞書で取得"""
        newDict = {}
        newDict["language"] = self.getTextFromComboBox(self.cmbLang)
        newDict["logFile"] = self.getTextFromComboBox(self.cmbLog)
        newDict["bashrcFOAM"] = self.getTextFromComboBox(self.cmbBashrc)
        newDict["paraFoam"] = self.getTextFromComboBox(self.cmbParaFoam)
        newDict["salomeMeca"] = self.getTextFromComboBox(self.cmbSalome)
        newDict["CAD"] = self.getTextFromComboBox(self.cmbCad)
        newDict["editor"] = self.getTextFromComboBox(self.cmbEditor)
        return newDict

    def editBashrcFOAM(self):
        fileName = self.getTextFromComboBox(self.cmbBashrc)
        self.openEditor(fileName)

    def editRunParaFoam(self):
        fileName = self.getTextFromComboBox(self.cmbParaFoam)
        self.openEditor(fileName)

    def editRunSalomeMeca(self):
        fileName = self.getTextFromComboBox(self.cmbSalome)
        self.openEditor(fileName)

    def editRunCad(self):
        fileName = self.getTextFromComboBox(self.cmbCad)
        self.openEditor(fileName)

    def editRunEditor(self):
        fileName = self.getTextFromComboBox(self.cmbEditor)
        self.openEditor(fileName)

    def editConfigTreeFoam(self):
        fileName = os.getenv("TreeFoamUserPath") + os.sep + "configTreeFoam"
        self.openEditor(fileName)

    def openAppFolder(self):
        pyTreeFoam.run(self.appDir).fileManager()

    def reloadFolder(self):
        self.loadAppDirSetComboBox()

    def setDefault(self):
        title = _("configTreeFoamの設定")
        mess = _("configTreeFoamの内容をデフォルトの状態に戻します\n")
        mess += _("  よろしいですか？")
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return
        print(_("configTreeFoamの内容をデフォルトの状態に戻します\n"))
        copyDir = os.getenv("TreeFoamPath") + os.sep + "app"
        pasteDir = os.getenv("TreeFoamUserPath") + os.sep + "app"
        copyFiles = glob.glob(copyDir+os.sep+"*")
        for copyFile in copyFiles:
            shutil.copy(copyFile, pasteDir)
        copyFile = os.getenv("TreeFoamPath") + os.sep + "configTreeFoam"
        pasteDir = os.getenv("TreeFoamUserPath")
        shutil.copy(copyFile, pasteDir)
        configDict = pyTreeFoam.readConfigTreeFoam()
        self.initialize()
        self.newConfigDict = {}
        self.newConfigDict["language"] = configDict["language"]
        self.newConfigDict["logFile"] = configDict["logFile"]
        self.newConfigDict["bashrcFOAM"] = configDict["bashrcFOAM"]
        self.newConfigDict["paraFoam"] = configDict["paraFoam"]
        self.newConfigDict["salomeMeca"] = configDict["salomeMeca"]
        self.newConfigDict["CAD"] = configDict["CAD"]
        self.newConfigDict["editor"] = configDict["editor"]

    def getTextFromComboBox(self, combo):
        """ 指定したcomboBoxの設定内容を取得"""
        entry = combo.get_child()
        text = entry.get_text()
        return text

    def openEditor(self, fileName):
        """ editorでfileを開く"""
        fileName = os.path.expandvars(fileName)
        if len(glob.glob(fileName)) == 0:
            title = _("エラー")
            mess = _("ファイルが存在しません\n")
            mess += "  file: " + fileName
            self.errDialog(title, mess)
        else:
            pyTreeFoam.run().editor([fileName])


    #  okDialog
    #
    def okDialog(self, title, mess):
        """ okDialog表示"""
        dialog = Gtk.MessageDialog(
                    message_type =  Gtk.MessageType.INFO,
                    #   INFO, WARNING, QUESTION, ERROR, OTHER
                    buttons = Gtk.ButtonsType.OK,
                    #   NONE, OK, CLOSE, CANCEL, YES_NO, OK_CANCEL
                    text = title)
        dialog.format_secondary_text(mess)
        dialog.run()
        dialog.destroy()

    #  okCancelDialog
    #
    def okCancelDialog(self, title, mess):
        """ okCancel dialog表示"""
        dialog = Gtk.MessageDialog(
                    message_type =  Gtk.MessageType.WARNING,
                    #   INFO, WARNING, QUESTION, ERROR, OTHER
                    buttons = Gtk.ButtonsType.OK_CANCEL,
                    #   NONE, OK, CLOSE, CANCEL, YES_NO, OK_CANCEL
                    text = title)
        dialog.format_secondary_text(mess)
        res = dialog.run()
        dialog.destroy()
        if res == Gtk.ResponseType.OK:
            stat = "OK"
        else:
            stat = "CANCEL"
        return stat

    #  errDialog
    #
    def errDialog(self, title, mess):
        """ errDialog表示"""
        dialog = Gtk.MessageDialog(
                    message_type =  Gtk.MessageType.ERROR,
                    #   INFO, WARNING, QUESTION, ERROR, OTHER
                    buttons = Gtk.ButtonsType.OK,
                    #   NONE, OK, CLOSE, CANCEL, YES_NO, OK_CANCEL
                    text = title)
        dialog.format_secondary_text(mess)
        dialog.run()
        dialog.destroy()

if __name__ == "__main__":
    import gettext
    import locale
    localeDir = os.getenv("TreeFoamPath") + "/data/locale"
    locale.bindtextdomain("treefoam", localeDir)
    gettext.install("treefoam", localeDir)
    #_ = gettext.gettext

