#!/usr/bin/python3
#  coding: utf-8
#
#       couplingToFistrDialog.py
#
#       FrontISTRと連成解析する。
#
#   23/02/20    新規作成
#      03/13    clearResults:並列処理の場合を検出していなかったので追加
#      03/21    readCouplingDataFile:solidAnaComm, easyistrの読込を
#               configTreeFoamからに変更。
#               writeCouplingDataFile:solidAnaCommは、configTreeFoamと
#               couplingDataに書込。easyistrは、configTreeFoamのみに変更。
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

import os, sys
import shutil
import glob
import pyTreeFoam
import GtkParts
import universalDialogs as unvDlg
import pyFistr_ptc as pyFistr

import locale
localeDir = os.getenv("TreeFoamPath") + "/data/locale"
locale.bindtextdomain("treefoam", localeDir)

import logFileCreater
logFileCreater.log()


#------------------
#  windowApp class
#------------------
class windowApp:

    def __init__(self, caseDir):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("treefoam")
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path+"couplingToFistrDialog.glade")
        self.mainWindow = self.builder.get_object("window1")
        self.mainWindow.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        self.mainWindow.set_title(_("流体-構造の連成解析"))
        #GUIのobect名を設定
        self.setGtkObject()
        self.setToolTips()
        #属性の設定
        self.caseDir = caseDir
        self.fields = []
        self.patches = []
        self.sgrps = []
        #GUIの設定
        self.setInitialSettingGUI()
        #coupling_FrontISTRの有無を確認、無ければ作成
        self.checkFolderDir()
        #couplingFileの内容を設定
        self.couplingDataDict = readCouplingDataFile(self.caseDir)
        self.setCoupligFileSetting()


    #
    #  setGtkObject
    #----------------
    def setGtkObject(self):
        """ GUIのobject名を設定する"""
        self.label_caseDir = self.builder.get_object("label_caseDir")
        self.entry_coeff = self.builder.get_object("entry_coeff")
        self.entry_easyistr = self.builder.get_object("entry_easyistr")
        self.entry_fistr1 = self.builder.get_object("entry_fistr1")
        self.entry_solidCaseDir = self.builder.get_object("entry_solidCaseDir")
        self.entry_nOF = self.builder.get_object("entry_nOF")
        self.entry_nFistr = self.builder.get_object("entry_nFistr")
        self.entry_nThreads = self.builder.get_object("entry_nThreads")
        self.entry_buffer = self.builder.get_object("entry_buffer")
        self.entry_maxWaitTime = self.builder.get_object("entry_maxWaitTime")
        combo_fields = self.builder.get_object("combo_fields")
        self.comboFields = GtkParts.comboBoxText(combo_fields)
        combo_pointField = self.builder.get_object("combo_pointField")
        self.comboPointField = GtkParts.comboBoxText(combo_pointField)
        tree_patches = self.builder.get_object("tree_patches")
        self.treePatches = GtkParts.treeList(tree_patches)
        self.treePatches.create()
        tree_sgrps = self.builder.get_object("tree_sgrps")
        self.treeSgrps = GtkParts.treeList(tree_sgrps)
        self.treeSgrps.create()
        self.check_parallel = self.builder.get_object("check_parallel")
        self.check_thread = self.builder.get_object("check_thread")
        self.button_refFistr1 = self.builder.get_object("button_refFistr1")
        self.button_refSolidCaseDir = self.builder.get_object("button_refSolidCaseDir")
        self.button_clearResults = self.builder.get_object("button_clearResults")
        self.button_showFistrLog = self.builder.get_object("button_showFistrLog")
        self.button_getSetting = self.builder.get_object("button_getSetting")
        self.button_runCoupling = self.builder.get_object("button_runCoupling")
        self.button_close = self.builder.get_object("button_close")

    #
    #  setToolTips
    #---------------
    def setToolTips(self):
        """ toolTipを設定する"""
        clearResults = _("OpenFOAM, FrontISTRの計算結果folder, fileを削除する。")
        runShowLog = _("FrontISTRの計算logを表示する。")
        getSetting = _("入力した設定を保存\n OpenFOAMのpatch内容を連成用に修正する。")
        runCoupling = _("流体-構造の連成計算を開始する。")
        self.button_clearResults.props.tooltip_text = clearResults
        self.button_showFistrLog.props.tooltip_text = runShowLog
        self.button_getSetting.props.tooltip_text = getSetting
        self.button_runCoupling.props.tooltip_text = runCoupling

    #
    #  setInitialsettingGUI
    #--------------------
    def setInitialSettingGUI(self):
        """ GUIの初期設定"""
        #基本dirを設定
        showDir = caseDir
        if len(showDir) > 80:
            showDir = "..." + showDir[-77:]
        self.label_caseDir.set_text(showDir)
        #OpenFOAMの内容を取得
        OF = openFoam(caseDir)
        self.fields = OF.fields()
        self.patches = OF.patches()
        self.entry_coeff.set_text("1.0")
        self.comboFields.setItems(self.fields)
        if "p" in self.fields:
            self.comboFields.selectItem("p")
        self.comboPointField.setItems(self.fields)
        if "pointDisplacement" in self.fields:
            self.comboPointField.selectItem("pointDisplacement")
        self.treePatches.setItems(self.patches)
        self.entry_nOF.set_text("1")
        #FrontISTR側の設定
        self.entry_easyistr.set_text("easyistr")
        self.entry_fistr1.set_text("fistr1")
        self.entry_nFistr.set_text("1")
        #bufferの設定
        self.entry_buffer.set_text("10")
        self.entry_maxWaitTime.set_text("60")

    #
    #  checkFolderDir
    #------------------
    def checkFolderDir(self):
        """ folderDirの有無を確認し、無ければ作成する"""
        folderDir = self.caseDir + "/coupling_FrontISTR"
        if os.path.exists(folderDir) == False:
            os.mkdir(folderDir)
        dataDir = folderDir + "/data"
        if os.path.exists(dataDir) == False:
            os.mkdir(dataDir)
        pythonDir = folderDir + "/python"
        if os.path.exists(pythonDir) == False:
            os.mkdir(pythonDir)

    #
    #  setCoupligFileSetting
    #------------------------
    def setCoupligFileSetting(self):
        """ couplingDataファイルの内容を設定する"""
        dataDict = self.couplingDataDict
        if len(dataDict.keys()) == 0:
            return
        #OpenFOAM側
        if "mappingCoeff" in dataDict.keys():
            self.entry_coeff.set_text(dataDict["mappingCoeff"])
        if "field" in dataDict.keys():
            self.comboFields.selectItem(dataDict["field"])
        if "pointField" in dataDict.keys():
            self.comboPointField.selectItem(dataDict["pointField"])
        if "mappingPatch" in dataDict.keys():
            self.treePatches.selectItems([dataDict["mappingPatch"]])
        if "nStepsOF" in dataDict.keys():
            self.entry_nOF.set_text(dataDict["nStepsOF"])
        if "nProcs" in dataDict.keys():
            if int(dataDict["nProcs"]) > 1:
                self.check_parallel.set_active(True)
            else:
                self.check_parallel.set_active(False)
        #FrontISTR側
        if "easyistr" in dataDict.keys():
            self.entry_easyistr.set_text(dataDict["easyistr"])
        if "solidAnaComm" in dataDict.keys():
            self.entry_fistr1.set_text(dataDict["solidAnaComm"])
        if "solidCaseDir" in dataDict.keys():
            self.entry_solidCaseDir.set_text(dataDict["solidCaseDir"])
            solidCaseDir = dataDict["solidCaseDir"]
            if os.path.exists(solidCaseDir) == True:
                self.sgrps = fistr(solidCaseDir).sgrps()
                self.treeSgrps.setItems(self.sgrps)
        if "mappingSgrp" in dataDict.keys():
            if dataDict["mappingSgrp"] in self.sgrps:
                self.treeSgrps.selectItems([dataDict["mappingSgrp"]])
        if "nSteps" in dataDict.keys():
            self.entry_nFistr.set_text(dataDict["nSteps"])
        if "nThreads" in dataDict.keys():
            self.entry_nThreads.set_text(dataDict["nThreads"])
            if int(dataDict["nThreads"]) > 1:
                self.check_thread.set_active(True)
            else:
                self.check_thread.set_active(False)
        #streamBuffer
        if "streamBuffer" in dataDict.keys():
            bufferSize = int(float(dataDict["streamBuffer"]) / 1000.0)
            self.entry_buffer.set_text(str(bufferSize))
        if "maxWaitTime" in dataDict.keys():
            maxTime = int(float(dataDict["maxWaitTime"]) / 100.0)
            self.entry_maxWaitTime.set_text(str(maxTime))

    #
    #  main
    #----------
    def main(self):
        self.mainWindow.show()
        Gtk.main()

    def close(self, *args):
        Gtk.main_quit()

    #------------ event handler -------------------------
    #
    def onRefEasyistr(self, event):     #参照...(EasyIstr)
        self.refEasyistr()
    def onRefFistr1(self, event):       #参照...(fistr1)
        self.referFistr1()
    def onRefSolidCaseDir(self, event): #参照...(solidCaseDir)
        self.referSolidCaseDir()
    def onClose(self, event):           #閉じる
        self.close()
    def onClearResults(self, event):    #結果クリア
        self.clearResults()
    def onGetSetting(self, event):      #設定取得・保存
        self.getSetting()
    def onRunCoupling(self, event):     #連成計算開始
        self.runCoupling()
    def onShowFistrLog(self, event):    #Fistrのlog
        self.showFistrLog()
    def onRunEasyistr(self, event):     #easyIstrの起動
        self.runEasyistr()
    def onOpenFluidCase(self, event):   #流体folder開く
        self.openFluidCase()
    def onOpenSolidCase(self, event):   #個体floder開く
        self.openSolidCase()

    #
    #  refEasyistr
    #--------------
    def refEasyistr(self):
        """ easyIstrの参照"""
        title = _("command「easyistr」を指定")
        path = os.path.dirname(self.caseDir)
        newPath = self.fileDialog(title, path)
        if newPath != "":
            self.entry_easyistr.set_text(newPath)

    #
    #  referFistr1
    #--------------
    def referFistr1(self):
        """ fistr1の参照..."""
        title = _("command「fistr1」を指定")
        path = os.path.dirname(self.caseDir)
        newPath = self.fileDialog(title, path)
        if newPath != "":
            self.entry_fistr1.set_text(newPath)

    #
    #  referSolidCaseDir
    #--------------------
    def referSolidCaseDir(self):
        """ solidCaseDirを取得"""
        title = _("FrontISTRの計算folder（workFolder）を指定")
        path = os.path.dirname(self.caseDir)
        newPath = self.folderDialog(title, path)
        if newPath == "":
            return
        relPath = os.path.relpath(newPath, self.caseDir)
        self.entry_solidCaseDir.set_text(relPath)
        self.sgrps = fistr(newPath).sgrps()
        self.treeSgrps.setItems(self.sgrps)

    #
    #  clearResults
    #---------------
    def clearResults(self):
        """ OpenFOAM, FrontISTR両方の結果をクリアする"""
        #途中再開の場合、クリアできない
        OF = openFoam(self.caseDir)
        self.couplingDataDict = readCouplingDataFile(self.caseDir)
        if int(self.couplingDataDict["nProcs"]) > 1:
            startTime = OF.runStartTime("processor0")
        else:
            startTime = OF.runStartTime()
        if float(startTime) > 0.0:
            title = _("エラー")
            mess = _("startTimeが「0.0」以外の設定になっている為、結果をクリアできません。")
            print(mess)
            self.errDialog(title, mess)
            return
        #OpenFOAM側----------------
        case = pyTreeFoam.case(self.caseDir)
        #case直下
        timeFolders = case.getTimeFolders()
        timeFolders.sort()
        delFolders = timeFolders[1:]
        for delFolder in delFolders:
            delDir = self.caseDir + os.sep + delFolder
            shutil.rmtree(delDir)
        #並列の場合
        folders = pyTreeFoam.getFolderNames(self.caseDir)
        for folder in folders:
            if folder[:len("processor")] == "processor":
                procFolders = case.getTimeFolders(folder)
                procFolders.sort()
                delFolders = procFolders[1:]
                for delFolder in delFolders:
                    delDir = self.caseDir + os.sep + folder + os.sep + delFolder
                    shutil.rmtree(delDir)
        #FrontISTR側------------
        resFiles = ["FistrModel.res.*", "FistrModel*.inp", "FistrModel.restart*", "*.vtk",
             "conv*", "*.log", "FSTR.dbg.*", "FSTR.msg", "FSTR.sta",
             "*.pvtu", "*.vtu", "*.plt"]
        resFiles += ["FistrModel_eigen.res.*", "eigen_log"]
        resFiles += ["FistrModel_dyna.res.*", "dyna_*.txt"]
        resFolders = ["FistrModel.*"]
        solidCaseDir = self.entry_solidCaseDir.get_text()
        #fileを削除
        for pattern in resFiles:
            delNames = glob.glob(solidCaseDir + os.sep + pattern)
            if len(delNames) > 0:
                for delName in delNames:
                    os.remove(delName)
        #folderを削除
        for folder in resFolders:
            delNames = glob.glob(solidCaseDir + os.sep + folder)
            for delName in delNames:
                if os.path.isdir(delName) == True:
                    shutil.rmtree(delName)
        #message出力
        title = _("結果のクリア")
        mess = _("OpenFOAMとFrontISTRの計算結果folder、fileを削除しました。")
        self.okDialog(title, mess)
        print(mess)

    #
    #  getSetting
    #-------------
    def getSetting(self):
        """ 連成解析の為の設定を取得する"""
        dataDict = self.couplingDataDict
        messTitle = _("エラー")
        #OpenFOAM側取得
        dataDict["mappingCoeff"] = self.entry_coeff.get_text()
        dataDict["field"] = self.comboFields.getSelectedItem()
        dataDict["pointField"] = self.comboPointField.getSelectedItem()
        selItems = self.treePatches.getSelectedItems()
        if len(selItems) == 0:
            mess = _("OpenFOAMのpatch名が選択されていません。") + "\n"
            mess += _("patchを選択してください。")
            self.errDialog(messTitle, mess)
            return
        dataDict["mappingPatch"] = selItems[0]
        dataDict["nStepsOF"] = self.entry_nOF.get_text()
        if self.check_parallel.get_active() == True:
            folNames = pyTreeFoam.getFolderNames(self.caseDir)
            nProcs = len(list(filter(lambda x: x[:len("processor")] == "processor", folNames)))
            if nProcs == 0:
                mess  = _("メッシュが並列計算用に分割されていません。") + "\n"
                mess += _("decomposeParを実行して、メッシュ分割してください。")
                self.errDialog(messTitle, mess)
                return
        else:
            nProcs = 1
        dataDict["nProcs"] = str(nProcs)
        #FrontISTR側を取得
        dataDict["easyistr"] = self.entry_easyistr.get_text()
        dataDict["solidAnaComm"] = self.entry_fistr1.get_text()
        dataDict["solidCaseDir"] = self.entry_solidCaseDir.get_text()
        selItems = self.treeSgrps.getSelectedItems()
        if len(selItems) != 1:
            mess = _("FrontISTRのSGRP名が選択されていません。") + "\n"
            mess += _("SGRPを選択してください。")
            self.errDialog(messTitle, mess)
            return
        err = self.checkFistrAnaType(dataDict["solidCaseDir"])
        if err != "OK":
            mess = _("FrontISTRの解析種類が動解析になっていません。") + "\n"
            mess += _("動解析に設定してください。")
            self.errDialog(messTitle, mess)
            return
        #couplingDataファイルを更新
        dataDict["mappingSgrp"] = selItems[0]
        dataDict["nSteps"] = self.entry_nFistr.get_text()
        dataDict["nThreads"] = self.entry_nThreads.get_text()
        bufferSize = self.entry_buffer.get_text()
        dataDict["streamBuffer"] = str(int(bufferSize) * 1000)
        maxTime = self.entry_maxWaitTime.get_text()
        dataDict["maxWaitTime"] = str(int(maxTime) * 100)
        writeCouplingDataFile(self.caseDir, dataDict)
        self.couplingDataDict = dataDict
        #OpenFOAMのpatchを修正
        stat = self.replaceCouplingPatchConts()
        if stat != "OK":
            return
        #Pythonスクリプトをコピー
        self.copyPythonScript()
        title = _("設定取得・保存")
        mess = _("OpenFOAM, FrontISTRの設定を取得し、保存しました。") + "\n"
        mess += _("OpenFOAMのpatch内容をcoded形式に変更しました。") + "\n"
        mess += _("連成解析の為の実行ファイルをコピーしました。")
        self.okDialog(title, mess)

    #
    #  replaceCouplingPatchConts
    def replaceCouplingPatchConts(self):
        """ OpenFOAMのpatch内容をcoupling用に修正する"""
        OF = openFoam(self.caseDir)
        fields = OF.fields()
        if not ("pointDisplacement" in fields):
            title = _("エラー")
            mess = _("「pointDisplacement」のfieldが存在しません。") + "\n"
            mess += _("設定できません。")
            self.errDialog(title, mess)
            return "NG"
        #patchNameのpatch内容をcodedMixedに設定
        field = self.couplingDataDict["field"]
        patchName = self.couplingDataDict["mappingPatch"]
        errMess = ""
        patches, patchData = OF.fieldPatches(field)
        for i in range(len(patches)):
            patchCont = patchData[i]
            patchContOp = pyTreeFoam.strOp(patchCont)
            newPatchCont = patchContOp.deleteComments()
            firstLine, pp, _kind = pyTreeFoam.strOp(newPatchCont).get1line(0)
            if patches[i] == patchName:
                if firstLine.find("codedMixed") < 0:
                    if firstLine.find("zeroGradient") >= 0:
                        #codedMixedを設定
                        self.setCodedMixed(field, patchName)
                    else:
                        errMess = _("指定したfieldのpatchが「zeroGradient」ではありません。") + "\n"
                        errMess += _("fieldとpatch名を選択し直してください。")
                        break
            else:
                if firstLine.find("codedMixed") >= 0:
                    self.replaceZeroGradient(field, patches[i])
        if errMess != "":
            title = _("エラー")
            self.errDialog(title, errMess)
            return "NG"
        #patchNameのpatch内容をcodedFixedValueに設定
        patches, patchData = OF.fieldPatches("pointDisplacement")
        for i in range(len(patches)):
            patchCont = patchData[i]
            patchContOp = pyTreeFoam.strOp(patchCont)
            newPatchCont = patchContOp.deleteComments()
            firstLine, pp, _kind = pyTreeFoam.strOp(newPatchCont).get1line(0)
            pField = "pointDisplacement"
            if patches[i] == patchName:
                if firstLine.find("codedFixedValue") < 0:
                    if firstLine.find("fixedValue") >= 0:
                        self.setCodedFixedValue(pField, patchName)
                    else:
                        errMess = _("field:pointDisplacementのpatchが「fixedValue」ではありません。") + "\n"
                        errMess += _("patch名を選択し直してください。")
                        break
            else:
                if firstLine.find("codedFixedValue") >= 0:
                    self.replaceFixedValue(pField, patches[i])
        if errMess != "":
            title = _("エラー")
            self.errDialog(title, errMess)
            return "NG"
        return "OK"

    #
    #  setCodedMixed
    def setCodedMixed(self, field, patchName):
        """ 指定したfieldのpatchをcodedMixedに置き換える"""
        fileName = os.sep.join([os.getenv("TreeFoamPath"), "frontIstr", "patchCont_codedMixed"])
        f = open(fileName); newPatchCont = f.read(); f.close()
        self.setNewPatchCont(field, patchName, newPatchCont)

    #
    #  setCodedFixedValue
    def setCodedFixedValue(self, field, patchName):
        """ 指定したfieldのpatchをcodedFixedValueに置き換える"""
        fileName = os.sep.join([os.getenv("TreeFoamPath"), "frontIstr", "patchCont_codedFixedValue"])
        f = open(fileName); newPatchCont = f.read(); f.close()
        self.setNewPatchCont(field, patchName, newPatchCont)

    #
    #  replaceZeroGradient
    def replaceZeroGradient(self, field, patchName):
        """ 指定したfieldのpatchをzeroGradientに置き換える"""
        newPatchCont  = "type    zeroGradient;\n"
        newPatchCont += "value   uniform 0;"
        self.setNewPatchCont(field, patchName, newPatchCont)

    #
    #  replaceFixedValue
    def replaceFixedValue(self, field, patchName):
        """ 指定したfieldのpatchをfixedValueに置き換える"""
        newPatchCont  = "type    fixedValue;\n"
        newPatchCont += "value   uniform (0 0 0);"
        self.setNewPatchCont(field, patchName, newPatchCont)

    #
    #  setNewPatchCont
    def setNewPatchCont(self, field, patchName, newPatchCont):
        """ 指定したfieldのpatch内容をnewPatchContに置き換える"""
        case = pyTreeFoam.case(self.caseDir)
        currTime = case.getCurrTimeFolder()
        foamFile = pyTreeFoam.foamFile()
        fieldPath = self.caseDir + os.sep + currTime + os.sep + field
        fieldCont = foamFile.read(fieldPath)
        fieldContOp = pyTreeFoam.strOp(fieldCont)
        #boundaryFieldを探す
        pBnd = fieldContOp.getKeywordPointer("boundaryField", 0)
        (bndCont, pp) = fieldContOp.getMiddlePair(pBnd)
        bndContOp = pyTreeFoam.strOp(bndCont)
        pPatch = bndContOp.getKeywordPointer(patchName, 0)
        (patchCont, pp) = bndContOp.getMiddlePair(pPatch)
        #header, footerを設定
        fieldHeader = fieldCont[:pBnd]
        bndHeader  = "boundaryField\n"
        bndHeader += "{\n"
        bndHeader += bndCont[:pPatch]
        bndFooter = bndCont[pp:] + "}\n"
        fieldFooter  = "\n"
        fieldFooter += "// ************************************************************************* //"
        #newPatchを作成
        newPatch  = patchName + "\n"
        newPatch += "    {\n"
        newPatchLines = newPatchCont.split("\n")
        for line in newPatchLines:
            newPatch += "        " + line + "\n"
        newPatch += "    }\n"
        #newPatchを置き換える
        newFieldCont = fieldHeader
        newFieldCont += bndHeader + newPatch
        newFieldCont += bndFooter + fieldFooter
        #保存する
        foamFile.write(fieldPath, newFieldCont)

    #
    #  checkFistrAnaType
    def checkFistrAnaType(self, solidCaseDir):
        """ fistr側の解析typeが!DYNAMICの設定になっているか確認"""
        fileName = solidCaseDir + os.sep + "FistrModel.cnt"
        f = open(fileName); lines = f.readlines(); f.close()
        err = "error"
        for line in lines:
            words = pyFistr.deleteSp(line).split(",")
            if len(words) >= 1:
                if words[0] == "!DYNAMIC":
                    err = "OK"
                    break
        return err

    #
    #  copyPythonScript
    def copyPythonScript(self):
        """ pythonスクリプトをコピーする"""
        #folder有無確認
        couplingFolder = self.caseDir + "/coupling_FrontISTR"
        if os.path.exists(couplingFolder) == False:
            os.mkdir(couplingFolder)
        pythonFolder = couplingFolder + "/python"
        if os.path.exists(pythonFolder) == False:
            os.mkdir(pythonFolder)
        #pythonスクリプトをコピー
        #  sourceを取得
        trDir = os.getenv("TreeFoamPath")
        scripts = []
        scripts.append(trDir + "/frontIstr/convertMesh.py")
        scripts.append(trDir + "/frontIstr/coupling_OF_fistr.py")
        scripts.append(trDir + "/frontIstr/createPointCloudFromOpenFoam.py")
        scripts.append(trDir + "/frontIstr/mappingFromPointCloud.py")
        scripts.append(trDir + "/frontIstr/pyFistr_ptc.py")
        scripts.append(trDir + "/frontIstr/pyTreeFoam_ptc.py")
        scripts.append(trDir + "/frontIstr/setPress_getDisp.py")
        scripts.append(trDir + "/frontIstr/stringOp.py")
        scripts.append(trDir + "/frontIstr/teeCorrect.py")
        for name in scripts:
            shutil.copy(name, pythonFolder)

    #
    #  runCoupling
    #--------------
    def runCoupling(self):
        """ 連成計算開始"""
        configDict = pyTreeFoam.readConfigTreeFoam()
        bashrc = configDict["bashrcFOAM"]
        OF = openFoam(self.caseDir)
        contDict = OF.controlDict()
        deltaT = contDict["deltaT"]
        writeInterval = contDict["writeInterval"]
        #adjustTimeStep = contDict["adjustTimeStep"]
        self.couplingDataDict = readCouplingDataFile(self.caseDir)
        #startTimeのfolderが存在するか確認
        if int(self.couplingDataDict["nProcs"]) > 1:
            #並列処理
            timeFolders = pyTreeFoam.case(self.caseDir).getTimeFolders("processor0")
            startTime = OF.runStartTime("processor0")
        else:
            #シングルコア
            timeFolders = pyTreeFoam.case(self.caseDir).getTimeFolders()
            startTime = OF.runStartTime()
        timeNums = list(map(float, timeFolders))
        if float(startTime) in timeNums == False:
            title = _("エラー")
            mess = _("timeFolder「") + startTime + _("」が存在しません。") + "\n"
            mess += _("計算開始できません。")
            self.errDialog(title, mess)
            return
        #fistr1の有無を確認
        fistr1 = self.couplingDataDict["solidAnaComm"]
        comm = "which " + fistr1
        stat, res, err = pyTreeFoam.run(self.caseDir).commandReturnCont(comm)
        if stat != "OK" or len(res) <= 1:
            title = _("エラー")
            mess = _("command「fistr1」が見つかりません。") + "\n"
            mess += _("計算開始できません。")
            self.errDialog(title, mess)
            return
        #定数を取得
        self.couplingDataDict["startTime"] = startTime
        self.couplingDataDict["deltaT"] = deltaT
        self.couplingDataDict["writeInterval"] = writeInterval
        writeCouplingDataFile(self.caseDir, self.couplingDataDict)
        #起動確認
        title = _("流体-構造連成解析")
        mess  = _("OpenFOAMとFrontISTRの連成解析を開始します。") + "\n\n"
        mess += _("      startTime: ") + startTime + "\n"
        mess += _("  writeInterval: ") + writeInterval + "\n"
        mess += _("         nProcs: ") + self.couplingDataDict["nProcs"] + _(" (OpenFOAM)") + "\n"
        mess += _("       nThreads: ") + self.couplingDataDict["nThreads"] + _(" (FrontISTR)")  + "\n"
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return
        #実行
        runComm  = "#!/bin/bash\n"
        runComm += ". " + bashrc + "\n"
        runComm += "python3 coupling_FrontISTR/python/coupling_OF_fistr.py"
        fileName = self.caseDir + "/runFsi"
        f = open(fileName, "w"); f.write(runComm); f.close()
        runCase = pyTreeFoam.run(self.caseDir)
        runCase.command("chmod a+x runFsi")
        runCase.foamTerminal("runFsi")

    #
    #  showFistrLog
    #----------------
    def showFistrLog(self):
        """ FrontISTRのlog表示"""
        comm = pyTreeFoam.TerminalRun + "tail -f solidSolve.log"
        runCase = pyTreeFoam.run(self.caseDir)
        runCase.command(comm)

    #
    #  runEasyistr
    #--------------
    def runEasyistr(self):
        """ EasyISTRの起動"""
        solidCaseDir = self.entry_solidCaseDir.get_text()
        if solidCaseDir[0] != os.path:
            solidCaseDir = os.path.abspath(solidCaseDir)
        easyistrPath = self.entry_easyistr.get_text()
        comm = "which " + easyistrPath
        stat, res, err = pyTreeFoam.run().commandReturnCont(comm)
        if stat != "OK" or len(res) <= 1:
            title = _("エラー")
            mess = _("command「easyistr」が見つかりません。") + "\n"
            mess += _("計算開始できません。")
            self.errDialog(title, mess)
            return
        comm = easyistrPath + " -d " + solidCaseDir + " &"
        print(comm)
        pyTreeFoam.run().command(comm)

    #
    #  openFluidCase
    #----------------
    def openFluidCase(self):
        """ 流体folderを開く"""
        pyTreeFoam.run(self.caseDir).fileManager()

    #
    #  openSolidCase
    #----------------
    def openSolidCase(self):
        """ 固体folderを開く"""
        solidCaseDir = self.entry_solidCaseDir.get_text()
        if solidCaseDir[0] != os.path:
            solidCaseDir = os.path.abspath(solidCaseDir)
        pyTreeFoam.run(solidCaseDir).fileManager()

    # --------- universal dialog ----------------

    #  okDialog
    def okDialog(self, title, mess):
        """ okDialog表示"""
        unvDlg.okDialog(self, title, mess)

    #  okCancelDialog
    def okCancelDialog(self, title, mess):
        """ okCancel dialog表示"""
        return unvDlg.okCancelDialog(self, title, mess)

    #  errDialog
    def errDialog(self, title, mess):
        """ errDialog表示"""
        unvDlg.errDialog(self, title, mess)

    # folderDialog
    def folderDialog(self, title, path):
        return unvDlg.folderDialog(self, title, path)

    #fileDialog
    def fileDialog(self, title, path, patter=""):
        return unvDlg.fileDialog(self, title, path, pattern="")

#----------------------
#  openFoam class
#----------------------
class openFoam:
    """ openFOAM関連の内容を取得"""
    def __init__(self, caseDir):
        self.caseDir = caseDir

    def timeFolders(self):
        case = pyTreeFoam.case(self.caseDir)
        timeFolders = case.getTimeFolders()
        return timeFolders

    def fields(self):
        """ fieldを取得し、list形式で返す"""
        case = pyTreeFoam.case(self.caseDir)
        timeFolders = case.getTimeFolders()
        timeFolder = timeFolders[0]
        fields = case.getFieldNames(timeFolder)
        fields.sort()
        return fields

    def patches(self):
        """ patchを取得し、list形式で返す"""
        case = pyTreeFoam.case(self.caseDir)
        timeFolders = case.getTimeFolders()
        maxTime = timeFolders[0]
        meshDir = case.getCurrMeshDir(maxTime, ".", "boundary")
        patchConts = case.getPatchNameFromBoundary(meshDir)
        patches = list(map(lambda x: x[0], patchConts))
        return patches

    def runStartTime(self, processor="."):
        """ 計算開始時間を取得"""
        getTime = ""
        case = pyTreeFoam.case(self.caseDir)
        contDict = case.getControlDict()
        if contDict["startFrom"] == "firstTime":
            timeFolders = case.getTimeFolders(processor)
            getTime = timeFolders[0]
        elif contDict["startFrom"] == "startTime":
            getTime = contDict["startTime"]
        elif contDict["startFrom"] == "latestTime":
            timeFolders = case.getTimeFolders(processor)
            getTime = timeFolders[-1]
        return getTime

    def controlDict(self):
        """ controlDictを取得"""
        case = pyTreeFoam.case(self.caseDir)
        controlDict = case.getControlDict()
        return controlDict

    def deltaT(self):
        """ deltaTを取得"""
        case = pyTreeFoam.case(self.caseDir)
        contDict = case.getControlDict()
        deltaT = contDict["deltaT"]
        return deltaT

    def writeInterval(self):
        """ writeIntervalを取得"""
        case = pyTreeFoam.case(self.caseDir)
        contDict = case.getControlDict()
        writeInterval = contDict["writeInterval"]
        return writeInterval

    def adjustTimeStep(self):
        case = pyTreeFoam.case(self.caseDir)
        contDict = case.getControlDict()
        adjustTimeStep = contDict["adjustTimeStep"]
        return adjustTimeStep

    def fieldPatches(self, field):
        """指定したfieldの全patch名とpatch内容を取得"""
        case = pyTreeFoam.case(self.caseDir)
        currTimeFolder = case.getCurrTimeFolder()
        _fields, patches, patchData = case.getFieldsPatchNames(currTimeFolder, [field])
        return patches, patchData[0]


#----------------
#  fistr class
#----------------
class fistr:

    def __init__(self, solidCaseDir):
        self.solidCaseDir = solidCaseDir
        saveDir = os.getcwd()
        os.chdir(solidCaseDir)
        lines = pyFistr.readFistrModelMsh()
        self.headerData = pyFistr.getHeaderData(lines)
        os.chdir(saveDir)

    def sgrps(self):
        """ sgrpを取得し、list形式で返す。"""
        sgrps = []
        for header in self.headerData:
            line = header[0]
            item = pyFistr.deleteSp(line).split(",")[0]
            if item == "!SGROUP":
                name = pyFistr.getValName(line, "SGRP")
                sgrps.append(name)
        return sgrps

#
#  readCouplingDataFile
#-----------------------
def readCouplingDataFile(caseDir):
    """ couplingDataファイル読み込み"""
    fileName = caseDir + "/coupling_FrontISTR/couplingData"
    dataDict = {}
    if os.path.exists(fileName) == False:
        return dataDict
    f = open(fileName); lines = f.readlines(); f.close()
    for line in lines:
        words = line.split()
        if len(words) >= 2:
            if words[0] == "nProcs":
                dataDict["nProcs"] = words[1]
            elif words[0] == "field":
                dataDict["field"] = words[1]
            elif words[0] == "pointField":
                dataDict["pointField"] = words[1]
            elif words[0] == "mappingPatch":
                dataDict["mappingPatch"] = words[1]
            elif words[0] == "startTime":
                dataDict["startTime"] = words[1]
            elif words[0] == "deltaT":
                dataDict["deltaT"] = words[1]
            elif words[0] == "writeInterval":
                dataDict["writeInterval"] = words[1]
            elif words[0] == "nStepsOF":
                dataDict["nStepsOF"] = words[1]
            elif words[0] == "streamBuffer":
                dataDict["streamBuffer"] = words[1]
            elif words[0] == "maxWaitTime":
                dataDict["maxWaitTime"] = words[1]
            elif words[0] == "solidCaseDir":
                dataDict["solidCaseDir"] = words[1]
            elif words[0] == "nThreads":
                dataDict["nThreads"] = words[1]
            elif words[0] == "mappingSgrp":
                dataDict["mappingSgrp"] = words[1]
            elif words[0] == "mappingCoeff":
                dataDict["mappingCoeff"] = words[1]
            elif words[0] == "nSteps":
                dataDict["nSteps"] = words[1]
            #elif words[0] == "solidAnaComm":
            #    dataDict["solidAnaComm"] = words[1]
            #elif words[0] == "easyistr":
            #    dataDict["easyistr"] = words[1]
    configDict = pyTreeFoam.readConfigTreeFoam()
    dataDict["solidAnaComm"] = configDict["fistr1"]
    dataDict["easyistr"] = configDict["easyistr"]
    return dataDict

#
#  writeCouplingDataFile
#-------------------------
def writeCouplingDataFile(caseDir, dataDict):
    """ couplingDataFileに保存する"""
    fileName = caseDir + "/coupling_FrontISTR/couplingData"
    if os.path.exists(fileName) == True:
        f = open(fileName); lines = f.readlines(); f.close()
    else:
        folder = os.path.dirname(fileName)
        if os.path.exists(folder) == False:
            os.mkdir(folder)
        lines = []
    configDict = {}
    fistr1 = dataDict["solidAnaComm"]
    easyistr = dataDict.pop("easyistr")
    configDict["fistr1"] = fistr1
    configDict["easyistr"] = easyistr
    pyTreeFoam.writeConfigTreeFoam(configDict)
    for key in dataDict.keys():
        cont = dataDict[key]
        flag = 0
        for i in range(len(lines)):
            line = lines[i]
            words = line.split()
            if len(words) >= 2:
                if words[0] == key:
                    words[1] = cont
                    word = ("%-15s" % words[0] + " ")
                    lines[i] = word + " ".join(words[1:]) + "\n"
                    flag = 1
                    break
        if flag == 0:
            word = ("%-15s" % key + " ")
            line = word + cont + "\n"
            lines.append(line)
    f = open(fileName, "w"); f.writelines(lines); f.close()



if __name__ == "__main__":
    import gettext
    gettext.install("treefoam", localeDir)
    #_ = gettext.gettext

    caseDir = sys.argv[1]
    caseDir = os.path.abspath(caseDir)
    winApp = windowApp(caseDir)
    winApp.main()
