#!/usr/bin/python3
# coding: utf-8
#
#   createLayerDDialog.py
#
#       meshにlayerを追加するためのsnappyHexMeshDictを作成する
#
#   14/08/31    新規作成
#      09/07    修正
#      09/08    decomposeParDictのコピーを追加
#      09/09    initialize, createLayerPatch:バグ修正
#   15/01/24    layer設定方法がpatch毎に設定できるように修正
#      01/25    バグ修正（layer設定がｸﾘｱされる時があった）
#      06/25    wxPython-2.8→3.0に伴う修正
#               listCtrlのsetSelection(0)でデータが存在しない場合、エラー発生
#      07/17    dialogの起動方法修正
#      11/30    copySnappyHexMeshDict:dictFileのコピー元Dir取得方法を変更
#               copyDecomposeParDict:　　↑
#      12/12    dict内にcomment文が有っても、修正できるように変更
#   19/10/20    GTK+3, python3用に大幅修正
#   20/04/21    多言語化対応
#      11/21    getLayerSetting:defaultのexpRatioを1.0→1.2に変更
#               setSnappyHexMeshDict:取得したlayerPatchNameが反映されなかった
#               事を修正。
#   22/07/19    getLayerPatchNamesFromDict:OFversionのcheck方法を修正（OF-10対応）
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
import os, sys
import glob
import shutil

import pyTreeFoam
import universalDialogs as unvDlg

import locale
localeDir = os.getenv("TreeFoamPath") + "/data/locale"
locale.bindtextdomain("treefoam", localeDir)

#-------------------
#  dialogApp class
#-------------------
class dialogApp:

    def __init__(self, caseDir):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("treefoam")
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path + "createLayerDDialog.glade")
        self.dialog = self.builder.get_object("dialog1")
        self.dialog.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        self.setGtkObject()
        #変数の設定
        self.caseDir = caseDir
        self.stat = "CANCEL"
        self.patchNames = []            #設定しないpatch名
        self.layerPatchNames = []       #layer設定するpatch名
        self.layerPatchSetting = []     #[patch, [layer数, 相対, 厚さ, min厚さ, 拡大率, 角度]]
        self.layerSetting = []          #[layer数, 相対, 厚さ, min厚さ, 拡大率, 角度]
        self.showPatchName = ""         #layerSettigを表示しているpatch名
        #初期化
        self.initialize()
        self.iniGui()

    #
    #  show
    #  ----
    def show(self):
        self.dialog.run()
        return self.stat

    def close(self, *args):
        self.dialog.destroy()

    def setGtkObject(self):
        """ GUIのobject名を取得する"""
        self.tree_patch = self.builder.get_object("tree_patch")
        self.tree_setPatch = self.builder.get_object("tree_setPatch")
        self.combo_relSize = self.builder.get_object("combo_relSize")
        self.entry_angle = self.builder.get_object("entry_angle")
        self.entry_nLayer = self.builder.get_object("entry_nLayer")
        self.entry_layThick = self.builder.get_object("entry_layThick")
        self.entry_minThick = self.builder.get_object("entry_minThick")
        self.entry_exRatio = self.builder.get_object("entry_exRatio")

    #---- event handler ------------
    #選択
    def onSelectPatch(self, event):
        self.selectPatch()
    #戻す
    def onReturnPatch(self, event):
        self.returnPatch()
    #setPatch側のtreeを選択
    def onSelectShowLayer(self, event):
        self.selectShowLayer()
    #OK
    def onOk(self, event):
        self.stat = "OK"
        self.setSnappyHexMeshDict()
        self.close()
    #キャンセル
    def onCancel(self, event):
        self.stat = "CANCEL"
        self.close()

    def iniGui(self):
        """ GUIの初期化"""
        #tree_patchの初期化
        self.tree_patch.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
        #self.tree_setPatch.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
        self.tree_patch.set_headers_visible(False)
        self.tree_setPatch.set_headers_visible(False)
        cellRenderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn()
        column.pack_start(cellRenderer, True)
        column.add_attribute(cellRenderer, "text", 0)
        self.tree_patch.append_column(column)
        setColumn = Gtk.TreeViewColumn()
        setColumn.pack_start(cellRenderer, True)
        setColumn.add_attribute(cellRenderer, "text", 0)
        self.tree_setPatch.append_column(setColumn)
        #treePatchの表示
        self.setListDataToBox(self.tree_patch, self.patchNames)
        self.setListDataToBox(self.tree_setPatch, self.layerPatchNames)
        #combBoxにデータセット
        items = ["true", "false"]
        self.combo_relSize.clear()
        listModel = Gtk.ListStore(str)
        for name in items:
            listModel.append([name])
        cell = Gtk.CellRendererText()
        self.combo_relSize.pack_start(cell, True)
        self.combo_relSize.add_attribute(cell, "text", 0)
        self.combo_relSize.set_model(listModel)
        self.combo_relSize.set_entry_text_column(0)
        #comboを選択
        if self.layerSetting[1] == "true":
            self.combo_relSize.set_active(0)
        else:
            self.combo_relSize.set_active(1)
        #entryにデータセット
        angle = self.layerSetting[5]
        self.entry_angle.set_text(angle)

    def setListDataToBox(self, treeList, names):
        """ treeListにdataをセット"""
        listStore = Gtk.ListStore(str)
        names.sort()
        for name in names:
            listStore.append([name])
        treeList.set_model(listStore)

    def getSelectPatches(self, treeList):
        """ treeListから選択しているpatch名を取得する"""
        listSelection = treeList.get_selection()
        selPatches = []
        (listStore, pathes) = listSelection.get_selected_rows()
        for path in pathes:
            iter = listStore.get_iter(path)
            name = listStore.get_value(iter, 0)
            selPatches.append(name)
        return selPatches

    def selectPatchNames(self, treeList, names):
        """ treeList内のpatchを選択表示させる"""
        listSelection = treeList.get_selection()
        listStore = treeList.get_model()
        for i in range(len(listStore)):
            path = Gtk.TreePath(i)
            iter = listStore.get_iter(path)
            patch = listStore.get_value(iter, 0)
            if patch in names:
                listSelection.select_iter(iter)

    def initialize(self):
        """ 変数の初期化"""
        case = pyTreeFoam.case(self.caseDir)
        self.configDict = pyTreeFoam.readConfigTreeFoam()
        #全patchNamesを取得
        maxTime = case.getCurrTimeFolder()
        region = "."
        meshName = "boundary"
        relativeMeshDir = case.getCurrMeshDir(maxTime, region, meshName)
        meshDir = self.caseDir + "/" + "/".join(relativeMeshDir.split("/")[:-1])
        names = case.getPatchNameFromBoundary(meshDir)
        names = list(map(lambda x: x[0], names))
        #layerPatchとlayerSettingを取得
        (layerPatchNames, layerSetting,
            layerPatchSetting) = self.getLayerPatchNamesFromDict()
        layerPatchNames.sort()
        layerPatchSetting.sort()
        #全patch名からlayerPatchを削除
        patchNames = list(set(names).difference(layerPatchNames))
        patchNames.sort()
        #全patchに含まれないlayerPatchを削除
        layerPatches = list(set(names).intersection(layerPatchNames))
        layerPatches.sort()
        #self付変数に置き換え
        self.patchNames = patchNames
        self.layerSetting = layerSetting
        self.layerPatchNames = layerPatchNames
        self.layerPatchSetting = layerPatchSetting

    def getLayerPatchNamesFromDict(self):
        """ snappyDictからlayerを設定しているpatchを取得。
        Dictファイルをチェックし、無ければコピーする。"""
        if len(glob.glob(self.caseDir+"/system")) == 0:
            return []
        #Dictファイル有無確認し、無い場合は、copyする
        getFoamContents = pyTreeFoam.getFoamContents(self.caseDir)
        OFversion = self.configDict["OFversion"]
        numVer = pyTreeFoam.getNumVersion(OFversion)
        sourceDir = getFoamContents.getOFversionFolder(OFversion)
        pasteDir = self.caseDir + "/system"
        fileName = self.caseDir + "/system/decomposeParDict"
        if len(glob.glob(fileName)) == 0:
            #decomposeParDictをコピーする
            sourceFile = sourceDir + "/system/decomposeParDict"
            shutil.copy(sourceFile, pasteDir)
        fileName = self.caseDir + "/system/snappyHexMeshDict"
        if len(glob.glob(fileName)) == 0:
            sourceFile = sourceDir + "/system/snappyHexMeshDict"
            shutil.copy(sourceFile, pasteDir)
        #meshQualityDictを確認
        #if (OFversion >= "6" and OFversion[0] != "v") or OFversion >= "v1906":
        if ( numVer >= 6.0 or OFversion >= "v1906" ):
            fileName = self.caseDir + "/system/meshQualityDict"
            if (len(glob.glob(fileName))) == 0:
                sourceFile = sourceDir + "/system/meshQualityDict"
                shutil.copy(sourceFile, pasteDir)
        #snappyContからlayerPatchを取得
        fileName = pasteDir + "/snappyHexMeshDict"
        f = open(fileName); cont = f.read(); f.close()
        contOp = pyTreeFoam.strOp(cont)
        cont = contOp.deleteCommentsNullLines()
        (layerCont, p) = contOp.getKeywordContents("addLayersControls", 0)
        layerContOp = pyTreeFoam.strOp(layerCont)
        (layerPatches, p) = layerContOp.getKeywordContents("layers", 0)
        #layerPatch名を取得
        patches = []
        patchSetting = []
        layerPatchesOp = pyTreeFoam.strOp(layerPatches)
        (lineCont, p, _kind) = layerPatchesOp.get1line(0)
        while lineCont != "":
            patchName = lineCont.split()[0]
            patches.append(patchName)
            (patchLayerCont, _pp) = pyTreeFoam.strOp(lineCont).getMiddlePair(0)
            setting = self.getLayerSetting(patchLayerCont)
            patchSetting.append([patchName, setting])
            (lineCont, p, _kind) = layerPatchesOp.get1line(p)
        #layerSettingを取得（全体設定
        setting = self.getLayerSetting(layerCont)
        #patchLayerを整形
        for (patchName, patchCont) in patchSetting:
            for i in range(len(setting)):
                if patchCont[i] == "":
                    patchCont[i] = setting[i]
        return (patches, setting, patchSetting)

    def getLayerSetting(self, cont):
        """ layerSettingを取得する"""
        setting = ["3", "true", "0.3", "0.1", "1.2", "120"]
        contOp = pyTreeFoam.strOp(cont)
        (lineCont, p, kind) = contOp.get1line(0)
        while lineCont != "":
            if kind == "line":
                items = lineCont.split()
                item = items[0]
                value = ""
                if len(items) > 1:
                    value = items[1].split(";")[0]
                if item == "nSurfaceLayers":
                    setting[0] = value
                elif item == "relativeSizes":
                    setting[1] = value
                elif item == "finalLayerThickness":
                    setting[2] = value
                elif item == "minThickness":
                    setting[3] = value
                elif item == "expansionRatio":
                    setting[4] = value
                elif item == "featureAngle":
                    setting[5] = value
            (lineCont, p, kind) = contOp.get1line(p)
        return setting

    #
    #  selectPatch
    #  -----------
    def selectPatch(self):
        """ 選択したpatchをsetPatch側に移動する"""
        #選択patch名を取得
        selNames = self.getSelectPatches(self.tree_patch)
        #patchNames、layerPatchNamesを取得
        self.patchNames = list(set(self.patchNames).difference(selNames))
        self.layerPatchNames += selNames
        #各listBoxを再表示
        self.setListDataToBox(self.tree_patch, self.patchNames)
        self.setListDataToBox(self.tree_setPatch, self.layerPatchNames)
        #選択したpatchのlayerSettingが存在しない場合defaultのlayerSettingを作成
        self.createLayerPatchSetting(selNames)
        #移動後のpatch名を選択表示
        self.selectPatchNames(self.tree_setPatch, selNames)
        #layerSettingを表示
        self.selectShowLayer()

    def createLayerPatchSetting(self, selNames):
        """ patchLayerSettingを確認し、存在しない場合はdefaultのsettingを作成する"""
        for name in selNames:
            flag = 0
            for setting in self.layerPatchSetting:
                if setting[0] == name:
                    flag = 1
                    break
            if flag == 0:
                #defaultのsettingを作成
                patchSetting = [name, self.layerSetting]
                self.layerPatchSetting.append(patchSetting)

    #
    #  returnPatch
    #  -----------
    def returnPatch(self):
        """ 選択したpatchを戻す"""
        selNames = self.getSelectPatches(self.tree_setPatch)
        self.patchNames += selNames
        self.layerPatchNames = list(set(self.layerPatchNames).difference(selNames))
        #各listBoxを再表示
        self.setListDataToBox(self.tree_patch, self.patchNames)
        self.setListDataToBox(self.tree_setPatch, self.layerPatchNames)
        #treeSet側のpatchが選択されていない状態のため、layerSettingをクリアする
        self.entry_nLayer.set_text("")
        self.entry_layThick.set_text("")
        self.entry_minThick.set_text("")
        self.entry_exRatio.set_text("")
        self.showPatchName = ""
        #移動後のpatch名を選択表示
        self.selectPatchNames(self.tree_patch, selNames)

    #
    #  selectShowLayer
    #  ---------------
    def selectShowLayer(self):
        """ 選択したpatchのlayerSettingを表示する"""
        selNames = self.getSelectPatches(self.tree_setPatch)
        if len(selNames) == 0:
            return
        selName = selNames[0]
        #現在layerSettingを格納する
        nLayer   = self.entry_nLayer.get_text()
        layThick = self.entry_layThick.get_text()
        minThick = self.entry_minThick.get_text()
        exRatio  = self.entry_exRatio.get_text()
        for i in range(len(self.layerPatchSetting)):
            setting = self.layerPatchSetting[i][:]       
            name = setting[0]
            if name == self.showPatchName:
                setting[1][0] = nLayer
                setting[1][2] = layThick
                setting[1][3] = minThick
                setting[1][4] = exRatio
                self.layerPatchSetting[i][1] = setting[1][:]
                break
        #layerSettingを書き換え
        patchSetting = []
        for setting in self.layerPatchSetting:
            name = setting[0]
            if name == selName:
                patchSetting = setting[1]
                break
        if len(patchSetting) != 0:
            #layerSettingをentryに表示させる
            nLayer = patchSetting[0]
            layThick = patchSetting[2]
            minThick = patchSetting[3]
            exRatio = patchSetting[4]
            self.entry_nLayer.set_text(nLayer)
            self.entry_layThick.set_text(layThick)
            self.entry_minThick.set_text(minThick)
            self.entry_exRatio.set_text(exRatio)
        #setting表示patch名を更新
        self.showPatchName = selName

    #
    #  setSnappyHexMeshDict
    #  --------------------
    def setSnappyHexMeshDict(self):
        """ snappyHexmeshDictを書き換える"""
        (layerPatchNames, layerSetting,
            layerPatchSetting) = self.getLayerPatchAndSetting()
        error = self.checkLayerPatchSetting(layerPatchNames, layerSetting, layerPatchSetting)
        if error != "":
            return
        self.layerPatchSetting = layerPatchSetting
        snappyFileName = self.caseDir + "/system/snappyHexMeshDict"
        f = open(snappyFileName); contents = f.read(); f.close()
        #layerSwitchを設定
        contentsOp = pyTreeFoam.strOp(contents)
        ps = contentsOp.skipFoamFile()
        newLine = "castellatedMesh false;"
        (contents, p, _stat) = contentsOp.replace1lineKeyword(ps, newLine)
        newLine = "snap false;"
        (contents, p, _stat) = contentsOp.replace1lineKeyword(ps, newLine)
        newLine = "addLayers true;"
        (contents, p, _stat) = contentsOp.replace1lineKeyword(ps, newLine)
        #layersPatchをセット
        (lineCont, p, kind) = contentsOp.get1line(ps)
        while lineCont != "":
            if kind == "keyword":
                lineContOp = pyTreeFoam.strOp(lineCont)
                (keyword, _pp) = lineContOp.getKeyword(0)
                if keyword == "addLayersControls":
                    ps = p - len(lineCont)      #ポインタを頭に戻す
                    header = contents[:ps]
                    footer = contents[p:]
                    #layersPatchをセット
                    ps = lineContOp.skipIntoPair(0)
                    newLine  = "layers\n        {\n"
                    for (patch, setting) in self.layerPatchSetting:
                        newLine += "            " + patch + "\n"
                        newLine += "            {\n"
                        newLine += "                nSurfaceLayers " + setting[0] + ";\n"
                        newLine += "                finalLayerThickness " + setting[2] + ";\n"
                        newLine += "                minThickness " + setting[3] + ";\n"
                        newLine += "                expansionRatio " + setting[4] + ";\n"
                        newLine += "            }\n"
                    newLine += "        }"
                    (lineCont, _pp, _keyword) = lineContOp.replace1lineKeyword(ps, newLine)
                    #relativeSizesをセット
                    newLine = "relativeSizes " + layerSetting[1] + ";"
                    (lineCont, _pp, _keyword) = lineContOp.replace1lineKeyword(ps, newLine)
                    #featureAngleをセット
                    newLine = "featureAngle " + layerSetting[5] + ";"
                    (lineCont, _pp, _keyWord) = lineContOp.replace1lineKeyword(ps, newLine)
                    break
            (lineCont, p, kind) = contentsOp.get1line(p)
        contents = header + lineCont + footer
        #contentsを書き込み
        f = open(snappyFileName, "w"); f.write(contents); f.close()
        title = _("layerの設定")
        mess = _("snappyHexMeshDictを作成しました。")
        print(mess)
        self.okDialog(title, mess)

    def getLayerPatchAndSetting(self):
        """ layerの設定を取得する"""
        #layerPatchを取得
        patchNames = []
        listStore = self.tree_setPatch.get_model()
        for i in range(len(listStore)):
            path = Gtk.TreePath(i)
            iter = listStore.get_iter(path)
            patch = listStore.get_value(iter, 0)
            patchNames.append(patch)
        #layerSettingを取得(現在の全設定)
        iter = self.combo_relSize.get_active_iter()
        listModel = self.combo_relSize.get_model()
        relSize = listModel.get_value(iter, 0)
        angle = self.entry_angle.get_text()
        nLayer   = self.entry_nLayer.get_text()
        layThick = self.entry_layThick.get_text()
        minThick = self.entry_minThick.get_text()
        exRatio  = self.entry_exRatio.get_text()
        layerSetting = [nLayer, relSize, layThick, minThick, exRatio, angle]
        #選択patch名を取得
        selPatches = self.getSelectPatches(self.tree_setPatch)
        if len(selPatches) > 0:
            selPatch = selPatches[0]
        else:
            selPatch = ""
        #patchLayerSettingを取得
        layerPatches = []
        for setting in self.layerPatchSetting:
            name = setting[0]
            if name == selPatch:
                setting[1] = layerSetting[:]
                layerPatches.append(setting[:])
            elif name in patchNames:
                layerPatches.append(setting[:])
        return (patchNames, layerSetting, layerPatches)

    def checkLayerPatchSetting(self, patchNames, setting, patchSetting):
        """ errorチェック"""
        title = _("エラー")
        #patchのチェック
        if len(patchNames) == 0:
            mess = _(u"layerを設定するpatchが選択されていません。")
            mess += _(u"patchを選択してください。")
            error = "NG"
            self.errDialog(title, mess)
            return error
        #setting（relative、angle）
        errFlag = 0
        mess = _("layer設定内容に数字以外の文字が入力されています。")
        try:
            angle = setting[5]
            float(angle)
        except:
            errFlag = 1
        if errFlag == 0:
            for setting in patchSetting:
                sets = setting[1]
                nums = [sets[0], sets[2], sets[3], sets[4]]
                try:
                    map(float, nums)
                except:
                    errFlag = 1
                    break
            if errFlag == 0:
                return ""
        mess = _("layer設定内容に数字以外の文字が入力されています。")
        error = "NG"
        self.errDialog(title, mess)
        return error

    #---- universalDialog --------
    def okDialog(self, title, mess):
        unvDlg.okDialog(self, title, mess)
    def errDialog(self, title, mess):
        unvDlg.errDialog(self, title, mess)

if __name__ == "__main__":
    import gettext
    gettext.install("treefoam", localeDir)
    #_ = gettext.gettext

