#!/usr/bin/python3
# coding: utf-8
#
#   createNewCaseDDialog.py
#
#       新しいcaseをtutorialsから選択する。
#
#       13/03/24    新規作成
#          04/18    casesを取得時に、subCaseがダブって表示されていたものを修正
#          04/20    tutorialsのsolver選択時、稀にエラーが発生していたのを修正
#          05/12    cases取得時、取得洩れがあったものを修正
#          10/15    国際化のため、修正
#          10/20    英語版のsolverListデータを準備
#       15/07/13    clipBoardの場所をTreeFoam/dataに変更
#          07/17    dialogの起動方法修正
#       16/12/31    iniSetTutorialsDir:OF-ext用に修正
#       19/09/14    python3, GTK+3に置き換えにより大幅修正
#       20/04/21    多言語化対応
#       22/02/12    iniSetTutorialsDir:tutorialsDirの取得方法を変更
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
import os
import sys
import glob

import pyTreeFoam
#from getOpenFolderDDialog import Show as getOpenFolderDDialogShow 
import getOpenFolderDDialog
import universalDialogs as unvDlg

import locale
localeDir = os.getenv("TreeFoamPath") + "/data/locale"
locale.bindtextdomain("treefoam", localeDir)

tutorials_data = os.getenv("TreeFoamUserPath") + "/data/tutorials_data"
tutorialsDir = ""
OFversion = ""
items = []
solvers = []
solverConts = []
cases = []
solverList = ""
selItemNo = 0
selSolverNo = 0
selCaseNo = 0


#
#  Show
#  ----
def Show(title, mess, caseDir):
    initialize()
    dialog = dialogApp(title, mess, caseDir)
    dialog.showDialog()
    selCaseCopyDir = dialog.selectDir
    status = dialog.status
    return [status, selCaseCopyDir]

#
#  initialize
#
def initialize():
    """ 初期化"""
    global solverList
    #環境に応じてsolverListファイルを設定する
    if os.getenv("LANG")[:2] == "ja":
        solverList = os.getenv("TreeFoamUserPath") + "/data/solverList.csv"
    else:
        solverList = os.environ["TreeFoamUserPath"] + "/data/solverList_en.csv"
    

#  ---------------
#  dialogApp class
#  ---------------
class dialogApp:
    """ tutorialsからcaseを選択して、dirを返す"""
    def __init__(self, dialogTitle, dialogMessage, caseDir):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("treefoam")
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path + "createNewCaseDDialog.glade")
        self.dialog = self.builder.get_object("dialog1")
        self.dialog.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        self.setGtkObject()
        #変数をセット
        self.dialogTitle = dialogTitle
        self.dialogMessage = dialogMessage
        self.status = "CANCEL"
        self.caseDir = caseDir
        self.seltDir = ""
        #tutorialsDirを取得
        iniSetTutorialsDir()
        #csvデータ（solverList）読み込み
        getSolverList()
        #tutorials内のsolver、caseを取得
        getItemsSolversCases()
        #GUIの初期化
        self.maskEvent = False
        self.subInitialize()


    #
    #  showDialog
    #  ----------
    def showDialog(self):
        """ dialogを表示する"""
        self.dialog.run()

    def close(self, *args):
        """ dialogのwindowを直接閉じた時"""
        self.selectDir = ""
        self.status = "CANCEL"
        self.dialog.destroy()

    #  ---------- event handler ----------------
    def onChangeDir(self, event):
        self.changeTutorialsDir()
    def onReference(self, event):
        self.getReferenceDir()
    def onChangeItem(self, event):
        if self.maskEvent == False:
            self.selectItemShowSolver()
    def onChangeSolver(self, event):
        if self.maskEvent == False:
            self.selectSolversShowCase()
    def onOk(self, event):
        self.selectDir = self.getTutoCaseDir()
        self.status = "OK"
        self.dialog.destroy()
    def onCancel(self, event):
        self.selectDir = ""
        self.status = "CANCEL"
        self.dialog.destroy()
        

    #---------------------------------------------

    def setGtkObject(self):
        """ GUIのid名を取得する"""
        self.label_conts = self.builder.get_object("label_conts")
        self.label_solverCont = self.builder.get_object("label_solverCont")
        self.check_changeDir = self.builder.get_object("check_changeDir")
        self.entry_tutorialsDir = self.builder.get_object("entry_tutorialsDir")
        self.button_reference = self.builder.get_object("button_reference")
        self.tree_items = self.builder.get_object("tree_items")
        self.tree_solvers = self.builder.get_object("tree_solvers")
        self.tree_cases = self.builder.get_object("tree_cases")
        

    def subInitialize(self):
        """ GUIの初期化"""
        #dialogのtitle、labelをセット
        self.dialog.set_title(self.dialogTitle)
        self.label_conts.set_text(self.dialogMessage)
        #textBox, 参照buttonを隠す
        self.entry_tutorialsDir.set_sensitive(False)
        self.button_reference.set_sensitive(False)
        self.entry_tutorialsDir.set_text(tutorialsDir)
        #listBoxにデータをセット
        self.setListBoxData()

    def setListBoxData(self):
        """ listBoxにデータをセット"""
        global selItemNo, selSolverNo, selCaseNo
        global items, solvers, cases, solverConts
        selItemNo = 0
        selSolverNo = 0
        selCaseNo = 0
        #itemsをセット
        self.tree_items = self.setDataToListBox(self.tree_items, items)
        #solversをセット
        self.tree_solvers = self.setDataToListBox(self.tree_solvers, solvers)
        #casesをセット
        self.tree_cases = self.setDataToListBox(self.tree_cases, cases)

    def setDataToListBox(self, listBox, items):
        """ listBox(TreeView)にデータをセットし、項目を選択する。
        実行は、eventをmaskして実行する。"""
        self.maskEvent = True       #event mask
        listmodel = Gtk.ListStore(str)
        for name in items:
            listmodel.append([name])
        listBox.set_model(model=listmodel)
        #columnを削除
        cols = listBox.get_columns()
        for _i in range(len(cols)):
            col = listBox.get_column(0)
            listBox.remove_column(col)
        #columnをappend
        cell = Gtk.CellRendererText()
        col = Gtk.TreeViewColumn("", cell, text=0)
        listBox.append_column(col)
        #treeViewのheaderを隠す
        listBox.set_headers_visible(False)
        if len(items) == 0:
            #mask解除して戻る
            self.maskEvent = False
            return listBox
        #第1番目を選択する
        path = Gtk.TreePath.new_from_indices([0])
        listBox.get_selection().select_path(path)
        #mask解除して戻る
        self.maskEvent = False
        return listBox

    def changeTutorialsDir(self):
        if self.check_changeDir.get_active() == True:
            self.entry_tutorialsDir.set_sensitive(True)
            self.button_reference.set_sensitive(True)
        else:
            self.entry_tutorialsDir.set_sensitive(False)
            self.button_reference.set_sensitive(False)

    def getReferenceDir(self):
        """ 他のtutorialsDirを取得する"""
        global tutorialsDir
        currDir = self.caseDir
        selDir = tutorialsDir
        title = _("tutorialsDirの設定")
        mess = _("使用するTutorialsのdirを選択してください") + "\n"
        mess += _("  変更したTutorialsDirは、保存されます")
        #rootDir = pyTreeFoam.readConfigTreeFoam()["rootDir"]
        #dialog = getOpenFolderDDialog.dialogApp(title, mess, rootDir, currDir, selDir)
        #newTutoDir = dialog.showDialog()
        newTutoDir = unvDlg.folderDialog(self, title, selDir)
        if newTutoDir != "":
            tutorialsDir = newTutoDir
            self.entry_tutorialsDir.set_text(tutorialsDir)
            writeTutorials_data(OFversion, tutorialsDir)
            print(_("tutorialsDirを変更し、保存しました。"))
            print(_("次回起動時に反映されます。"))
            getItemsSolversCases()
            self.setListBoxData()            

    def selectItemShowSolver(self):
        """ itemを選択した時、solversとcasesを再表示する"""
        global items, solvers, selItemNo, selSolverNo, selCaseNo
        if len(items) == 0:
            return
        _model, pathes = self.tree_items.get_selection().get_selected_rows()
        if len(pathes) == 0:
            return
        [selItemNo] = pathes[0].get_indices()
        item = items[selItemNo]
        #solversをセット
        selSolverNo = 0
        solvers = getSolverNames(item)
        self.tree_solvers = self.setDataToListBox(self.tree_solvers, solvers)
        self.label_solverCont.set_text("")
        if len(solvers) == 0:
            self.tree_cases = self.setDataToListBox(self.tree_cases, [])
            return
        solverCont = self.getSolverContents(solvers[selSolverNo])
        self.label_solverCont.set_text(solverCont)
        #casesをセット
        selCaseNo = 0
        self.selectSolversShowCase()

    def selectSolversShowCase(self):
        """ solverを選択した時、casesを再表示する"""
        global items, solvers, cases, selSolverNo, selCaseNo
        if len(items) == 0:
            return
        item = items[selItemNo]
        itemName = getItemName(item)
        if len(solvers) == 0:
            self.tree_solvers = self.setDataToListBox(self.tree_solvers, [])
            self.label_solverCont.set_text("")
            self.tree_cases = self.setDataToListBox(self.tree_cases, [])
            return
        _model, pathes = self.tree_solvers.get_selection().get_selected_rows()
        if len(pathes) == 0:
            return

        [selSolverNo] = pathes[0].get_indices()
        solver = solvers[selSolverNo]
        #solver内容をセット
        solverCont = self.getSolverContents(solver)
        self.label_solverCont.set_text(solverCont)
        #caseを取得
        cases = getCaseNames(itemName, solver)
        self.tree_cases = self.setDataToListBox(self.tree_cases, cases)
        selCaseNo = 0

    def getSolverContents(self, solver):
        """ solverの内容をを取得"""
        global listData
        cont = ""
        for i in range(len(listData)):
            names = listData[i]
            if names[1] == solver:
                cont = listData[i][2]
                break
        return cont

    def getTutoCaseDir(self):
        """ 選択しているTutorialsのcaseDirを取得"""
        global selCaseNo
        item = items[selItemNo]
        solver = solvers[selSolverNo]
        _model, pathes = self.tree_cases.get_selection().get_selected_rows()
        [selCaseNo] = pathes[0].get_indices()
        caseName = cases[selCaseNo]
        ansDir = getCaseDir(item, solver, caseName)
        return ansDir

#
#  iniSetTutorialsDir
#
def iniSetTutorialsDir():
    """ tutorialsDirを取得して、tutorials_dataファイルに保存する"""
    global tutorialsDir, OFversion
    configDict = pyTreeFoam.readConfigTreeFoam()
    OFversion = configDict["OFversion"]
    if len(glob.glob(tutorials_data)) != 0:
        [ver, tutoDir] = readTutorials_data()
        if (len(glob.glob(tutoDir)) != 0) and (ver == OFversion):
            tutorialsDir = tutoDir
            return
    #tutorialsDirを取得
    tutorialsDir = ""
    if OFversion[:3] == "ext":
        #foam-extend
        ver = OFversion[4:]
        tutoDir = os.getenv("HOME") + "/foam/" + os.getenv("USER") + "-" + ver + "/run/tutorials"
        if len(glob.glob(tutoDir)) == 0:
            tutoDir = os.getenv("HOME") + "/foam/foam-extend-" + ver + "/tutorials"
        tutorialsDir = tutoDir
    else:
        #OpenFOAM
        tutoDir = os.getenv("HOME") + "/OpenFOAM/" + os.getenv("USER") + "-" + OFversion + "/run/tutorials"
        if len(glob.glob(tutoDir)) == 0:
            tutoDir = os.getenv("HOME") + "/OpenFOAM/OpenFOAM-" + OFversion + "/tutorials"
        tutorialsDir = tutoDir
    if len(glob.glob(tutorialsDir)) == 0:
        #環境変数からdirを取得
        bashrcFOAM = configDict["bashrcFOAM"]
        comm = ". " + bashrcFOAM + "; "
        comm += "echo $WM_PROJECT_DIR"
        _stat, res, _err = pyTreeFoam.run().commandReturnCont(comm)
        resLines = res.split("\n")
        if resLines[-1] != "":
            tutorialsDir = resLines[-1] + "/tutorials"
        else:
            tutorialsDir = resLines[-2] + "/tutorials"
        # fileName = os.getenv("TreeFoamUserPath") + "/temp/run"
        # bashrcFOAM = configDict["bashrcFOAM"]
        # comm = ". " + bashrcFOAM + "; "
        # comm += "echo $WM_PROJECT_DIR > " + fileName
        # pyTreeFoam.run().command(comm)
        # f = open(fileName); cont = f.read(); f.close()
        # tutorialsDir = cont.split()[0] + "/tutorials"
    #ファイルに書き込み
    writeTutorials_data(OFversion, tutorialsDir)
    return

#
#  readTutorials_data
#
def readTutorials_data():
    """ OFversionとtutorialsDirをファイルから読み込み"""
    f=open(tutorials_data); cont=f.readlines(); f.close()
    ver = ""
    tutoDir = ""
    for line in cont:
        comms = line.split()
        if len(comms) > 1:
            item = comms[0]
            data = comms[1]
            if item == "OFversion":
                ver = data
            elif item == "tutorialsDir":
                tutoDir = data
    return [ver, tutoDir] 

#
#  writeTutorials_data
#
def writeTutorials_data(ver, tutoDir):
    """ tutorials_dataの書き込み"""
    cont  = "\n"
    cont += "OFversion    " + ver + "\n"
    cont += "tutorialsDir " + tutoDir + "\n"
    f=open(tutorials_data, "w")
    f.write(cont)
    f.close()

#
#  getSolverList
#
def getSolverList():
    global listData
    fileName = solverList
    solverConts = pyTreeFoam.readCsvData(fileName)
    listData = solverConts[1:]

#
#  getItemName
#
def getItemName(itemCont):
    """ 区分からitemを取得する"""
    item = itemCont.split(":")[0]
    if len(item) == len(itemCont):
        item = itemCont.split("：")[0]
    return item

#
#  getSolverNames
#
def getSolverNames(item):
    """ itemに該当するsolver名を取得（nestingは2段まで）"""
    itemName = getItemName(item)
    solverList = glob.glob(tutorialsDir + "/" + itemName + "/*")
    solvers = []
    if len(solverList) > 0:
        # solverを取得
        for solverDir in solverList:
            name = solverDir.split("/")[-1]
            if name[-4:] == "Foam" or name[-4:] == "Mesh":
                solvers.append(name)
            else:
                subList = glob.glob(solverDir + "/*")
                if len(subList) > 0:
                    for subDir in subList:
                        name = subDir.split("/")[-1]
                        if name[-4:] == "Foam" or name[-4:] == "Mesh":
                            solvers.append(name)
        solvers.sort()
    return solvers

#
#  getSolverDir
#
def getSolverDir(item, solver):
    """ itemとsolverNameからsolverDirを取得"""
    itemName = getItemName(item)
    checkDir = tutorialsDir + "/" + itemName
    solverDir = walkGetFolderDir(checkDir, solver)
    return solverDir

#
#  getCaseDir
#
def getCaseDir(item, solver, case):
    """ tutorialsのcaseDirを取得"""
    solverDir = getSolverDir(item, solver)
    caseDir = solverDir + "/" + case
    return caseDir

#
#  walkGetFolderDir
#
def walkGetFolderDir(rootDir, folder):
    """ rootDirからfolderを探し、dirを返す"""
    folderDir = ""
    for (root, dirs, _files) in os.walk(rootDir):
        for dirName in dirs:
            name = dirName.split("/")[-1]
            if name == folder:
                folderDir = root + "/" + dirName
                break
        if folderDir != "":
            break
    return folderDir

#
#  getCaseNames
#
def getCaseNames(item, solver):
    """ solverに該当するcase名を取得"""
    #solverDirの取得
    solverDir = getSolverDir(item, solver)
    #caseを取得
    #  3レベルまで探す
    caseNames = []
    folders = pyTreeFoam.getFolders(solverDir)
    for checkDir in folders:
        if pyTreeFoam.getFolderType(checkDir) == "case":
            folder = checkDir.split("/")[-1]
            caseNames.append(folder)
        else:
            folders1 = pyTreeFoam.getFolders(checkDir)
            for checkDir1 in folders1:
                if pyTreeFoam.getFolderType(checkDir1) == "case":
                    folder = "/".join(checkDir1.split("/")[-2:])
                    caseNames.append(folder)
                else:
                    folders2 = pyTreeFoam.getFolders(checkDir1)
                    for checkDir2 in folders2:
                        if pyTreeFoam.getFolderType(checkDir2) == "case":
                            folder = "/".join(checkDir2.split("/")[-3:])
                            caseNames.append(folder)
    caseNames.sort()
    return caseNames

#
#  getItemsSolversCases
#
def getItemsSolversCases():
    """ tutorials内から項目、solver、caseを取得する"""
    global items, solvers, solverConts, cases
    # csvデータ内の区分を取得
    setData = ""
    readItems = []
    for data in listData:
        if setData != data[0]:
            readItems.append(data[0])
            setData = data[0]
    # tutorials内の区分を取得
    itemNames = []
    if len(glob.glob(tutorialsDir)) != 0:
        itemNames = pyTreeFoam.getFolderNames(tutorialsDir)
    subItems = []
    for gName in itemNames:
        flag = 0
        for i in range(len(readItems)):
            item = readItems[i]
            iName = getItemName(item)
            if gName == iName:
                subItems.append([i, item])
                flag = 1
                break
        if flag == 0:
            subItems.append([100, gName])
    # itemsをセット
    subItems.sort()
    items = []
    for item in subItems:
        items.append(item[1])
    if len(items) != 0:
        # solver, 内容を取得
        solvers = getSolverNames(items[0])
        solverConts = []
        for solver in solvers:
            cont = ""
            for data in listData:
                if solver == data[1]:
                    cont = data[2]
                    break
            solverConts.append(cont)
        # caseを取得
        if len(solvers) != 0:
            cases = getCaseNames(items[0], solvers[0])
    else:
        solvers = []
        cases = []
        solverConts = []


if __name__ == "__main__":
    import gettext
    gettext.install("treefoam", localeDir)
    #_ = gettext.gettext

    os.environ["TreeFoamPath"] = "/home/caeuser/TreeFoam-glade3"
    app = dialogApp("title", "mmesage", ".")
    app.showDialog()
