#!/usr/bin/python3
# coding: utf-8
#
#   getOpenFilesDDialog.py
#
#       file名の取得
#
#   13/02/09    新規作成
#      06/02    Dクリックでfileがopenするを追加
#      07/13    複数fileの選択を追加
#      10/15    国際化による修正
#   15/07/17    dialogの起動方法を修正
#   19/10/16    GTK+3, python3用に大幅修正
#   20/04/21    多言語化対応
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
import os, sys
import glob

import locale
localeDir = os.getenv("TreeFoamPath") + "/data/locale"
locale.bindtextdomain("treefoam", localeDir)

#-------------------
#  dialogApp class
#-------------------
class dialogApp:

    def __init__(self, title, message, currDir, fileFilter, seledFiles):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("treefoam")
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path + "getOpenFilesDDialog.glade")
        self.dialog = self.builder.get_object("dialog1")
        self.dialog.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        self.setGtkObject()
        #変数の設定
        self.title = title
        self.message = message
        self.currDir = currDir
        self.fileFilter = fileFilter
        self.seledFiles = seledFiles
        self.selFiles = []
        #初期化
        self.initialize()

    #
    #  show
    #  ----
    def show(self):
        self.dialog.run()
        return self.selFiles

    def close(self, *args):
        """ dialogを直接閉じた時のevent"""
        self.dialog.destroy()

    def setGtkObject(self):
        """ GUIのobject名を取得する"""
        self.label_selCont = self.builder.get_object("label_selCont")
        self.entry_dir = self.builder.get_object("entry_dir")
        self.entry_filter = self.builder.get_object("entry_filter")
        self.treeView = self.builder.get_object("treeView")
        self.button_reference = self.builder.get_object("button_reference")

    def initialize(self):
        """ 初期化"""
        self.dialog.set_title(self.title)
        self.label_selCont.set_label(self.message)
        self.entry_dir.set_text(self.currDir)
        self.entry_filter.set_text(self.fileFilter)
        self.entry_dir.set_sensitive(False)
        self.button_reference.set_sensitive(False)
        #fileListを作成
        self.setFilesInListBox()

    def setFilesInListBox(self):
        """ 該当dir内のfileを表示する"""
        files = glob.glob(self.currDir + os.sep + self.fileFilter)
        fileNames = list(map(lambda x: x.split(os.sep)[-1], files))
        fileNames.sort()
        #treeViewのcolumnを定義し、セット
        listSelection = self.treeView.get_selection()
        listSelection.set_mode(Gtk.SelectionMode.MULTIPLE)
        cellRenderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn()
        column.pack_start(cellRenderer, False)
        column.add_attribute(cellRenderer, "text", 0)
        self.treeView.set_headers_visible(False)
        self.treeView.append_column(column)
        #listDataをセット
        listStore = Gtk.ListStore(str)
        for name in fileNames:
            listStore.append([name])
        self.treeView.set_model(listStore)
        #seledFilesを選択する
        #listSelection = self.treeView.get_selection()
        #listStore = self.treeView.get_model()
        for i in range(len(listStore)):
            item = list(listStore)[i]
            if item[0] in self.seledFiles:
                path = Gtk.TreePath(i)
                iter = listStore.get_iter(path)
                listSelection.select_iter(iter)
        


    #------- event handler ------------
    def onReference(self, event):
        print("----reference")
    def onShowFiles(self, event):
        self.showFiles()
    def onOk(self, event):
        self.selFiles = self.getSelection()
        self.close()
    def onCancel(self, event):
        self.selFiles = []
        self.close()

    #
    #  showFiles
    #  ---------
    def showFiles(self):
        """ fileFilterで再度filesを取得し、表示する"""
        self.fileFilter = self.entry_filter.get_text()
        files = glob.glob(self.currDir + os.sep + self.fileFilter)
        fileNames = list(map(lambda x: x.split(os.sep)[-1], files))
        fileNames.sort()
        #listDataをセット
        listStore = Gtk.ListStore(str)
        for name in fileNames:
            listStore.append([name])
        self.treeView.set_model(listStore)
        #seledFilesを選択する
        listSelection = self.treeView.get_selection()
        #listStore = self.treeView.get_model()
        for i in range(len(listStore)):
            item = list(listStore)[i]
            if item[0] in self.seledFiles:
                path = Gtk.TreePath(i)
                iter = listStore.get_iter(path)
                listSelection.select_iter(iter)

    #
    #  getSelection
    #  ------------
    def getSelection(self):
        """ 選択しているfileを取得して返す"""
        selFiles = []
        listSelection = self.treeView.get_selection()
        (listStore, pathes) = listSelection.get_selected_rows()
        for path in pathes:
            iter = listStore.get_iter(path)
            name = listStore.get_value(iter, 0)
            selFiles.append(name)
        return selFiles
