#!/usr/bin/python3
# coding: utf-8
#
#       inputTextDialog.py
#
#       テキストデータの入力dialog
#
#   13/10/15    国際化のため、修正
#   15/07/17    dialogの起動方法を修正
#   16/05/28    wx.EVT_KEY_DOWNを追加
#   19/10/15    GTK+3, python3用に大幅修正
#   20/04/21    多言語化対応
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
import os, sys

import locale
localeDir = os.getenv("TreeFoamPath") + "/data/locale"
locale.bindtextdomain("treefoam", localeDir)

#-------------------
#  dialogApp class
#-------------------
class dialogApp:
    """ テキスト入力データを取得する。
    
    Attribute:
        title (str)     :dialogのtitle
        message (str)   :出力するmessage
        entryText (str) :entryにdefaultで出力させる内容
        stat (str)      :"OK" or "CANCEL"
    Retrns:
        entryText (str) :entryの内容を戻す。
                        :"CANCEL"の場合は、「""」が戻る。
    """

    def __init__(self, title, message, entryText):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("treefoam")
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path + "inputTextDDialog.glade")
        self.dialog = self.builder.get_object("dialog1")
        self.dialog.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        self.setGtkObject()
        #変数の設定
        self.title = title
        self.message = message
        self.entryText = entryText
        self.stat = "CANCEL"
        #初期化
        self.initialize()

    #
    #  show
    #  ----
    def show(self):
        """ dialogを表示、entryのtextを戻す"""
        self.dialog.run()
        if self.stat != "OK":
            return ""
        else:
            return self.entryText

    def close(self, *args):
        """ dialogを直接閉じた時のevent"""
        self.dialog.destroy()

    def setGtkObject(self):
        """ GUIのobject名を取得する"""
        self.label_mess = self.builder.get_object("label_message")
        self.entry = self.builder.get_object("entry_text")

    def initialize(self):
        self.dialog.set_title(self.title)
        self.label_mess.set_label(self.message)
        self.entry.set_text(self.entryText)

    def onOk(self, event):
        self.stat = "OK"
        self.entryText = self.entry.get_text()
        self.close()

    def onCancel(self, event):
        self.stat = "CANCEL"
        self.entryText = ""
        self.close()

    def onEntryActive(self, event):
        self.onOk(event)
