#!/user/bin/python3
# coding: utf-8
#
#       mountServerDDialog.py
#
#       sshfsコマンドでマウントするサーバを選択する。
#
#   17/06/20    新規作成
#      06/30    textBoxを追加
#   19/10/31    GTK+3, python3用に大幅修正
#   20/04/22    多言語化対応
# 

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
import os, sys

try:
    import pyTreeFoam
except:
    import python.pyTreeFoam as pyTreeFoam

import locale
localeDir = os.getenv("TreeFoamPath") + "/data/locale"
locale.bindtextdomain("treefoam", localeDir)

import logFileCreater
logFileCreater.log()
 

#-------------------
#  dialogApp class
#-------------------
class dialogApp:
    """ sshfsコマンドでマウントするサーバを選択する。
    
    Attribute:
        serverNames (list(str)) :マウントできるserver名
    Returns:
        serverName (str)        :選択したserver名 or 「""」"""
    def __init__(self, serverNames):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("treefoam")
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path + "mountServerDDialog.glade")
        self.dialog = self.builder.get_object("dialog1")
        self.dialog.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        self.setGtkObject()
        #変数の設定
        self.serverNames = serverNames
        self.selServer = serverNames[0]
        self.stat = "CANCEL"
        #serverのデータを取得
        server = pyTreeFoam.server(self.selServer)
        self.sshfsDict = server.sshfsDict
        self.sshDict = server.sshDict
        #comboにデータセット
        self.setDataToCombo()
        #entryにデータセット
        self.setDataToEntry()
        #textViewにデータセット
        self.setDataToTextView()

    def setGtkObject(self):
        """ GUIのobject名を取得する"""
        self.combo_server = self.builder.get_object("combo_server")
        self.entry_hostName = self.builder.get_object("entry_hostName")
        self.entry_hostDir = self.builder.get_object("entry_hostDir")
        self.textView_setEnv = self.builder.get_object("textView_setEnviron")

    #
    #  show
    #  ----
    def show(self):
        """ dialogの表示"""
        self.dialog.run()
        if self.stat != "OK":
            return ""
        else:
            return self.selServer

    #
    #  close
    def close(self, *args):
        self.dialog.destroy()

    
    #-------- event handler -----------
    def onChangeServer(self, event):
        self.changeServer()
    def onOk(self, event):
        self.stat = "OK"
        self.setServerContData()
        self.close()
    def onCancel(self, event):
        self.stat = "CANCEL"
        self.close()
    #-----------------------------------

    #
    #  setDataToCombo
    #  --------------
    def setDataToCombo(self):
        self.combo_server.clear()
        listModel = Gtk.ListStore(str)
        for name in self.serverNames:
            listModel.append([name])
        cell = Gtk.CellRendererText()
        self.combo_server.pack_start(cell, True)
        self.combo_server.add_attribute(cell, "text", 0)
        self.combo_server.set_model(listModel)
        self.combo_server.set_entry_text_column(0)
        self.combo_server.set_active(0)

    #
    #  setDataToEntry
    #  --------------
    def setDataToEntry(self):
        hostName = self.sshfsDict["hostName"]
        self.entry_hostName.set_text(hostName)
        hostDir = self.sshfsDict["hostDir"]
        self.entry_hostDir.set_text(hostDir)

    #
    #  setDataToTextView
    #  -----------------
    def setDataToTextView(self):
        setEnv = self.sshfsDict["setEnviron"]
        buffer = self.textView_setEnv.get_buffer()
        buffer.set_text(setEnv)

    #
    #  changeServer
    #  ------------
    def changeServer(self):
        iter = self.combo_server.get_active_iter()
        if iter == None:
            return
        listModel = self.combo_server.get_model()
        self.selServer = listModel.get_value(iter, 0)
        server = pyTreeFoam.server(self.selServer)
        self.sshfsDict = server.sshfsDict
        self.sshdict = server.sshDict
        self.setDataToEntry()
        self.setDataToTextView()

    #
    #  setServerContData
    #  -----------------
    def setServerContData(self):
        entry = self.combo_server.get_child()
        self.selServer = entry.get_text()
        hostName = self.entry_hostName.get_text()
        hostDir = self.entry_hostDir.get_text()
        buffer = self.textView_setEnv.get_buffer()
        st = buffer.get_start_iter()
        ed = buffer.get_end_iter()
        setEnv = buffer.get_text(st, ed, True)
        self.sshfsDict["hostName"] = hostName
        self.sshfsDict["hostDir"] = hostDir
        self.sshfsDict["setEnviron"] = setEnv
        server = pyTreeFoam.server(self.selServer)
        server.sshfsDict = self.sshfsDict
        server.writeSshfsData()
