#!/usr/bin/python3
# coding: utf-8
#
#   runParaFoamOptionDialog
#
#       paraFoamをoption付きで起動するかどうかを確認して起動する。
#
#   14/07/13    新規作成
#      07/27    ラジオボタンに変更
#   15/07/17    dialogの起動方法を修正
#      07/23    radioボタンの設定を常時「option無し」に設定
#   19/09/05    GTK+3, python3用に書き換え
#   20/04/22    多言語化対応
#      06/11    if __name__ == "__main__" 以下を追加
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
import os
import sys

import locale
localeDir = os.getenv("TreeFoamPath") + "/data/locale"
locale.bindtextdomain("treefoam", localeDir)

regFlag = False         #mulRegionの場合、True

#
#  Show
#
def Show(isRegion):
    global regFlag
    regFlag = isRegion          #multiRegionのflag
    dialog = dialogApp()
    dialog.main()
    status = dialog.status
    option = ""
    if dialog.radioParaFoamOption.get_active() == True:
        option = "-builtin"
    return (status, option)

#
#  dialogApp class
#
class dialogApp(Gtk.Dialog):

    def __init__(self, *argv, **kwds):
        Gtk.Dialog.__init__(self, **kwds)
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("treefoam")
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path + "runParaFoamOptionDialog.glade")
        self.dialogApp = self.builder.get_object("dialog1")
        self.dialogApp.connect("delete-event", self.closeCancel)
        self.builder.connect_signals(self)
        self.dialogApp.set_transient_for(self)
        self.setGtkObject()
        self.status = "CANCEL"
        mess = _("paraFoamの起動方法（option）を指定してください。")
        if regFlag == True:
            mess += "\n\n" 
            mess += _("このcaseは、multiRegionのcaseです。\n")
            mess += _("  「-builtin」optionを選択するか\n")
            mess += _("  paraFoamにマクロを追加してください。")
            self.radioParaFoamOption.set_active(True)
        else:
            self.radioParaFoam.set_active(True)
        self.labelCont.set_label(mess)

    def main(self):
        self.dialogApp.show()
        Gtk.main()

    def closeCancel(self, *args):
        self.status = "CANCEL"
        self.dialogApp.destroy()
        Gtk.main_quit()

    def setGtkObject(self):
        self.labelCont = self.builder.get_object("labelCont")
        self.radioParaFoam = self.builder.get_object("radioParaFoam")
        self.radioParaFoamOption = self.builder.get_object("radioParaFoamBuiltin")

    def onOK(self, event):
        self.dialogApp.destroy()
        self.status = "OK"
        Gtk.main_quit()

    def onCancel(self, event):
        self.dialogApp.destroy()
        self.status = "CANCEL"
        Gtk.main_quit()

if __name__ == "__main__":
    import gettext
    gettext.install("treefoam", localeDir)
    #_ = gettext.gettext

    paraFoamFix = sys.argv[1]
    caseDir = sys.argv[2]
    (stat, option) = Show(False)
    if stat == "OK":
        comm = "cd " + caseDir + ";" + paraFoamFix + " " + option + " &"
        os.system(comm)
