#!/usr/bin/python3
# coding: utf-8
#
#   separateCellZoneGroupDialog.py
#
#       cellZoneを流体と固体に分ける
#
#   11/04/18    FrameからDialogに置き換え
#      04/21    変数(liquidCellZones, solidCellZones)の初期化追加
#   12/03/18    アイコンの読み込み場所変更
#   13/10/12    多言語化の為、修正
#   15/07/17    dialogの起動方法を修正
#   19/10/24    GTK+3, python3用に大幅修正
#   20/04/22    多言語化対応    
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

import os, sys

import locale
localeDir = os.getenv("TreeFoamPath") + "/data/locale"
locale.bindtextdomain("treefoam", localeDir)

#-------------------
#  dialogApp class
#-------------------
class dialogApp:
    """ cellZoneを流体と固体に分ける"""

    def __init__(self, zoneNames):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("treefoam")
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path + "separateCellZoneGroupDDialog.glade")
        self.dialog = self.builder.get_object("dialog1")
        self.dialog.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        self.setGtkObject()
        #変数の設定
        self.zoneNames = zoneNames
        self.fluidSolid = None
        self.fluidNames = []
        self.solidNames = []
        #GUIの初期化
        self.iniGui()

    def setGtkObject(self):
        self.tree_all = self.builder.get_object("tree_all")
        self.tree_fluid = self.builder.get_object("tree_fluid")
        self.tree_solid = self.builder.get_object("tree_solid")
    
    #
    #  show
    #  ----
    def show(self):
        """ dialogを表示する"""
        self.dialog.run()
        return self.fluidSolid

    def close(self, *args):
        """ dialogを閉じる"""
        self.dialog.destroy()

    #----event handler -----------
    #選択>> fluid
    def onSelectFluid(self, event):
        self.selectFluid()
    #<<戻す fluid
    def onReturnFluid(self, event):
        self.returnFluid()
    #選択>> solid
    def onSelectSolid(self, event):
        self.selectSolid()
    #<<戻す solid
    def onReturnSolid(self, event):
        self.returnSolid()
    #OK
    def onOk(self, event):
        self.getFluidSolidNames()
        self.close()
    #キャンセル
    def onCancel(self, event):
        self.fluidSolid = None
        self.close()

    #
    #  iniGui
    def iniGui(self):
        """ GUIの初期化
        treeViewの設定と表示"""
        #selectionModeを設定
        self.tree_all.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
        self.tree_fluid.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
        self.tree_solid.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
        #title行を非表示
        self.tree_all.set_headers_visible(False)
        self.tree_fluid.set_headers_visible(False)
        self.tree_solid.set_headers_visible(False)
        #treeColumnの設定
        cellRenderer = Gtk.CellRendererText()
        colAll = Gtk.TreeViewColumn()
        colAll.pack_start(cellRenderer, True)
        colAll.add_attribute(cellRenderer, "text", 0)
        self.tree_all.append_column(colAll)
        colFluid = Gtk.TreeViewColumn()
        colFluid.pack_start(cellRenderer, True)
        colFluid.add_attribute(cellRenderer, "text", 0)
        self.tree_fluid.append_column(colFluid)
        colSolid = Gtk.TreeViewColumn()
        colSolid.pack_start(cellRenderer, True)
        colSolid.add_attribute(cellRenderer, "text", 0)
        self.tree_solid.append_column(colSolid)
        #tree表示
        self.setListDataToBox(self.tree_all, self.zoneNames)

    #
    #  setListDataToBox
    def setListDataToBox(self, treeList, names):
        """ treeListにdataをセットし、dataを表示する"""
        listStore = Gtk.ListStore(str)
        names.sort()
        for name in names:
            listStore.append([name])
        treeList.set_model(listStore)

    #
    #  getSelectNames
    def getSelectNames(self, treeList):
        """ 指定されたtreeListから選択したnameを取得する"""
        listSelection = treeList.get_selection()
        selNames = []
        (listStore, pathes) = listSelection.get_selected_rows()
        for path in pathes:
            iter = listStore.get_iter(path)
            name = listStore.get_value(iter, 0)
            selNames.append(name)
        return selNames

    #
    #  selectZoneNames
    def selectZoneNames(self, treeList, selNames):
        """ 指定されたtreeList内から指定したzone名を選択表示する"""
        listSelection = treeList.get_selection()
        listStore = treeList.get_model()
        for i in range(len(listStore)):
            path = Gtk.TreePath(i)
            iter = listStore.get_iter(path)
            name = listStore.get_value(iter, 0)
            if name in selNames:
                listSelection.select_iter(iter)

    #
    #  deleteNamesFromBox
    def deleteNamesFromBox(self, treeList, delNames):
        """ 指定されたtreeListから項目を削除する"""
        allNames = []
        listStore = treeList.get_model()
        if listStore != None:
            for i in range(len(listStore)):
                path = Gtk.TreePath(i)
                iter = listStore.get_iter(path)
                name = listStore.get_value(iter, 0)
                allNames.append(name)
            remNames = list(set(allNames).difference(delNames))
        else:
            remNames = []
        self.setListDataToBox(treeList, remNames)
    
    #
    #  addNamesToBox
    def addNamesToBox(self, treeList, addNames):
        """ 指定されたtreeListに項目を追加する"""
        allNames = []
        listStore = treeList.get_model()
        if listStore != None:
            for i in range(len(listStore)):
                path = Gtk.TreePath(i)
                iter = listStore.get_iter(path)
                name = listStore.get_value(iter, 0)
                allNames.append(name)
        newNames = allNames + addNames
        self.setListDataToBox(treeList, newNames)
        self.selectZoneNames(treeList, addNames)

    #
    #  selectFluid
    #  -----------
    def selectFluid(self):
        """ 全zonesの選択zoneをflud側へ移動する"""
        selNames = self.getSelectNames(self.tree_all)
        if len(selNames) > 0:
            self.deleteNamesFromBox(self.tree_all, selNames)
            self.addNamesToBox(self.tree_fluid, selNames)

    #
    #  returnFluid
    #  -----------
    def returnFluid(self):
        """ fluid側のzone名を全zone側に戻す"""
        selNames = self.getSelectNames(self.tree_fluid)
        if len(selNames) > 0:
            self.deleteNamesFromBox(self.tree_fluid, selNames)
            self.addNamesToBox(self.tree_all, selNames)

    #
    #  selectSolid
    #  -----------
    def selectSolid(self):
        """ 全zonesの選択zoneをsolid側へ移動する"""
        selNames = self.getSelectNames(self.tree_all)
        if len(selNames) > 0:
            self.deleteNamesFromBox(self.tree_all, selNames)
            self.addNamesToBox(self.tree_solid, selNames)

    #
    #  returnSolid
    #  -----------
    def returnSolid(self):
        """ solid側のzone名を全zone側に戻す"""
        selNames = self.getSelectNames(self.tree_solid)
        if len(selNames) > 0:
            self.deleteNamesFromBox(self.tree_solid, selNames)
            self.addNamesToBox(self.tree_all, selNames)

    #
    #  getFluidSolidNames
    #  ------------------
    def getFluidSolidNames(self):
        """ fluid, solidの内容を取得する"""
        #fluid側を取得
        fluidNames = []
        listStore = self.tree_fluid.get_model()
        if listStore != None:
            for i in range(len(listStore)):
                path = Gtk.TreePath(i)
                iter = listStore.get_iter(path)
                name = listStore.get_value(iter, 0)
                fluidNames.append(name)
        #solid側を取得
        solidNames = []
        listStore = self.tree_solid.get_model()
        if listStore != None:
            for i in range(len(listStore)):
                path = Gtk.TreePath(i)
                iter = listStore.get_iter(path)
                name = listStore.get_value(iter, 0)
                solidNames.append(name)
        self.fluidNames = fluidNames
        self.solidNames = solidNames
        self.fluidSolid = [fluidNames, solidNames]
