#!/usr/bin/python3
# coding: utf-8
#
#   snappyHexMeshDialog.py
#
#       snappyHexMeshDict作成、実行。
#
#   13/12/08    新規作成
#      12/21    stdout、stderrの設定（import logFileCreater）を追加
#   14/01/04    バグ修正（dictファイルが無かった場合エラー発生）
#      02/03    csv作成後、csv編集するように修正
#      03/17    checkBlockMeshSnappyHexMesh:デフォルトのblockMeshDictの場所を修正
#      04/12    runFeatureEdge:triSurfaceﾌｫﾙﾀﾞが無かった場合、ｴﾗｰ発生を修正
#               runSnappy_sub:   同上
#      06/04    changePatchName:空patch削除を追加
#               getStlDir:stlDirを保存する様に修正
#      07/12    バグ修正（圧縮時、eMesh.gzを認識しなかった）
#      08/31    layer作成ルーチンを追加
#      09/08    runLayer:snappy実行前にcellLevel、pointLevelを削除
#      12/23    checkBlockMeshSnappyHexMesh:polyMeshフォルダが無い場合作成する
#   15/02/08    runSnappy_sub:mesh作成を端末表示させて実行する様に修正
#      07/17    dialogの起動方法を修正
#      10/02    checkMeshボタン追加
#      11/28    OF-3.0.0用に修正。
#               gnome-terminalの「-t」オプションを「--name=」に変更
#               defaultのblockMeshDictの場所を「system」フォルダに変更
#      11/30    checkBlockMeshSnappyHexMesh:dictFileのコピー元dir取得方法変更。
#      12/12    constant内のblockMeshDictは、削除する様に修正。
#   16/06/02    runLayer:layer追加時、cellLevel、pointLevelを削除しない様修正
#      08/29    snappyの並列処理を追加
#   17/03/15    runLayer:layer作成にも並列処理を追加
#      08/04    getAbsPath:相対参照のfullpathの取得方法を修正
#   18/06/05    gnome-terminalのオプション「-x」を「--」に変更
#   19/02/01    checkBlockMeshSnappyHexMesh:blokMeshDictを無条件に入れ替えする
#               様に修正。
#      02/07    runFeatureEdge:eMeshファイルをtrisurfaceフォルダからstlフォルダ
#               に戻す時、stlに対応するeMeshのみ戻す様に修正
#      05/22    runFeatureEdge,copyFileToTriSurface:mkdirをos.mkdirに修正
#               cpをshutil.copyに修正
#      10/11    GTK+3, python3用に大幅修正
#   20/02/10    stringOpをpyTreeFoam.strOpに変更
#      04/22    多言語化対応
#      05/28    remakeDecomposeParDict:simpleCoeffesに「delta 0.001;」を追加
#               decomposeParでエラーが発生する為。（OF-7から）
#      11/22    setSelLocation:バグ修正。
#               stlDirを変更した時、comboBoxの内容が更新されていなかった。
#   21/05/31    OF-8対応。surfaceFeatureExtractが無くなりsurfaceFeaturesに
#               に変わり、Dictの書式も変わったのでこれに対応。               
#      12/29    meshViewer起動ボタンを追加。
#   22/01/09    runSnappy_sub,runLayer:OF-9対応
#               OF-9からreconstructParMeshコマンドが変更された為。
#      01/24    remakeDecomposeParDict:v2106対応として、"coeffs"の定義を追加
#      02/01    setSelLocation:stlDirをそのまま設定する様に修正
#      07/17    checkBlockMeshSnappyHexMesh:OFversion毎のcheckを修正（OF-10対応）
#               createReconstructMeshCommLine:OFversion毎のcheckを修正（OF-10対応）
#      08/15    runMeshViewer:zones, sets, patchesの指定をload allに変更。
#               stlが表示されなかった為。
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
import os, sys
import glob
import shutil

import pyTreeFoam
import getOpenFolderDDialog
import getOpenFilesDDialog
import inputTextDDialog
import createLayerDDialog
import createBlockAndSnappyDict
import runParaFoamOptionDialog
import universalDialogs as unvDlg
import meshViewerControl

import locale
localeDir = os.getenv("TreeFoamPath") + "/data/locale"
locale.bindtextdomain("treefoam", localeDir)

import logFileCreater
logFileCreater.log()

#------------
#  windowApp
#------------
class windowApp:

    def __init__(self, caseDir, modelDir):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("treefoam")
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path+"snappyHexMeshDialog.glade")
        self.mainWindow = self.builder.get_object("window1")
        self.mainWindow.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        #変数の設定
        self.caseDir = caseDir
        self.csvFileName = ""
        self.csvFileNames = []
        self.nCpu = ""
        self.nSeparate = ""
        self.meshViewer = ""
        #GUIのobject名を取得
        self.setGtkObject()
        #meshDataを取得
        self.meshDataDict = self.getCreateMeshDataDict()
        self.meshDataDict["stlDir"] = modelDir
        #meshDataをdialogに設定
        self.setCreateMeshData()
        #configTreeFoamの読み込み
        self.configDict = pyTreeFoam.readConfigTreeFoam()
        #comboBoxにデータセット
        self.setCsvFileNameToBox()

    def setGtkObject(self):
        """ glade内のobject名を取得する"""
        self.entry_stlDir = self.builder.get_object("entry_stlDir")
        self.entry_angle = self.builder.get_object("entry_angle")
        self.entry_nCpu = self.builder.get_object("entry_nCpu")
        self.entry_separate = self.builder.get_object("entry_separate")
        self.check_parallel = self.builder.get_object("check_parallel")
        self.combo_csvFile = self.builder.get_object("combo_csvFile")
        self.label_nCpu = self.builder.get_object("label_nCpu")
        self.label_separate = self.builder.get_object("label_separate")
        self.label_separateMethod = self.builder.get_object("label_separateMethod")

    def main(self):
        """ GUIの表示"""
        self.mainWindow.show()
        Gtk.main()

    def close(self, *args):
        """ 閉じる"""
        if self.meshViewer != "":
            self.meshViewer.closeViewer()
        Gtk.main_quit()

    def getCreateMeshDataDict(self):
        dataDict = pyTreeFoam.case(self.caseDir).getCreateMeshData()
        if dataDict["surfaceAngle"] == "":
            dataDict["surfaceAngle"] = "150"
        if dataDict["clearMesh"] == "":
            dataDict["clearMesh"] = "yes"
        if dataDict["parallel"] == "":
            dataDict["parallel"] = "no"
        if dataDict["nCpu"] == "":
            dataDict["nCpu"] = "4"
        if dataDict["nSeparate"] == "":
            dataDict["nSeparate"] = "2 2 2"
        return dataDict

    def setCsvFileNameToBox(self):
        """ csvファイル名をcomboBoxにセット"""
        stlDir = self.getStlDirFromTextBox()
        csvFiles = glob.glob(stlDir + os.sep + "*.csv")
        self.csvFileNames = list(map(lambda x: x.split(os.sep)[-1], csvFiles))
        self.csvFileNames.sort()
        self.combo_csvFile.clear()
        listModel = Gtk.ListStore(str)
        for name in self.csvFileNames:
            listModel.append([name])
        cell = Gtk.CellRendererText()
        self.combo_csvFile.pack_start(cell, True)
        self.combo_csvFile.add_attribute(cell, "text", 0)
        self.combo_csvFile.set_model(listModel)
        self.combo_csvFile.set_entry_text_column(0)

    def comboSelectFile(self):
        """ comboBox内のself.csvFileNameを選択表示する"""
        if self.csvFileName in self.csvFileNames:
            selNo = self.csvFileNames.index(self.csvFileName)
        else:
            selNo = 0
        self.combo_csvFile.set_active(selNo)

    #----- event handler ----------------
    #開く
    def onOpenFolder(self, event):
        self.openFolder()
    #参照
    def onSetStlLocation(self, event):
        self.setSelLocation()
    #stlチェック
    def onStlCheck(self, event):
        self.editStlFiles()
    #Dict編集
    def onFeatureEdge(self, event):
        self.featureEdge()
    #特徴線抽出
    def onRunFeatureEdge(self, event):
        self.runFeatureEdge()
    #並列処理チェック
    def onParaCheck(self, event):
        self.paraCheck()
    #csvファイル作成
    def onCreateCsv(self, event):
        self.createCsvFile()
        self.openCsvFile()
    #csvファイル編集
    def onEditCsvFile(self, event):
        self.openCsvFile()
    #meshDict作成
    def onCreateMeshDict(self, event):
        self.createMeshDict()
    #snappyDict編集
    def onEditSnappyDict(self, event):
        self.editSnappyDict()
    #snappy実行
    def onRunSnappyDict(self, event):
        self.runSnappy()
    #patch名修正
    def onChangePatchName(self, event):
        self.changePatchName()
    #layerDict作成
    def onCreateLayerDict(self, event):
        self.createLayerDict()
    #layerDict編集
    def onEditLayerDict(self, event):
        self.editLayerDict()
    #layer作成
    def onRunLayer(self, event):
        self.runLayer()
    #checkMesh実行
    def onCheckMesh(self, event):
        self.checkMesh()
    #meshViewer起動
    def onRunMeshViewer(self, event):
        self.runMeshViewer()
    #paraFoam起動
    def onRunParaFoam(self, event):
        self.runParaFoam()
    #閉じる
    def onCloseDialog(self, event):
        self.ending()
        self.close()

    #  setCreateMeshData
    def setCreateMeshData(self):
        """ メッシュ作成データをdialogに設定する"""
        self.entry_stlDir.set_text(self.meshDataDict["stlDir"])
        self.entry_angle.set_text(self.meshDataDict["surfaceAngle"])
        self.entry_nCpu.set_text(self.meshDataDict["nCpu"])
        self.entry_separate.set_text(self.meshDataDict["nSeparate"])
        if self.meshDataDict["parallel"] == "yes":
            self.check_parallel.set_active(True)
            self.label_nCpu.set_sensitive(True)
            self.entry_nCpu.set_sensitive(True)
            self.label_separate.set_sensitive(True)
            self.entry_separate.set_sensitive(True)
            self.label_separateMethod.set_sensitive(True)
        else:
            self.check_parallel.set_active(False)
            self.label_nCpu.set_sensitive(False)
            self.entry_nCpu.set_sensitive(False)
            self.label_separate.set_sensitive(False)
            self.entry_separate.set_sensitive(False)
            self.label_separateMethod.set_sensitive(False)


    #
    #  openFolder
    #  ----------
    def openFolder(self):
        """ stlDirを開く"""
        stlDir = self.getStlDirFromTextBox()
        if len(glob.glob(stlDir)) == 0:
            mess = _("設定したfolderは存在しません。\n場所を再設定してください。")
            self.errDialog(_("エラー"), mess)
            return
        fileManager = pyTreeFoam.fileManager
        comm = fileManager + " " + stlDir + " &"
        pyTreeFoam.run(self.caseDir).command(comm)

    def getStlDirFromTextBox(self):
        """ textBoxからstlDirをfullPathで取得する"""
        stlDir = self.entry_stlDir.get_text()
        if stlDir[0] != os.sep:
            stlDir = self.caseDir + os.sep + stlDir
            stlDir = os.path.abspath(stlDir)
        return stlDir

    #
    #  setSelLocation
    #  --------------
    def setSelLocation(self):
        """ stlDirを取得する"""
        print(_("stlDirを取得する"))
        # stlDir = self.getStlDirFromTextBox()
        # if len(glob.glob(stlDir)) == 1:
        #     selDir = stlDir
        # else:
        #     selDir = self.caseDir
        selDir = self.entry_stlDir.get_text()
        title = _("開く")
        cont = _("folderを選択する")
        rootDir = self.configDict["rootDir"]
        currDir = self.caseDir
        dialog = getOpenFolderDDialog.dialogApp(title, cont, rootDir, currDir, selDir)
        openDir = dialog.showDialog()
        if openDir != "":
            self.entry_stlDir.set_text(openDir)
            self.setCsvFileNameToBox()

    #
    #  editStlFiles
    #  ------------
    def editStlFiles(self):
        """ stlファイルを編集"""
        print(_("stlファイルを編集する"))
        stlDir = self.getStlDirFromTextBox()
        path = os.getenv("TreeFoamPath") + os.sep + "python"
        comm = "./editStlFilesDialog.py " + self.caseDir + " " + stlDir + " &"
        pyTreeFoam.run(path).command(comm)

    #
    #  featureEdge
    #--------------
    def featureEdge(self):
        """ OFversionによって、featureEdgeの作成方法を変える"""
        OFversion = self.configDict["OFversion"]
        ver = pyTreeFoam.getNumVersion(OFversion)
        if OFversion[0] == "v" or OFversion[:3] == "ext":
            #OF-v or OF-ext
            self.featureEdge_7()
        elif ver >= 8.0:
            #OF-8以降
            self.featureEdge_8()
        else:
            #OF-8未満
            self.featureEdge_7()
        # try:
        #     ver = float(OFversion)
        #     if ver < 8.0:
        #         #ver 7 以前
        #         self.featureEdge_7()
        #     else:
        #         #ver 8 以降
        #         self.featureEdge_8()
        # except:
        #     #ver vXXXX系
        #     self.featureEdge_7()

    #
    #  featureEdge_7
    #  --------------
    def featureEdge_7(self):
        """ ver 7.0 以前。特徴線抽出のためのDictを作成、編集する"""
        fileName = self.caseDir + "/system/surfaceFeatureExtractDict"
        angle = self.entry_angle.get_text()
        stlDir = self.getStlDirFromTextBox()
        if len(glob.glob(fileName)) == 0:
            #dictを新しく作成する
            selFiles = self.getFeatureFiles(stlDir, [])
            if len(selFiles) > 0:
                #stlファイルをが選択されている場合
                self.makeNewFeatureDict_7(fileName, selFiles, angle)
                title = _("surfaceFeatureExtractDictの作成")
                mess = _("「surfaceFeatureExtractDict」が無かったので、デフォルトのsurfaceFeatureExtractDictをsystemフォルダ内に作成しました。")
                self.okDialog(title, mess)
                pyTreeFoam.run(self.caseDir).editor([fileName])
        else:
            #dict内に記述されているstlファイルを取得
            seledFiles = self.getSelectedFiles(fileName)
            #特徴線を抽出するstlファイルを取得
            selFiles = self.getFeatureFiles(stlDir, seledFiles)
            if len(selFiles) > 0:
                self.makeReformFeatureDict_7(fileName, seledFiles, selFiles, angle)
                pyTreeFoam.run(self.caseDir).editor([fileName])
        return

    #
    #  featureEdge_8
    #  ---------------
    def featureEdge_8(self):
        """ ver 8.0 以降。特徴線抽出のためのDictを作成、編集する"""
        fileName = self.caseDir + "/system/surfaceFeaturesDict"
        angle = self.entry_angle.get_text()
        stlDir = self.getStlDirFromTextBox()
        if len(glob.glob(fileName)) == 0:
            #dictを新しく作成する
            selFiles = self.getFeatureFiles(stlDir, [])
            if len(selFiles) > 0:
                #stlファイルをが選択されている場合
                self.makeNewFeatureDict_8(fileName, selFiles, angle)
                title = _("surfaceFeaturesDictの作成")
                mess = _("「surfaceFeaturesDict」が無かったので、デフォルトのsurfaceFeatureExtractDictをsystemフォルダ内に作成しました。")
                self.okDialog(title, mess)
                pyTreeFoam.run(self.caseDir).editor([fileName])
        else:
            #dict内に記述されているstlファイルを取得
            seledFiles = self.getSelectedFiles(fileName)
            #特徴線を抽出するstlファイルを取得
            selFiles = self.getFeatureFiles(stlDir, seledFiles)
            if len(selFiles) > 0:
                self.makeReformFeatureDict_8(fileName, seledFiles, selFiles, angle)
                pyTreeFoam.run(self.caseDir).editor([fileName])
        return

    def getFeatureFiles(self, stlDir, seledFiles):
        """ 特徴線抽出用のstlファイルを選択する"""
        title = _("特徴線抽出fileの選択")
        mess = _("特徴線(featureEdge)を抽出します。")
        mess += "\n" + _("特徴線を抽出するfileを選択してください。")
        currDir = stlDir
        fileFilter = "*.stl"
        dialog = getOpenFilesDDialog.dialogApp(title, mess, currDir, fileFilter, seledFiles)
        selFiles = dialog.show()
        return selFiles

    def makeNewFeatureDict_7(self, fileName, selFiles, angle):
        """ ver 7 以前。デフォルトのsurfaceFeatureExtractDictを新しく作成する"""
        contents = ""
        i = 0
        getFoamContents = pyTreeFoam.getFoamContents(self.caseDir)
        for selFile in selFiles:
            newLineCont = self.getDefaultFeatureDict(selFile, angle)
            if i > 0:
                #コメント文を削除
                newLineCont = pyTreeFoam.strOp(newLineCont).deleteCommentsNullLines()
            contents += newLineCont
            i += 1
        OFversion = self.configDict["OFversion"]
        header = getFoamContents.makeFoamHeaderVersion(OFversion)
        foamFile = getFoamContents.makeFoamFile("2.0", "ascii", "dictionary",
                                "", "surfaceFeatureExtractDict")
        footer = getFoamContents.makeFoamFooter()
        contents = header + foamFile + contents + footer
        f = open(fileName, "w"); f.write(contents); f.close()

    def makeNewFeatureDict_8(self, fileName, selFiles, angle):
        """ ver 8以降。デフォルトのsurfaceFeatureExtractDictを新しく作成する"""
        getFoamContents = pyTreeFoam.getFoamContents(self.caseDir)
        contents = getFoamContents.defaultSurfaceFeaturesDict()
        contentsOp = pyTreeFoam.strOp(contents)
        pairCont, pp = contentsOp.getMiddlePair(0)
        pe = pp - 1
        ps = pe - len(pairCont)
        header = contents[:ps]
        footer = contents[pe:]
        pairContOp = pyTreeFoam.strOp(pairCont)
        #stlFileをセット
        newLine = "surfaces\n"
        newLine += "    (\n"
        stlLine = ""
        for stlFile in selFiles:
            stlLine += '        "' + stlFile + '"\n'
        newLine += stlLine
        newLine += "    );"
        pairCont, _pp, keyword = pairContOp.replace1lineKeyword(0, newLine)
        #angleをセット
        newLine = "includedAngle  " + angle + ";"
        pairCont, _pp, keyword = pairContOp.replace1lineKeyword(0, newLine)
        contents = header + pairCont + footer
        #全内容を作成
        OFversion = self.configDict["OFversion"]
        header = getFoamContents.makeFoamHeaderVersion(OFversion)
        foamFile = getFoamContents.makeFoamFile("2.0", "ascii", "dictionary",
                                "", "surfaceFeaturesDict")
        footer = getFoamContents.makeFoamFooter()
        contents = header + foamFile + contents + footer
        f = open(fileName, "w"); f.write(contents); f.close()

    def makeReformFeatureDict_7(self, fileName, seledFiles, selFiles, angle):
        """ dictを修正する"""
        f = open(fileName); cont = f.read(); f.close()
        #dictから削除するfileを取得
        delFiles = []
        for seledFile in seledFiles:
            if not(seledFile in selFiles):
                delFiles.append(seledFile)
        #dictからdelFilesを削除する
        contents = ""
        contOp = pyTreeFoam.strOp(cont)
        p = contOp.skipFoamFile(0)
        (lineCont, p, _kind) = contOp.get1line(p)
        #  headerを取得
        pe = p - len(lineCont) - 1
        contents = cont[:pe]
        #  残すfileを取得
        while lineCont != "" and p < len(cont):
            lineContOp = pyTreeFoam.strOp(lineCont)
            (keyword, _pp) = lineContOp.getKeyword(0)
            if not(keyword in delFiles):
                contents += "\n" + lineCont
            (lineCont, p, _kind) = contOp.get1line(p)
        contents += "\n"
        #追加するfileを取得
        addFiles = []
        for selFile in selFiles:
            if not(selFile in seledFiles):
                addFiles.append(selFile)
        #stlをDictに追加
        for addFile in addFiles:
            addCont = self.getDefaultFeatureDict(addFile, angle)
            addCont = pyTreeFoam.strOp(addCont).deleteCommentsNullLines()
            contents += addCont
        #footerを取得
        contents += pyTreeFoam.getFoamContents().makeFoamFooter()
        #dictを保存
        f = open(fileName, "w"); f.write(contents); f.close()

    def makeReformFeatureDict_8(self, fileName, seledFiles, selFiles, angle):
        """ dictを修正する"""
        f = open(fileName); contents = f.read(); f.close()
        contentsOp = pyTreeFoam.strOp(contents)
        p = contentsOp.skipFoamFile()
        pairCont, pp = contentsOp.getMiddlePair(p)
        pe = pp - 1
        ps = pe - len(pairCont)
        header = contents[:ps]
        footer = contents[pe:]
        pairContOp = pyTreeFoam.strOp(pairCont)
        #stlFileをセット
        newLine = "surfaces\n"
        newLine += "    (\n"
        stlLine = ""
        for stlFile in selFiles:
            stlLine += '        "' + stlFile + '"\n'
        newLine += stlLine
        newLine += "    );"
        pairCont, _pp, keyword = pairContOp.replace1lineKeyword(0, newLine)
        #angleをセット
        newLine = "includedAngle  " + angle + ";"
        pairCont, _pp, keyword = pairContOp.replace1lineKeyword(0, newLine)
        contents = header + pairCont + footer
        f = open(fileName, "w"); f.write(contents); f.close()

    def getDefaultFeatureDict(self, stlFile, angle):
        """ OFver7以前、defaultのFeatureDictにstlファイル名と角度を設定して返す"""
        #defaultの内容を取得
        getFoamContents = pyTreeFoam.getFoamContents(self.caseDir)
        cont = getFoamContents.defaultSurfaceFeatureExtractDict()
        #stlファイル名をセット
        contOp = pyTreeFoam.strOp(cont)
        (_keyword, p) = contOp.getKeyword(0)
        cont = stlFile + cont[p:]
        #角度をセット
        contOp = pyTreeFoam.strOp(cont)
        p = contOp.skipIntoPair(0)              #ポインタを{}内に進める
        (lineCont, p, kind) = contOp.get1line(p)
        while lineCont != "":
            if kind == "keyword":
                lineContOp = pyTreeFoam.strOp(lineCont)
                (keyword, _pp) = lineContOp.getKeyword(0)
                if keyword == "extractFromSurfaceCoeffs":
                    ps = p - len(lineCont)      #ポインタをkeywordに戻す
                    header = cont[:ps]
                    footer = cont[p:]
                    #角度をセット
                    ps = lineContOp.skipIntoPair(0)
                    newLine = "includedAngle  " + angle + ";"
                    (lineCont, _pp, keyword) = lineContOp.replace1lineKeyword(ps, newLine)
                    break
            (lineCont, p, kind) = contOp.get1line(p)
        newCont = header + lineCont + footer
        return newCont
    
    def getSelectedFiles(self, fileName):
        OFversion = self.configDict["OFversion"]
        ver = pyTreeFoam.getNumVersion(OFversion)
        if ver >= 8.0:
            #OF-8以上
            seledFiles = self.getSelectedFiles_8(fileName)
        else:
            #OF-8未満、OF-v、OF-ext
            seledFiles = self.getSelectedFiles_7(fileName)
        # try:
        #     ver = float(OFversion)
        #     if ver < 8.0:
        #         seledFiles = self.getSelectedFiles_7(fileName)
        #     else:
        #         seledFiles = self.getSelectedFiles_8(fileName)
        # except:
        #     seledFiles = self.getSelectedFiles_7(fileName)
        return seledFiles

    def getSelectedFiles_7(self, fileName):
        """ ver 7 以前。surfaceFeatureExtractDictを読み込み、ここで設定しているstlファイルを取得する"""
        seledFiles = []
        f = open(fileName); cont = f.read(); f.close()
        contOp = pyTreeFoam.strOp(cont)
        p = contOp.skipFoamFile(0)
        flag = 0
        while flag == 0:
            (keyCont, p) = contOp.getNextKeywordAndContents(p)
            name = keyCont[0]
            if name != "":
                seledFiles.append(name)
            else:
                flag = 1
        return seledFiles

    def getSelectedFiles_8(self, fileName):
        """ ver 8 以降。surfaceFeatureExtractDictを読み込み、ここで設定しているstlファイルを取得する"""
        seledFiles = []
        f = open(fileName); cont = f.read(); f.close()
        contOp = pyTreeFoam.strOp(cont)
        p = contOp.skipFoamFile(0)
        pairCont, pp = contOp.getMiddlePair(p)
        pe = pp - 1
        ps = pe - len(pairCont)
        header = cont[:ps]
        footer = cont[pe:]
        pairContOp = pyTreeFoam.strOp(pairCont)
        lineCont, p, kind = pairContOp.get1line(0)
        while lineCont != "":
            lineContOp = pyTreeFoam.strOp(lineCont)
            keyword, pp = lineContOp.getKeyword(0)
            if keyword == "surfaces":
                stlLines, _pp = lineContOp.getSmallPair(pp)
                words = stlLines.split()
                seledFiles = list(map(lambda x: x[1:-1], words))
                break
        return seledFiles

    def checkParallelData(self):
        """ parallelDataをチェック"""
        self.nCpu = self.entry_nCpu.get_text()
        self.nSeparate = self.entry_separate.get_text()
        error = "OK"
        if self.check_parallel.get_active() == True:
            words = self.nSeparate.split()
            nSum = 1
            try:
                for word in words:
                    nSum = nSum * int(word)
            except:
                nSum = 0
                error = "ERROR"
            try:
                n = int(self.nCpu)
            except:
                n = 0
            if error == "OK":
                if n != nSum:
                    error = "ERROR"
            if error != "OK":
                title = _("エラー")
                mess = _("並列処理の為の「nCPU」と「分割数(xyz)」が合いません。\n")
                mess += _("    nCPU=") + str(n) + _(u"  分割数=") + str(nSum)
                self.errDialog(title, mess)
        return error

    def remakeDecomposeParDict(self):
        """ decomposeParDictを修正する"""

        def getInsertPointFromEnd(cont):
            """ 挿入場所を検索する。挿入場所は、footerの上"""
            #footerを検索
            n = cont.find("*********")
            if n < 0:
                #footerが存在しない場合
                ip = len(cont)
                return ip
            #位置「n」から遡って、「\n」を検索
            ip = len(cont)
            i = n
            while i >= 0:
                i -= 1
                if cont[i] == "\n":
                    ip = i
                    break
            return ip

        error = self.checkParallelData()
        if error != "OK":
            return error
        fileName = self.caseDir + "/system/decomposeParDict"
        f = open(fileName); cont = f.read(); f.close()
        contOp = pyTreeFoam.strOp(cont)
        ps = contOp.skipFoamFile()
        #numberOfSubdomainsを書き換え
        newLine = "numberOfSubdomains  " + self.nCpu + ";"
        (cont, _p, _keyword) = contOp.replace1lineKeyword(ps, newLine)
        #methodを書き換え
        newLine = "method   simple;"
        (cont, _p, _keyword) = contOp.replace1lineKeyword(ps, newLine)
        #simpleCoffesを書き換え
        newLine  = "simpleCoeffs\n"
        newLine += "{\n"
        newLine += "    n    (" + self.nSeparate + ");\n"
        newLine += "    delta 0.001;\n"
        newLine += "}"
        (retCont, _p, keyword) = contOp.replace1lineKeyword(ps, newLine)
        if keyword == "":
            #最後に追加する
            p = getInsertPointFromEnd(cont)
            cont = cont[:p] + newLine + "\n" + cont[p:]
        else:
            cont = retCont
        #書き込み
        f = open(fileName, "w"); f.write(cont); f.close()
        return "OK"

    #
    #  runSnappy_sub
    def runSnappy_sub(self, job):
        """ mesh作成。
        job:(block: blockMesh作成), (snappy: snappyHexMesh作成), (patch: patch名修正)"""
        blockDict = "system/blockMeshDict"
        #meshデータ削除
        delFolder = self.caseDir + "/constant/polyMesh"
        files = pyTreeFoam.getFiles(delFolder)
        for file in files:
            os.remove(file)
        
        logFile = self.caseDir + os.sep + "solve.log"
        #os.remove(logFile)
        f=open(logFile, "w"); f.close()
        envOpenFoam = self.configDict["bashrcFOAM"]
        cont  = "#!/bin/bash\n"
        cont += ". " + envOpenFoam + "\n"
        if self.check_parallel.get_active() == True:
            #blockMesh
            if "block" in job:
                cont += "blockMesh -dict " + blockDict + " 2>&1 | tee -a solve.log\n"
            #snappy
            if "snappy" in job:
                #並列処理
                error = self.remakeDecomposeParDict()
                if error != "OK":
                    return
                cont += os.getenv("TreeFoamPath") + "/python/meetPatchNameDeleteNullPatch.py " + self.caseDir + "\n"
                cont += "rm -rf processor*\n"
                cont += "decomposePar\n"
                cont += "mpirun -np " + self.nCpu + " snappyHexMesh -parallel 2>&1 | tee -a solve.log\n"
                #contDict = pyTreeFoam.case(self.caseDir).getControlDict()
                #pre = contDict["writePrecision"]
                #cont += "reconstructParMesh -latestTime -cellDist -mergeTol 1e-" + pre + "\n"
                cont += self.createReconstructMeshCommLine()
                cont += os.getenv("TreeFoamPath") + "/python/paraSnappyPost.py " + self.caseDir + "\n"
            #patch名
            if "patch" in job:
                cont += os.getenv("TreeFoamPath") + "/python/meetPatchNameDeleteNullPatch.py " + self.caseDir + "\n"
        else:
            #blockMesh
            if "block" in job:
                cont += "blockMesh -dict " + blockDict + " 2>&1 | tee -a solve.log\n"
            if "snappy" in job:
                cont += "snappyHexMesh -overwrite 2>&1 | tee -a solve.log\n"
            #patch名
            if "patch" in job:
                cont += os.getenv("TreeFoamPath") + "/python/meetPatchNameDeleteNullPatch.py " + self.caseDir + "\n"
        #スクリプト作成
        fileName = self.caseDir + os.sep + "run"
        f = open(fileName, "w"); f.write(cont); f.close()
        comm = "chmod a+x run"
        run = pyTreeFoam.run(self.caseDir)
        run.command(comm)
        run.foamTerminal("./run")

    #
    #  createReconstructMeshCommLine
    def createReconstructMeshCommLine(self):
        """ reconstructParMeshコマンド行を作成する。
        OF-9からコマンド変更"""

        def createCommLine_Org():
            contDict = pyTreeFoam.case(self.caseDir).getControlDict()
            pre = contDict["writePrecision"]
            line = "reconstructParMesh -latestTime -cellDist -mergeTol 1e-" + pre + "\n"
            return line

        def createCommLine_9():
            line = "reconstructParMesh -latestTime -cellDist\n"
            return line

        OFversion = self.configDict["OFversion"]
        numVer = pyTreeFoam.getNumVersion(OFversion)
        if OFversion >= "v1804":
            line = createCommLine_Org()
        elif OFversion[:3] == "ext":
            line = createCommLine_Org()
        else:
            if numVer >= 9.0:
                #OF-9以上
                line = createCommLine_9()
            else:
                #OF-9未満
                line = createCommLine_Org()
        return line

    #
    #  runFeatureEdge
    #-----------------
    def runFeatureEdge(self):
        OFversion = self.configDict["OFversion"]
        ver = pyTreeFoam.getNumVersion(OFversion)
        if ver >= 8.0:
            #OF-8以上
            self.runFeatureEdge_8()
        else:
            #OF-8未満、OF-v、OF-ext
            self.runFeatureEdge_7()

        # try:
        #     ver = float(OFversion)
        #     if ver < 8.0:
        #         self.runFeatureEdge_7()
        #     else:
        #         self.runFeatureEdge_8()
        # except:
        #     self.runFeatureEdge_7()

    #
    #  runFeatureEdge_7
    #  -----------------
    def runFeatureEdge_7(self):
        """ ver 7 以前。特徴線を抽出する"""
        #Dictファイルの確認
        if len(glob.glob(self.caseDir + "/system/surfaceFeatureExtractDict")) == 0:
            title = _("エラー")
            mess = _("surfaceFeatureExtractDictファイルがありません。\n特徴線が抽出できません。")
            self.errDialog(title, mess)
            return
        #実行の確認
        title = _("特徴線の抽出")
        mess = _("surfaceFeatureExtractDict内容に基づき、特徴線を抽出します。")
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return
        #stlFileをtriSurfaceに移動
        stlDir = self.getStlDirFromTextBox()
        if len(glob.glob(self.caseDir + "/constant/triSurface/")) == 0:
            os.mkdir(self.caseDir + "/constant/triSurface")
        files = glob.glob(stlDir + "/*.stl")
        for stlFile in files:
            shutil.copy(stlFile, self.caseDir + "/constant/triSurface")
        #実行
        envOpenFoam = self.configDict["bashrcFOAM"]
        comm = ". " + envOpenFoam + "; "
        comm += "surfaceFeatureExtract"
        (stat, _res, _err) = pyTreeFoam.run(self.caseDir).commandReturnCont(comm)
        #eMeshファイルを取得
        eMeshFiles = []
        for stlFile in files:
            name = stlFile.split("/")[-1]
            eMeshFile = self.caseDir + "/constant/triSurface/" + ".".join(name.split(".")[:-1]) + ".eMesh"
            eMeshFiles += glob.glob(eMeshFile)
        #eMeshファイルを戻す
        for eMeshFile in eMeshFiles:
            shutil.copy(eMeshFile, stlDir)
        #エラーチェック
        eMeshFilesStl = glob.glob(stlDir + "/*eMesh*")
        title = _("特徴線（feature edge）の抽出")
        if len(eMeshFilesStl) == 0 or stat != "OK":
            message = _("エラーが発生しました。\n「./constant/triSurface」フォルダ内の*.eMeshファイルを確認してください。")
            self.errDialog(title, message)
        else:
            message = _("surfaceFeatureExtractDict から特徴線（feature edge）を抽出しました。")
            self.okDialog(title, message)

    #
    #  runFeatureEdge_8
    #  -----------------
    def runFeatureEdge_8(self):
        """ ver 8 以降。特徴線を抽出する"""
        #Dictファイルの確認
        if len(glob.glob(self.caseDir + "/system/surfaceFeaturesDict")) == 0:
            title = _("エラー")
            mess = _("surfaceFeaturesDictファイルがありません。\n特徴線が抽出できません。")
            self.errDialog(title, mess)
            return
        #実行の確認
        title = _("特徴線の抽出")
        mess = _("surfaceFeaturesDict内容に基づき、特徴線を抽出します。")
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return
        #stlFileをtriSurfaceに移動
        stlDir = self.getStlDirFromTextBox()
        if len(glob.glob(self.caseDir + "/constant/triSurface/")) == 0:
            os.mkdir(self.caseDir + "/constant/triSurface")
        files = glob.glob(stlDir + "/*.stl")
        for stlFile in files:
            shutil.copy(stlFile, self.caseDir + "/constant/triSurface")
        #実行
        envOpenFoam = self.configDict["bashrcFOAM"]
        comm = ". " + envOpenFoam + "; "
        comm += "surfaceFeatures"
        (stat, _res, _err) = pyTreeFoam.run(self.caseDir).commandReturnCont(comm)
        #eMeshファイルを取得
        eMeshFiles = []
        for stlFile in files:
            name = stlFile.split("/")[-1]
            eMeshFile = self.caseDir + "/constant/triSurface/" + ".".join(name.split(".")[:-1]) + ".eMesh"
            eMeshFiles += glob.glob(eMeshFile)
        #eMeshファイルを戻す
        for eMeshFile in eMeshFiles:
            shutil.copy(eMeshFile, stlDir)
        #エラーチェック
        eMeshFilesStl = glob.glob(stlDir + "/*eMesh*")
        title = _("特徴線（feature edge）の抽出")
        if len(eMeshFilesStl) == 0 or stat != "OK":
            message = _("エラーが発生しました。\n「./constant/triSurface」フォルダ内の*.eMeshファイルを確認してください。")
            self.errDialog(title, message)
        else:
            message = _("surfaceFeaturesDict から特徴線（feature edge）を抽出しました。")
            self.okDialog(title, message)

    #
    #  paraCheck
    #  ---------
    def paraCheck(self):
        """ 並列処理"""
        if self.check_parallel.get_active() == True:
            self.check_parallel.set_active(True)
            self.label_nCpu.set_sensitive(True)
            self.entry_nCpu.set_sensitive(True)
            self.label_separate.set_sensitive(True)
            self.entry_separate.set_sensitive(True)
            self.label_separateMethod.set_sensitive(True)
        else:
            self.check_parallel.set_active(False)
            self.label_nCpu.set_sensitive(False)
            self.entry_nCpu.set_sensitive(False)
            self.label_separate.set_sensitive(False)
            self.entry_separate.set_sensitive(False)
            self.label_separateMethod.set_sensitive(False)

    #
    #  createCsvFile
    #  -------------
    def createCsvFile(self):
        """ blockMesh, snappyHexMeshDict作成用のcsvファイル作成"""
        #file名の取得
        title = _("ファイル名の入力")
        message = _("csvファイル名を入力してください。")
        inputText = "snappyMeshDict.csv"
        dialog = inputTextDDialog.dialogApp(title, message, inputText)
        self.csvFileName = dialog.show()
        if self.csvFileName == "":
            return
        #csvFileを作成
        stlDir = self.getStlDirFromTextBox()
        createBlockAndSnappyDict.createCsvFile(stlDir, self.csvFileName)
        title = _("csvファイル作成")
        mess = _("blockMesh、snappyHexMeshDict作成用のcsvファイルをstlDir内に作成しました。\n")
        mess += _("office等でcsvファイルを編集し、cellSize等を設定してください。")
        self.okDialog(title, mess)
        #comboBoxにcsvを追加
        self.setCsvFileNameToBox()
        #comboBox内のself..csvFileNameを選択する
        self.comboSelectFile()

    #
    #  openCsvFile
    #  -----------
    def openCsvFile(self):
        """ csvFileを開き、編集する"""
        stlDir = self.getStlDirFromTextBox()
        #csvFileを取得
        iter = self.combo_csvFile.get_active_iter()
        if iter == None:
            title = _("csvファイル編集")
            mess = _("csvファイルが選択されていません。\n")
            mess += _("編集するcsvファイルを選択してください。")
            self.errDialog(title, mess)
            return
        model = self.combo_csvFile.get_model()
        self.csvFileName = model.get_value(iter, 0)
        fileName = stlDir + os.sep + self.csvFileName
        comm = pyTreeFoam.Office + " " + fileName + " &"
        pyTreeFoam.run(stlDir).command(comm)

    #
    #  createMeshDict
    #  --------------
    def createMeshDict(self):
        """ blockMesh, snappyHexMeshDictを作成する"""
        stlDir = self.getStlDirFromTextBox()
        #csvFileを取得
        iter = self.combo_csvFile.get_active_iter()
        if iter == None:
            title = _("csvファイル編集")
            mess = _("csvファイルが選択されていません。\n")
            mess += _("編集するcsvファイルを選択してください。")
            self.errDialog(title, mess)
            return
        model = self.combo_csvFile.get_model()
        self.csvFileName = model.get_value(iter, 0)
        fileName = stlDir + os.sep + self.csvFileName
        #blockMesh, snappyHexMeshDict, decomposeParDict有無チェック
        self.checkBlockMeshSnappyHexMesh()
        #csv, stl, eMeshファイルの有無チェック
        (error, nonFiles) = createBlockAndSnappyDict.checkStlEMeshFiles(
                                self.caseDir, stlDir, fileName)
        eMeshFiles = []
        if error != 0:
            title = _("エラー")
            #csvfileのエラー
            if len(nonFiles) == 0:
                mess =_("csvファイルから値が取得できません。\n")
                mess += _("詳細は、logを確認してください。")
                self.errDialog(title, mess)
                return
            else:
                #必要なファイル（stl）が存在しない場合
                stlFiles = []
                for name in nonFiles:
                    fileType = name.split(".")[-1]
                    if fileType == "stl":
                        stlFiles.append(name)
                #stlFile有無チェック
                if len(stlFiles) != 0:
                    mess = _("以下のstlFileが存在しません。\n")
                    for name in stlFiles:
                        mess += name +", "
                    self.errDialog(title, mess)
                    return
                #必要なファイル（eMesh）が存在しない場合
                eMeshFiles = []
                for name in nonFiles:
                    words = name.split(".")
                    fileType = words[-1]
                    if fileType == "eMesh":
                        eMeshFiles.append(".".join(words[:-1])+".stl")
        #eMesh作成(eMeshFilesについて作成する)
        if len(eMeshFiles) != 0:
            #eMeshを作成する
            OFversion = self.configDict["OFversion"]
            ver = pyTreeFoam.getNumVersion(OFversion)
            if ver >= 8.0:
                #OF-8以上
                self.addSurfaceFeatureEdge_8(eMeshFiles)
            else:
                #OF-8未満、OF-V、OF-ext
                self.addSurfaceFeatureEdge_7(eMeshFiles)

            # try:
            #     ver = float(OFversion)
            #     if ver < 8.0:
            #         self.addSurfaceFeatureEdge_7(eMeshFiles)
            #     else:
            #         self.addSurfaceFeatureEdge_8(eMeshFiles)
            # except:
            #     self.addSurfaceFeatureEdge_7(eMeshFiles)

        #blockMeshDict, snappyDictを作成する
        fileName = stlDir + os.sep + self.csvFileName
        error = createBlockAndSnappyDict.createMeshDict(self.caseDir, stlDir, fileName)
        if error != 0:
            title = _("エラー")
            mess = _("Dictファイル作成中にエラーが発生しました。\n")
            mess += _("詳細はlogを確認してください。")
            self.errDialog(title, mess)
            return
        title = _("meshDict作成")
        mess = _("blocMeshDict、snappyHexMeshDictファイが作成できました。\n")
        mess += _("引き続き、snappyHexMeshを実行してmeshを作成しますか？")
        print(mess)
        stat = self.yesNoDialog(title, mess)
        if stat == "NO":
            return
        #mesh作成
        job = ["block", "snappy", "patch"]
        self.runSnappy_sub(job)

    #
    #  addSurfaceFeatureEdge_7
    def addSurfaceFeatureEdge_7(self, eMeshFiles):
        """ 指定のeMeshファイルを作成する。"""
        stlDir = self.getStlDirFromTextBox()
        angle = self.entry_angle.get_text()
        dictFile = self.caseDir + "/system/surfaceFeatureExtractDict"
        if len(glob.glob(dictFile)) == 0:
            #dictを新たに作成する
            self.makeNewFeatureDict_7(dictFile, eMeshFiles, angle)
        else:
            #dictにnonFilesを追加する
            # dict内に記述されているstlファイルを取得
            seledFiles = self.getSelectedFiles(dictFile)
            self.makeReformFeatureDict_7(dictFile, seledFiles, eMeshFiles, angle)
        #eMeshFileを作成
        if len(glob.glob(self.caseDir + "/constant/triSurface/")) == 0:
            os.mkdir(self.caseDir + "/constant/triSurface")
        files = glob.glob(stlDir + "/*.stl")
        for stlFile in files:
            shutil.copy(stlFile, self.caseDir + "/constant/triSurface")
        #実行
        envOpenFoam = self.configDict["bashrcFOAM"]
        comm = ". " + envOpenFoam + "; "
        comm += "surfaceFeatureExtract"
        (stat, _res, _err) = pyTreeFoam.run(self.caseDir).commandReturnCont(comm)
        #eMeshファイルを取得
        eMeshFiles = []
        for stlFile in files:
            name = stlFile.split("/")[-1]
            eMeshFile = self.caseDir + "/constant/triSurface/" + ".".join(name.split(".")[:-1]) + ".eMesh"
            eMeshFiles += glob.glob(eMeshFile)
        #eMeshファイルを戻す
        for eMeshFile in eMeshFiles:
            shutil.copy(eMeshFile, stlDir)

    #
    #  addSurfaceFeatureEdge_8
    def addSurfaceFeatureEdge_8(self, eMeshFiles):
        """ 指定のeMeshファイルを作成する。"""
        stlDir = self.getStlDirFromTextBox()
        angle = self.entry_angle.get_text()
        dictFile = self.caseDir + "/system/surfaceFeaturesDict"
        if len(glob.glob(dictFile)) == 0:
            #dictを新たに作成する
            self.makeNewFeatureDict_8(dictFile, eMeshFiles, angle)
        else:
            #dictにnonFilesを追加する
            # dict内に記述されているstlファイルを取得
            seledFiles = self.getSelectedFiles(dictFile)
            self.makeReformFeatureDict_8(dictFile, seledFiles, eMeshFiles, angle)
        #eMeshFileを作成
        if len(glob.glob(self.caseDir + "/constant/triSurface/")) == 0:
            os.mkdir(self.caseDir + "/constant/triSurface")
        files = glob.glob(stlDir + "/*.stl")
        for stlFile in files:
            shutil.copy(stlFile, self.caseDir + "/constant/triSurface")
        #実行
        envOpenFoam = self.configDict["bashrcFOAM"]
        comm = ". " + envOpenFoam + "; "
        comm += "surfaceFeatures"
        (stat, _res, _err) = pyTreeFoam.run(self.caseDir).commandReturnCont(comm)
        #eMeshファイルを取得
        eMeshFiles = []
        for stlFile in files:
            name = stlFile.split("/")[-1]
            eMeshFile = self.caseDir + "/constant/triSurface/" + ".".join(name.split(".")[:-1]) + ".eMesh"
            eMeshFiles += glob.glob(eMeshFile)
        #eMeshファイルを戻す
        for eMeshFile in eMeshFiles:
            shutil.copy(eMeshFile, stlDir)

    #
    #  editSnappyDict
    #  --------------
    def editSnappyDict(self):
        """ snappyHexMeshDict、blockMeshの編集"""
        editFiles = [self.caseDir + "/system/snappyHexMeshDict"]
        editFiles += [self.caseDir + "/system/blockMeshDict"]
        pyTreeFoam.run(self.caseDir).editor(editFiles)

    #
    #  runSnappy
    #  ---------
    def runSnappy(self):
        """ snappyHexMesh実行"""
        snappyDict = self.caseDir + "/system/snappyHexMeshDict"
        blockDict = self.caseDir + "/system/blockMeshDict"
        if len(glob.glob(blockDict)) == 0 or len(glob.glob(snappyDict)) == 0:
            title = _("エラー")
            message = _("「blockMeshDict」または「snappyHexhMeshDict」ファイルがsystemフォルダ内に存在しない為、実行できません。")
            self.errDialog(title, message)
            return
        #実行の確認
        title = _("dict実行")
        mess = _("meshデータを削除して、blockMesh、snappyHexMeshを実行し、meshを作り出します。")
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return
        #mesh作成
        job = ["block", "snappy"]
        self.runSnappy_sub(job)

    #
    #  changePatchName
    #  ---------------
    def changePatchName(self):
        """ patch名の修正"""
        title = _("patch名修正")
        mess  = _("patch名をstlファイル名に変更します。\n")
        mess += _("  未定義patchを「zeroGradient」で定義した上で\n")
        mess += _("  patch名に含まれるsolid名を削除する。\n")
        mess += _("  全ての空patchを削除する。")
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return
        #patch名修正
        comm = os.getenv("TreeFoamPath") + "/python/meetPatchNameDeleteNullPatch.py " + self.caseDir
        pyTreeFoam.run(self.caseDir).command(comm)
        title = _("mesh作成")
        mess = _("patch名をstlファイル名に変更しました。")
        print(mess)
        self.okDialog(title, mess)
    
    #
    #  createLayerDict
    #  ---------------
    def createLayerDict(self):
        """ layer作成用のDictを作成"""
        dialog = createLayerDDialog.dialogApp(self.caseDir)
        _ans = dialog.show()

    #
    #  editLayerDict
    #  -------------
    def editLayerDict(self):
        """ snappyHexMeshDictの編集"""
        editFiles = [self.caseDir + "/system/snappyHexMeshDict"]
        pyTreeFoam.run(self.caseDir).editor(editFiles)

    #
    #  runLayer
    #  --------
    def runLayer(self):
        """ layerの作成"""
        title = _("layerの作成")
        mess = _("現在のmeshにlayerを追加します。\n（snappyHexMeshの実行）")
        ans = self.okCancelDialog(title, mess)

        envOpenFoam = self.configDict["bashrcFOAM"]
        if ans != "OK":
            return
        if self.check_parallel.get_active() == True:
            error = self.remakeDecomposeParDict()
            if error != "OK":
                return
            envOpenFoam = self.configDict["bashrcFOAM"]
            cont  = "#!/bin/bash\n"
            cont += ". " + envOpenFoam + "\n"
            cont += "rm -rf processor*\n"
            cont += "decomposePar 2>&1 | tee solve.log\n"
            cont += "mpirun -np " + self.nCpu + " snappyHexMesh -parallel 2>&1 | tee -a solve.log\n"
            #contDict = pyTreeFoam.case(self.caseDir).getControlDict()
            #pre = contDict["writePrecision"]
            #cont += "reconstructParMesh -latestTime -cellDist -mergeTol 1e-" + pre + "\n"
            cont += self.createReconstructMeshCommLine()
            #cont += os.getenv("TreeFoamPath") + "/python/paraSnappyPost.py " + self.caseDir + "\n"
            cont += "rm -rf processor*\n"
        else:
            #シングルコアの場合
            cont = "#!/bin/bash\n"
            cont += ". " + envOpenFoam + "\n"
            cont += "snappyHexMesh | tee solve.log\n"
        comm = "echo " + "'" + cont + "'" + " > run"
        run = pyTreeFoam.run(self.caseDir)
        run.command(comm)
        run.command("chmod a+x run")
        run.foamTerminal("./run")

    #
    #  checkMesh
    #  ---------
    def checkMesh(self):
        """ checkMeshを実行"""
        envOpenFoam = self.configDict["bashrcFOAM"]
        cont = "#!/bin/bash\n"
        cont += ". " + envOpenFoam + "\n"
        cont += "checkMesh | tee solve.log\n"
        comm = "echo " + "'" + cont + "'" + " > run"
        run = pyTreeFoam.run(self.caseDir)
        run.command(comm)
        run.command("chmod a+x run")
        run.foamTerminal("./run")

    #
    #  runMeshViewer
    #----------------
    def runMeshViewer(self):
        """ meshViewerを起動する"""
        if self.meshViewer != "":
            if self.meshViewer.isAlive() == True:
                title = _("meshViewer")
                mess = _("meshViewerは、既に起動しています。")
                self.errDialog(title, mess)
                return
        folType = pyTreeFoam.getFolderType(self.caseDir)
        if folType == "case" or folType == "caselink":
            caseDir = self.caseDir
            timeFolder = pyTreeFoam.case(caseDir).getCurrTimeFolder()
            region = "."
            title = "meshViewer_snappyOp_" + str(os.getpid())
            self.meshViewer = meshViewerControl.control(caseDir, timeFolder, region, title)
            self.meshViewer.openViewer("full")
            commLines  = [_("start\n")]
            commLines += [_("mess snappyHexMesh dialogから起動しました。他とlinkしていません。\n")]
            commLines += [_("addMess '  patch, zone, setが表示できます。'\n")]
            commLines += [_("addMess '  tree内の選択した項目が表示されます。'\n")]
            commLines += [_("time startFrom\n")]
            commLines += [_("print 'loading ...'\n")]
            commLines += [_("wait 0.2s\n")]     #GUIを完成させる為のwait
            #commLines += [_("load patchesZones sets patchTypes\n")]
            commLines += [_("load all\n")]
            commLines += [_("show\n")]
            commLines += [_("resetCamera\n")]
            commLines += [_("setTreeList patches zones sets\n")]
            commLines += [_("end\n")]
            _stat = self.meshViewer.runMultiCommands(commLines)
        else:
            title = _("meshViewerの起動")
            mess = _("このfolderは、OpenFOAMのcaseフォルダではありません。") + "\n"
            self.errDialog(title, mess)
        return

    #
    #  runParaFoam
    #  -----------
    def runParaFoam(self):
        """ paraFoamの起動"""
        (isRegion, _loc, _regNames) = pyTreeFoam.case(self.caseDir).isMultiRegion()
        (stat, option) = runParaFoamOptionDialog.Show(isRegion)
        if stat == "OK":
            #出力をlogFileにもteeで出力させる
            paraFoam = self.configDict["paraFoam"]
            comm = paraFoam + " " + option + " &"
            pyTreeFoam.run(self.caseDir).commandWithLog(comm)

    #
    #  ending
    #  ------
    def ending(self):
        """ endingの処理"""
        if self.check_parallel.get_active() == True:
            parallel = "yes"
        else:
            parallel = "no"
        nCpu = self.entry_nCpu.get_text()
        nSeparate = self.entry_separate.get_text()
        angle = self.entry_angle.get_text()
        stlDir = self.entry_stlDir.get_text()
        dataDict = {}
        dataDict["stlDir"] = stlDir
        dataDict["surfaceAngle"] = angle
        dataDict["nCpu"] = nCpu
        dataDict["nSeparate"] = nSeparate
        dataDict["parallel"] = parallel
        pyTreeFoam.case(self.caseDir).setCreateMeshData(dataDict)

    #
    #  checkBlockMeshSnappyHexMesh
    def checkBlockMeshSnappyHexMesh(self):
        """ meshのDictファイルをチェック"""
        copyFiles = []
        #blockMeshのチェック
        fileName = self.caseDir + "/system/blockMeshDict"
        if len(glob.glob(fileName)) == 0:
            copyFiles.append("blockMeshDict")
        #snappyHexMeshDictのチェック
        fileName = self.caseDir + "/system/snappyHexMeshDict"
        if len(glob.glob(fileName)) == 0:
            copyFiles.append("snappyHexMeshDict")
        #decomposeParDictのチェック
        fileName = self.caseDir + "/system/decomposeParDict"
        if len(glob.glob(fileName)) == 0:
            copyFiles.append("decomposeParDict")
        #meshQualityDictのチェック
        OFversion = self.configDict["OFversion"]
        numVer = pyTreeFoam.getNumVersion(OFversion)
        if numVer >= 6.0 or OFversion >= "v1906":
            fileName = self.caseDir + "/system/meshQualityDict"
            if len(glob.glob(fileName)) == 0:
                copyFiles.append("meshQualityDict")
        #チェックしたファイルが存在しなかった場合、コピーする
        if len(copyFiles) != 0:
            OFversion = self.configDict["OFversion"]
            sourceDir = pyTreeFoam.getFoamContents().getOFversionFolder(OFversion)
            for name in copyFiles:
                sourceFile = sourceDir + "/system/" + name
                pasteDir = self.caseDir + "/system"
                shutil.copy(sourceFile, pasteDir)
            title = _("meshDictファイルのコピー")
            mess = _("以下のファイルが存在しなかったので、default設定のDictファイルをコピーしました。\n")
            for name in copyFiles:
                mess += name + ", "
            mess = mess[:-2]
            self.okDialog(title, mess)

    #----- universalDialog -----------
    def okDialog(self, title, mess):
        unvDlg.okDialog(self, title, mess)
    def errDialog(self, title, mess):
        unvDlg.errDialog(self, title, mess)
    def okCancelDialog(self, title, mess):
        return unvDlg.okCancelDialog(self, title, mess)
    def yesNoDialog(self, title, mess):
        return unvDlg.yesNoDialog(self, title, mess)

if __name__ == "__main__":
    import gettext
    gettext.install("treefoam", localeDir)
    #_ = gettext.gettext

    caseDir = sys.argv[1]
    modelDir = sys.argv[2]
    winApp = windowApp(caseDir, modelDir)
    winApp.main()
