#!/usr/bin/python3
# coding: utf-8
#
#   universalDialogs.py
#
#       一般的なdialogを集積したもの
#
#   19/09/20    新規作成
#

""" よく使用するokDialog等を集めたモジュール

Dialogs:
    okDialog        : 情報表示　(戻り値なし)
    errDialog       : エラー表示　(戻り値なし)
    warnimgDialog   :警告表示（戻り値なし）
    okCancelDialog  : 選択 (戻り値"OK" or "CANCEL")
    yesNoDialog     : 選択 (戻り値"YES" or "NO")
    folderDialog    : folderDir選択 (戻り値 dir or "")
    fileDialog      :file選択（戻り値 file or "")
Example:
    import univaersalDialogs as unvDlg
    unvDlg.okDialog(self, title, message)"""

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

#  okDialog
#
def okDialog(self, title, mess):
    """ okDialog表示"""
    dialog = Gtk.MessageDialog(
                message_type =  Gtk.MessageType.INFO,
                #   INFO, WARNING, QUESTION, ERROR, OTHER
                buttons = Gtk.ButtonsType.OK,
                #   NONE, OK, CLOSE, CANCEL, YES_NO, OK_CANCEL
                text = title)
    dialog.format_secondary_text(mess)
    #OKボタンにfocusを当てる
    dialog.set_default_response(Gtk.ResponseType.OK)
    dialog.run()
    dialog.destroy()

#
# warningDialog
def warningDialog(self, title, mess):
    """ 警告dialog表示"""
    dialog = Gtk.MessageDialog(
                message_type =  Gtk.MessageType.WARNING,
                #   INFO, WARNING, QUESTION, ERROR, OTHER
                buttons = Gtk.ButtonsType.OK,
                #   NONE, OK, CLOSE, CANCEL, YES_NO, OK_CANCEL
                text = title)
    dialog.format_secondary_text(mess)
    #OKボタンにfocusを当てる
    dialog.set_default_response(Gtk.ResponseType.OK)
    dialog.run()
    dialog.destroy()

#  okCancelDialog
#
#def okCancelDialog(self, title, mess):
def okCancelDialog(parent, title, mess):
    """ okCancel dialog表示"""
    dialog = Gtk.MessageDialog(
                message_type =  Gtk.MessageType.WARNING,
                #   INFO, WARNING, QUESTION, ERROR, OTHER
                buttons = Gtk.ButtonsType.OK_CANCEL,
                #   NONE, OK, CLOSE, CANCEL, YES_NO, OK_CANCEL
                text = title)
    dialog.format_secondary_text(mess)
    dialog.set_default_response(Gtk.ResponseType.OK)
    res = dialog.run()
    dialog.destroy()
    if res == Gtk.ResponseType.OK:
        stat = "OK"
    else:
        stat = "CANCEL"
    return stat

#  yesNoDialog
#
def yesNoDialog(self, title, mess):
    """ yesNo dialog表示"""
    dialog = Gtk.MessageDialog(
                message_type =  Gtk.MessageType.WARNING,
                #   INFO, WARNING, QUESTION, ERROR, OTHER
                buttons = Gtk.ButtonsType.YES_NO,
                #   NONE, OK, CLOSE, CANCEL, YES_NO, OK_CANCEL
                text = title)
    dialog.format_secondary_text(mess)
    dialog.set_default_response(Gtk.ResponseType.YES)
    res = dialog.run()
    dialog.destroy()
    if res == Gtk.ResponseType.YES:
        stat = "YES"
    else:
        stat = "NO"
    return stat

#  errDialog
#
def errDialog(self, title, mess):
    """ errDialog表示"""
    dialog = Gtk.MessageDialog(
                message_type =  Gtk.MessageType.ERROR,
                #   INFO, WARNING, QUESTION, ERROR, OTHER
                buttons = Gtk.ButtonsType.OK,
                #   NONE, OK, CLOSE, CANCEL, YES_NO, OK_CANCEL
                text = title)
    dialog.format_secondary_text(mess)
    dialog.set_default_response(Gtk.ResponseType.OK)
    dialog.run()
    dialog.destroy()

# folderDialog
#
def folderDialog(self, title, path):
    """ folderの選択"""
    dlg = Gtk.FileChooserDialog(
                title = title,
                #parent = mainWindow,
                action = Gtk.FileChooserAction.SELECT_FOLDER)
    dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                    Gtk.STOCK_OPEN, Gtk.ResponseType.OK)
    dlg.set_current_folder(path)
    dlg.set_default_response(Gtk.ResponseType.OK)
    filter = Gtk.FileFilter()
    filter.add_pattern("*")
    dlg.add_filter(filter)
    result = dlg.run()
    if result == Gtk.ResponseType.OK:
        folderDir = dlg.get_filename()
    else:
        folderDir = ""
    dlg.destroy()
    return folderDir

# fileDialog
#
def fileDialog(self, title, path, pattern=""):
    """ fileの選択"""
    dlg = Gtk.FileChooserDialog(
                title = title,
                #parent = mainWindow,
                action = Gtk.FileChooserAction.OPEN)
    dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                    Gtk.STOCK_OPEN, Gtk.ResponseType.OK)
    dlg.set_current_folder(path)
    #focusを設定
    dlg.set_default_response(Gtk.ResponseType.OK)
    if pattern != "":
        filterUsr = Gtk.FileFilter()
        filterUsr.set_name(pattern)
        filterUsr.add_pattern(pattern)
        dlg.add_filter(filterUsr)
    filterAny = Gtk.FileFilter()
    filterAny.set_name("Any files")
    filterAny.add_pattern("*")
    dlg.add_filter(filterAny)
    result = dlg.run()
    if result == Gtk.ResponseType.OK:
        folderDir = dlg.get_filename()
    else:
        folderDir = ""
    dlg.destroy()
    return folderDir
